@@ -1,8 +1,8 @@
 from __future__ import generators
-# @(#)root/pyroot:$Id: ROOT.py 41092 2011-10-03 07:30:06Z pcanal $
+# @(#)root/pyroot:$Id$
 # Author: Wim Lavrijsen (WLavrijsen@lbl.gov)
 # Created: 02/20/03
-# Last: 11/10/10
+# Last: 02/06/13
 
 """PyROOT user module.
 
@@ -15,7 +15,7 @@ from __future__ import generators
 
 """
 
-__version__ = '6.1.0'
+__version__ = '6.2.0'
 __author__  = 'Wim Lavrijsen (WLavrijsen@lbl.gov)'
 
 
@@ -52,11 +52,26 @@ try:
 
          return matches
 
+      def root_global_matches( self, text, prefix = '' ):
+         gClassTable = _root.GetRootGlobal( 'gClassTable' )
+         all = [ gClassTable.At(i) for i in xrange(gClassTable.Classes()) ]
+         all += [ g.GetName() for g in _root.gROOT.GetListOfGlobals() ]
+         matches = filter( lambda x: x[:len(text)] == text, all )
+         return [prefix + x for x in matches]
+
       def global_matches( self, text ):
          matches = rlcompleter.Completer.global_matches( self, text )
-         if not matches:
-            matches = []
-         return matches + self.file_matches( text )
+         if not matches: matches = []
+         matches += self.file_matches( text )
+         return matches
+
+      def attr_matches( self, text ):
+         matches = rlcompleter.Completer.attr_matches( self, text )
+         if not matches: matches = []
+         b = text.find('.')
+         if 0 <= b and self.namespace[text[:b]].__name__ == 'ROOT':
+            matches += self.root_global_matches( text[b+1:], text[:b+1] )
+         return matches
 
    readline.set_completer( FileNameCompleter().complete )
    readline.set_completer_delims(
@@ -210,6 +225,7 @@ class std( object ):
    string = _root.MakeRootClass( 'string' )
 
 _root.std = std
+sys.modules['ROOT.std'] = std
 
 
 ### special cases for gPad, gVirtualX (are C++ macro's) -------------------------
@@ -291,7 +307,7 @@ TPad or TPad.cd, etc.
       value.msg == "can't assign to function call":
          sys.stdout.write( """Are you trying to assign a value to a reference return, for example to the
 result of a call to "double& SMatrix<>::operator()(int,int)"? If so, then
-please use operator[] instead, as in e.g. "mymatrix[i,j] = somevalue".
+please use operator[] instead, as in e.g. "mymatrix[i][j] = somevalue".
 """ )
 
  # normal exception processing
@@ -402,7 +418,7 @@ class ModuleFacade( types.ModuleType ):
       return super( self.__class__, self ).__setattr__( name, value )
 
    def __getattr1( self, name ):             # "start-up" getattr
-    # special case, to allow "from ROOT import gROOT" w/o starting GUI thread
+    # special case, to allow "from ROOT import gROOT" w/o sending GUI events
       if name == '__path__':
          raise AttributeError( name )
 
@@ -416,6 +432,11 @@ class ModuleFacade( types.ModuleType ):
    def __getattr2( self, name ):             # "running" getattr
     # handle "from ROOT import *" ... can be called multiple times
       if name == '__all__':
+         if '__IPYTHON__' in __builtins__:
+            import warnings
+            warnings.warn( '"from ROOT import *" is not supported under IPython' )
+            # continue anyway, just in case it works ...
+
          caller = sys.modules[ sys._getframe( 1 ).f_globals[ '__name__' ] ]
 
        # we may be calling in from __getattr1, verify and if so, go one frame up
@@ -485,7 +506,7 @@ class ModuleFacade( types.ModuleType ):
     # must be called after gApplication creation:
       if '__IPYTHON__' in __builtins__:
        # IPython's FakeModule hack otherwise prevents usage of python from CINT
-         _root.gROOT.ProcessLine( 'TPython::Exec( "" )' )
+         _root.gROOT.ProcessLine( 'TPython::Exec( "" );' )
          sys.modules[ '__main__' ].__builtins__ = __builtins__
 
     # custom logon file (must be after creation of ROOT globals)
@@ -510,23 +531,33 @@ class ModuleFacade( types.ModuleType ):
                   appc.ExecuteFile( rootlogon )
             del rootlogon, logons
 
-    # root thread, if needed, to prevent GUIs from starving, as needed
+    # use either the input hook or thread to send events to GUIs
       if self.PyConfig.StartGuiThread and \
             not ( self.keeppolling or _root.gROOT.IsBatch() ):
-         import threading
-         self.__dict__[ 'keeppolling' ] = 1
-         self.__dict__[ 'PyGUIThread' ] = \
-            threading.Thread( None, _processRootEvents, None, ( self, ) )
-
-         def _finishSchedule( ROOT = self ):
+         if self.PyConfig.StartGuiThread == 'inputhook' or\
+               _root.gSystem.InheritsFrom( 'TMacOSXSystem' ):
+          # new, PyOS_InputHook based mechanism
+            if PyConfig.GUIThreadScheduleOnce:
+               for guicall in PyConfig.GUIThreadScheduleOnce:
+                  guicall()
+               PyConfig.GUIThreadScheduleOnce = []
+            _root.InstallGUIEventInputHook()
+         else:
+          # original, threading based approach
             import threading
-            if threading.currentThread() != self.PyGUIThread:
-               while self.PyConfig.GUIThreadScheduleOnce:
-                  self.PyGUIThread.join( 0.1 )
+            self.__dict__[ 'keeppolling' ] = 1
+            self.__dict__[ 'PyGUIThread' ] = \
+               threading.Thread( None, _processRootEvents, None, ( self, ) )
 
-         self.PyGUIThread.finishSchedule = _finishSchedule
-         self.PyGUIThread.setDaemon( 1 )
-         self.PyGUIThread.start()
+            def _finishSchedule( ROOT = self ):
+               import threading
+               if threading.currentThread() != self.PyGUIThread:
+                  while self.PyConfig.GUIThreadScheduleOnce:
+                     self.PyGUIThread.join( 0.1 )
+
+            self.PyGUIThread.finishSchedule = _finishSchedule
+            self.PyGUIThread.setDaemon( 1 )
+            self.PyGUIThread.start()
 
     # store already available ROOT objects to prevent spurious lookups
       for name in self.module.__pseudo__all__ + _memPolicyAPI + _sigPolicyAPI:
@@ -554,7 +585,13 @@ def cleanup():
 
    facade = sys.modules[ __name__ ]
 
- # prevent spurious lookups into ROOT libraries
+ # reset gRootModule on the C++ side to prevent fruther lookups
+   _root._ResetRootModule()
+
+ # shutdown GUI thread, as appropriate (always save to call)
+   _root.RemoveGUIEventInputHook()
+
+ # prevent further spurious lookups into ROOT libraries
    del facade.__class__.__getattr__
    del facade.__class__.__setattr__
 
