@@ -1,4 +1,4 @@
-// @(#)root/pyroot:$Id: Converters.cxx 43677 2012-04-10 15:50:45Z pcanal $
+// @(#)root/pyroot:$Id$
 // Author: Wim Lavrijsen, Jan 2005
 
 // Bindings
@@ -174,7 +174,7 @@ Bool_t PyROOT::TLongRefConverter::SetArg(
    return kTRUE;
 #else
    para.fLong = 0; func = 0;
-   return kFALSE; // there no longer is a PyIntObject in p3
+   return (Bool_t)func; // there no longer is a PyIntObject in p3
 #endif
 }
 
@@ -571,6 +571,15 @@ Bool_t PyROOT::TNonConstCStringConverter::SetArg(
    return CArraySetArg( pyobject, para, func, 'c', sizeof(char) );
 }
 
+//____________________________________________________________________________
+PyObject* PyROOT::TNonConstCStringConverter::FromMemory( void* address )
+{
+// assume this is a buffer access if the size is known; otherwise assume string
+   if ( fMaxSize != UINT_MAX )
+      return PyROOT_PyUnicode_FromStringAndSize( *(char**)address, fMaxSize );
+   return this->TCStringConverter::FromMemory( address );
+}
+
 //____________________________________________________________________________
 Bool_t PyROOT::TNonConstUCStringConverter::SetArg(
       PyObject* pyobject, TParameter_t& para, G__CallFunc* func, Long_t )
@@ -747,8 +756,8 @@ Bool_t PyROOT::TLongLongArrayConverter::SetArg(
 
 //- converters for special cases ----------------------------------------------
 #define PYROOT_IMPLEMENT_STRING_AS_PRIMITIVE_CONVERTER( name, type, F1, F2 )  \
-PyROOT::T##name##Converter::T##name##Converter() :                            \
-      TRootObjectConverter( TClass::GetClass( #type ) ) {}                    \
+PyROOT::T##name##Converter::T##name##Converter( Bool_t keepControl ) :        \
+      TRootObjectConverter( TClass::GetClass( #type ), keepControl ) {}       \
                                                                               \
 Bool_t PyROOT::T##name##Converter::SetArg(                                    \
       PyObject* pyobject, TParameter_t& para, G__CallFunc* func, Long_t user )\
@@ -1074,9 +1083,9 @@ PyROOT::TConverter* PyROOT::CreateConverter( const std::string& fullType, Long_t
 
    } else if ( ti.Property() & G__BIT_ISENUM ) {
    // special case (CINT): represent enums as unsigned integers
-      if ( cpd == "&" )
-         h = gConvFactories.find( "long&" );
-      else
+      if ( cpd == "&" ) {
+         h = isConst ? gConvFactories.find( "const long&" ) : gConvFactories.find( "long&" );
+      } else
          h = gConvFactories.find( "UInt_t" );
    }
 
@@ -1203,7 +1212,7 @@ namespace {
       NFp_t( "const char*",        &CreateCStringConverter            ),
       NFp_t( "char*",              &CreateNonConstCStringConverter    ),
       NFp_t( "TString",            &CreateTStringConverter            ),
-      NFp_t( "TString&",           &CreateTStringConverter            ),
+      NFp_t( "const TString&",     &CreateTStringConverter            ),
       NFp_t( "std::string",        &CreateSTLStringConverter          ),
       NFp_t( "string",             &CreateSTLStringConverter          ),
       NFp_t( "const std::string&", &CreateSTLStringConverter          ),
