@@ -1,4 +1,4 @@
-// @(#)root/pyroot:$Id: MethodProxy.cxx 43997 2012-04-27 18:09:46Z wlav $
+// @(#)root/pyroot:$Id$
 // Author: Wim Lavrijsen, Jan 2005
 
 // Bindings
@@ -95,7 +95,7 @@ namespace {
       }
 
    // if this new object falls inside self, make sure its lifetime is proper
-      if ( pymeth->fSelf && ObjectProxy_Check( result ) ) {
+      if ( ObjectProxy_Check( pymeth->fSelf ) && ObjectProxy_Check( result ) ) {
          Long_t ptrdiff = (Long_t)((ObjectProxy*)result)->GetObject() - (Long_t)pymeth->fSelf->GetObject();
          if ( 0 <= ptrdiff && ptrdiff < (Long_t)pymeth->fSelf->ObjectIsA()->Size() ) {
             if ( PyObject_SetAttr( result, PyStrings::gLifeLine, (PyObject*)pymeth->fSelf ) == -1 )
@@ -290,7 +290,7 @@ namespace {
 #else 	 
 // not important for functioning of most code, so not implemented for p3 for now (TODO) 	 
       pymeth = 0;
-      Py_INCREF( Py_None );
+      if ( pymeth || !pymeth) Py_INCREF( Py_None );
       return Py_None;
 #endif
    }
@@ -478,14 +478,8 @@ namespace {
 
    // simple case
       if ( nMethods == 1 ) {
-         PyObject* result = 0;
-         if ( pymeth->fMethodInfo->fFlags & MethodProxy::MethodInfo_t::kReleaseGIL ){
-            Py_BEGIN_ALLOW_THREADS
-            result = (*methods[0])( pymeth->fSelf, args, kwds, user );
-            Py_END_ALLOW_THREADS
-         } else
-            result = (*methods[0])( pymeth->fSelf, args, kwds, user );
-
+         PyObject* result = (*methods[0])( pymeth->fSelf, args, kwds, user,
+            (pymeth->fMethodInfo->fFlags & MethodProxy::MethodInfo_t::kReleaseGIL) );
          return HandleReturn( pymeth, result );
       }
 
