@@ -1,4 +1,4 @@
-// @(#)root/pyroot:$Id: Pythonize.cxx 43515 2012-03-27 21:15:53Z pcanal $
+// @(#)root/pyroot:$Id$
 // Author: Wim Lavrijsen, Jul 2004
 
 // Bindings
@@ -1281,7 +1281,8 @@ namespace PyROOT {      // workaround for Intel icc on Linux
    public:
       virtual PyCallable* Clone() { return new TTreeBranch( *this ); }
 
-      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject* kwds, Long_t )
+      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject* kwds,
+                                    Long_t, Bool_t /* release_gil */ )
       {
       // acceptable signatures:
       //   ( const char*, void*, const char*, Int_t = 32000 )
@@ -1405,7 +1406,8 @@ namespace PyROOT {      // workaround for Intel icc on Linux
 
       virtual PyCallable* Clone() { return new TTreeSetBranchAddress( *this ); }
 
-      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject* kwds, Long_t )
+      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject* kwds,
+                                    Long_t, Bool_t /* release_gil */ )
       {
       // acceptable signature:
       //   ( const char*, void* )
@@ -1635,7 +1637,8 @@ namespace {
 
       virtual PyCallable* Clone() { return new TF1InitWithPyFunc( *this ); }
 
-      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*, Long_t )
+      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*,
+                                    Long_t, Bool_t /* release_gil */ )
       {
       // expected signature: ( char* name, pyfunc, double xmin, double xmax, int npar = 0 )
          int argc = PyTuple_GET_SIZE( args );
@@ -1753,7 +1756,8 @@ namespace {
 
       virtual PyCallable* Clone() { return new TMinuitSetFCN( *this ); }
 
-      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*, Long_t )
+      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*,
+                                    Long_t, Bool_t /* release_gil */ )
       {
       // expected signature: ( pyfunc )
          int argc = PyTuple_GET_SIZE( args );
@@ -1811,7 +1815,8 @@ namespace {
 
       virtual PyCallable* Clone() { return new TMinuitFitterSetFCN( *this ); }
 
-      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*, Long_t )
+      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*,
+                                    Long_t, Bool_t release_gil )
       {
       // expected signature: ( pyfunc )
          int argc = PyTuple_GET_SIZE( args );
@@ -1822,7 +1827,7 @@ namespace {
             return 0;              // reported as an overload failure
          }
 
-         return TMinuitSetFCN::operator()( self, args, 0, 0 );
+         return TMinuitSetFCN::operator()( self, args, 0, 0, release_gil );
       }
    };
 
@@ -1880,7 +1885,8 @@ namespace {
 
       virtual PyCallable* Clone() { return new TFitterFitFCN( *this ); }
 
-      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*, Long_t )
+      virtual PyObject* operator()( ObjectProxy* self, PyObject* args, PyObject*,
+                                    Long_t, Bool_t /* release_gil */ )
       {
       // expected signature: ( self, pyfunc, int npar = 0, const double* params = 0, unsigned int dataSize = 0, bool chi2fit = false )
          int argc = PyTuple_GET_SIZE( args );
@@ -1924,6 +1930,25 @@ namespace {
    };
 
 
+//- TFile::Get -----------------------------------------------------------------
+   PyObject* TFileGetAttr( PyObject* self, PyObject* attr )
+   {
+   // Pythonization of TFile::Get that raises AttributeError on failure.
+      PyObject* result = CallPyObjMethod( self, "Get", attr );
+      if ( !result )
+         return result;
+
+      if ( !PyObject_IsTrue( result ) ) {
+         PyObject* astr = PyObject_Str( attr );
+         PyErr_Format( PyExc_AttributeError, "TFile object has no attribute \'%s\'",
+                       PyROOT_PyUnicode_AsString( astr ) );
+         Py_DECREF( astr );
+         Py_DECREF( result );
+         return 0;
+      }
+      return result;
+   }
+
 //- simplistic len() functions -------------------------------------------------
    PyObject* ReturnThree( ObjectProxy*, PyObject* ) {
       return PyInt_FromLong( 3 );
@@ -1957,6 +1982,13 @@ Bool_t PyROOT::Pythonize( PyObject* pyclass, const std::string& name )
    if ( HasAttrDirect( pyclass, PyStrings::gSize ) )
       Utility::AddToClass( pyclass, "__len__", "size" );
 
+// like-wise, some typical container sizings
+   if ( HasAttrDirect( pyclass, PyStrings::gGetSize ) )
+      Utility::AddToClass( pyclass, "__len__", "GetSize" );
+
+   if ( HasAttrDirect( pyclass, PyStrings::ggetSize ) )
+      Utility::AddToClass( pyclass, "__len__", "getSize" );
+
    if ( HasAttrDirect( pyclass, PyStrings::gBegin ) && HasAttrDirect( pyclass, PyStrings::gEnd ) ) {
    // some classes may not have dicts for their iterators, making begin/end useless
       PyObject* pyfullname = PyObject_GetAttr( pyclass, PyStrings::gName );
@@ -2038,7 +2070,6 @@ Bool_t PyROOT::Pythonize( PyObject* pyclass, const std::string& name )
 
       Utility::AddToClass( pyclass, "count", (PyCFunction) TCollectionCount, METH_O );
 
-      Utility::AddToClass( pyclass, "__len__",  "GetSize" );
       ((PyTypeObject*)pyclass)->tp_iter = (getiterfunc)TCollectionIter;
       Utility::AddToClass( pyclass, "__iter__",  (PyCFunction)TCollectionIter, METH_NOARGS );
 
@@ -2244,7 +2275,7 @@ Bool_t PyROOT::Pythonize( PyObject* pyclass, const std::string& name )
       return Utility::AddToClass( pyclass, "FitFCN", new TFitterFitFCN );
 
    if ( name == "TFile" )     // allow member-style access to entries in file
-      return Utility::AddToClass( pyclass, "__getattr__", "Get" );
+      return Utility::AddToClass( pyclass, "__getattr__", TFileGetAttr, METH_O );
 
    if ( name.substr(0,8) == "TVector3" ) {
       Utility::AddToClass( pyclass, "__len__", (PyCFunction) ReturnThree, METH_NOARGS );
