@@ -1,4 +1,4 @@
-// @(#)root/cintex:$Id: ROOTClassEnhancer.cxx 35513 2010-09-21 08:53:07Z axel $
+// @(#)root/cintex:$Id$
 // Author: Pere Mato 2005
 
 // Copyright CERN, CH-1211 Geneva 23, 2004-2005, All rights reserved.
@@ -442,7 +442,13 @@ namespace ROOT { namespace Cintex {
                                                    m->destruct_func,
                                                    m->feed_func,
                                                    m->collect_func,
-                                                   m->create_env);
+                                                   m->create_env,
+                                                   m->fCreateIterators,
+                                                   m->fCopyIterator,
+                                                   m->fNext,
+                                                   m->fDeleteSingleIterator,
+                                                   m->fDeleteTwoIterators
+                                                   );
                root_class->SetCollectionProxy(cpinfo);
 
                root_class->SetBit(TClass::kIsForeign);
