@@ -1,4 +1,4 @@
-// @(#)root/base:$Id: TStyle.h 38506 2011-03-18 14:08:49Z couet $
+// @(#)root/base:$Id$
 // Author: Rene Brun   12/12/94
 
 /*************************************************************************
@@ -401,8 +401,8 @@ public:
    void             ToggleEditor() { fShowEditor = fShowEditor ? 0 : 1; }
    void             ToggleToolBar() { fShowToolBar = fShowToolBar ? 0 : 1; }
    void             SetIsReading(Bool_t reading=kTRUE);
-   void             SetPalette(Int_t ncolors=0, Int_t *colors=0);
-   void             SavePrimitive(ostream &out, Option_t * = "");
+   void             SetPalette(Int_t ncolors=0, Int_t *colors=0, Float_t alpha=1.);
+   void             SavePrimitive(std::ostream &out, Option_t * = "");
    void             SaveSource(const char *filename, Option_t *option=0);
 
    ClassDef(TStyle, 14);  //A collection of all graphics attributes
