@@ -1,4 +1,4 @@
-// @(#)root/cont:$Id: TClonesArray.cxx 42332 2011-11-30 18:45:32Z axel $
+// @(#)root/cont:$Id$
 // Author: Rene Brun   11/02/96
 
 /*************************************************************************
@@ -434,18 +434,28 @@ void TClonesArray::Delete(Option_t *)
    // memory (e.g. objects inheriting from TNamed or containing TStrings
    // allocate memory). If not you better use Clear() since if is faster.
 
-   Long_t dtoronly = TObject::GetDtorOnly();
-   for (Int_t i = 0; i < fSize; i++) {
-      if (fCont[i] && fCont[i]->TestBit(kNotDeleted)) {
-         // Tell custom operator delete() not to delete space when
-         // object fCont[i] is deleted. Only destructors are called
-         // for this object.
-         TObject::SetDtorOnly(fCont[i]);
-         delete fCont[i];
+   if ( fClass->TestBit(TClass::kIsEmulation) ) {
+      // In case of emulated class, we can not use the delete operator
+      // directly, it would use the wrong destructor.
+      for (Int_t i = 0; i < fSize; i++) {
+         if (fCont[i] && fCont[i]->TestBit(kNotDeleted)) {
+            fClass->Destructor(fCont[i],kTRUE);
+         }
       }
+   } else {
+      Long_t dtoronly = TObject::GetDtorOnly();
+      for (Int_t i = 0; i < fSize; i++) {
+         if (fCont[i] && fCont[i]->TestBit(kNotDeleted)) {
+            // Tell custom operator delete() not to delete space when
+            // object fCont[i] is deleted. Only destructors are called
+            // for this object.
+            TObject::SetDtorOnly(fCont[i]);
+            delete fCont[i];
+         }
+      }
+      // Restore the state.
+      TObject::SetDtorOnly((void*)dtoronly);
    }
-   // Restore the state.
-   TObject::SetDtorOnly((void*)dtoronly);
 
    // Protect against erroneously setting of owner bit.
    SetOwner(kFALSE);
@@ -918,13 +928,8 @@ TObject *&TClonesArray::operator[](Int_t idx)
       //    obj->TestBit(TObject::kNotDeleted)
       // will behave correctly.
       // TObject::kNotDeleted is one of the higher bit that is not settable via the public
-      // interface.
-      static size_t fbitsOffset = 0;
-      if (fbitsOffset == 0) {
-         fbitsOffset = TObject::Class()->GetDataMemberOffset("fBits");
-      }
-      char *fbitsAddress = ((char*)fKeep->fCont[idx]) + fbitsOffset;
-      *(UInt_t*)fbitsAddress = 0;
+      // interface. But luckily we are its friend.
+      fKeep->fCont[idx]->fBits &= ~kNotDeleted;
    }
    fCont[idx] = fKeep->fCont[idx];
 
@@ -995,7 +1000,7 @@ void TClonesArray::AbsorbObjects(TClonesArray *tc)
       Expand(newSize);
 
    // move
-   for (Int_t i = 0; i <= tc->GetEntriesFast(); i++) {
+   for (Int_t i = 0; i < tc->GetEntriesFast(); ++i) {
       fCont[oldSize+i] = tc->fCont[i];
       (*fKeep)[oldSize+i] = (*(tc->fKeep))[i];
       tc->fCont[i] = 0;
@@ -1012,7 +1017,7 @@ void TClonesArray::AbsorbObjects(TClonesArray *tc)
 //______________________________________________________________________________
 void TClonesArray::AbsorbObjects(TClonesArray *tc, Int_t idx1, Int_t idx2)
 {
-   // Directly move the rang of object pointers from tc without cloning
+   // Directly move the range of object pointers from tc without cloning
    // (copying).
    // This TClonesArray takes over ownership of all of tc's object pointers
    // from idx1 to idx2. The tc array is re-arranged by return.
@@ -1041,8 +1046,10 @@ void TClonesArray::AbsorbObjects(TClonesArray *tc, Int_t idx1, Int_t idx2)
 
    // move
    for (Int_t i = idx1; i <= idx2; i++) {
-      fCont[oldSize+i -idx1] = tc->fCont[i];
-      (*fKeep)[oldSize+i-idx1] = (*(tc->fKeep))[i];
+      Int_t newindex = oldSize+i -idx1; 
+      fCont[newindex] = tc->fCont[i];
+      ::operator delete(fKeep->fCont[newindex]);
+      (*fKeep)[newindex] = (*(tc->fKeep))[i];
       tc->fCont[i] = 0;
       (*(tc->fKeep))[i] = 0;
    }
