@@ -1,4 +1,4 @@
-// @(#)root/meta:$Id: TCint.h 43515 2012-03-27 21:15:53Z pcanal $
+// @(#)root/meta:$Id$
 // Author: Fons Rademakers   01/03/96
 
 /*************************************************************************
@@ -108,6 +108,7 @@ public:
    Long_t  ProcessLineAsynch(const char *line, EErrorCode *error = 0);
    Long_t  ProcessLineSynch(const char *line, EErrorCode *error = 0);
    void    PrintIntro();
+   Int_t   SetClassSharedLibs(const char *cls, const char *libs);
    void    SetGetline(const char*(*getlineFunc)(const char* prompt),
                       void (*histaddFunc)(const char* line));
    void    Reset();
@@ -270,6 +271,7 @@ public:
    virtual void   MethodInfo_CreateSignature(MethodInfo_t *minfo, TString &signature) const;
    virtual void   MethodInfo_Delete(MethodInfo_t *minfo) const;
    virtual MethodInfo_t  *MethodInfo_Factory() const;
+   virtual MethodInfo_t  *MethodInfo_Factory(ClassInfo_t * /*clinfo*/) const;
    virtual MethodInfo_t  *MethodInfo_FactoryCopy(MethodInfo_t *minfo) const;
    virtual MethodInfo_t  *MethodInfo_InterfaceMethod(MethodInfo_t *minfo) const;
    virtual Bool_t MethodInfo_IsValid(MethodInfo_t *minfo) const;
@@ -282,11 +284,13 @@ public:
    virtual const char *MethodInfo_GetPrototype(MethodInfo_t *minfo) const;
    virtual const char *MethodInfo_Name(MethodInfo_t *minfo) const;
    virtual const char *MethodInfo_TypeName(MethodInfo_t *minfo) const;
+   virtual std::string MethodInfo_TypeNormalizedName(MethodInfo_t *minfo) const;
    virtual const char *MethodInfo_Title(MethodInfo_t *minfo) const;
 
    // G__MethodArgInfo interface
    virtual void   MethodArgInfo_Delete(MethodArgInfo_t *marginfo) const;
    virtual MethodArgInfo_t  *MethodArgInfo_Factory() const;
+   virtual MethodArgInfo_t  *MethodArgInfo_Factory(MethodInfo_t *minfo) const;
    virtual MethodArgInfo_t  *MethodArgInfo_FactoryCopy(MethodArgInfo_t *marginfo) const;
    virtual Bool_t MethodArgInfo_IsValid(MethodArgInfo_t *marginfo) const;
    virtual int    MethodArgInfo_Next(MethodArgInfo_t *marginfo) const;
@@ -294,7 +298,7 @@ public:
    virtual const char *MethodArgInfo_DefaultValue(MethodArgInfo_t *marginfo) const;
    virtual const char *MethodArgInfo_Name(MethodArgInfo_t *marginfo) const;
    virtual const char *MethodArgInfo_TypeName(MethodArgInfo_t *marginfo) const;
-
+   virtual std::string MethodArgInfo_TypeNormalizedName(MethodArgInfo_t *marginfo) const;
 
    // G__TypeInfo interface
    virtual void   TypeInfo_Delete(TypeInfo_t *tinfo) const;
@@ -316,6 +320,7 @@ public:
    virtual TypedefInfo_t  *TypedefInfo_FactoryCopy(TypedefInfo_t *tinfo) const;
    virtual void   TypedefInfo_Init(TypedefInfo_t *tinfo, const char *funcname) const;
    virtual Bool_t TypedefInfo_IsValid(TypedefInfo_t *tinfo) const;
+   virtual int    TypedefInfo_Next(TypedefInfo_t *tinfo) const;
    virtual Long_t TypedefInfo_Property(TypedefInfo_t *tinfo) const;
    virtual int    TypedefInfo_Size(TypedefInfo_t *tinfo) const;
    virtual const char *TypedefInfo_TrueName(TypedefInfo_t *tinfo) const;
