@@ -1,4 +1,4 @@
-// @(#)root/geom:$Id: TGeoManager.h 43895 2012-04-23 07:03:47Z agheata $
+// @(#)root/geom:$Id$
 // Author: Andrei Gheata   25/10/01
 
 /*************************************************************************
@@ -239,7 +239,7 @@ public:
    void                   DrawPath(const char *path);
    void                   PrintOverlaps() const; // *MENU*
    void                   RandomPoints(const TGeoVolume *vol, Int_t npoints=10000, Option_t *option="");
-   void                   RandomRays(Int_t nrays=1000, Double_t startx=0, Double_t starty=0, Double_t startz=0);
+   void                   RandomRays(Int_t nrays=1000, Double_t startx=0, Double_t starty=0, Double_t startz=0, const char *target_vol=0, Bool_t check_norm=kFALSE);
    TGeoNode              *SamplePoints(Int_t npoints, Double_t &dist, Double_t epsil=1E-5,
                                        const char *g3path="");
    void                   SetNmeshPoints(Int_t npoints=1000);
