@@ -1,4 +1,4 @@
-// @(#)root/geom:$Id: TGeoMatrix.h 40116 2011-07-04 12:00:32Z agheata $
+// @(#)root/geom:$Id$
 // Author: Andrei Gheata   25/10/01
 
 /*************************************************************************
@@ -42,6 +42,7 @@ class TGeoMatrix : public TNamed
 public:
 enum EGeoTransfTypes {
    kGeoIdentity  = 0,
+   kGeoShared       = BIT(14),
    kGeoTranslation  = BIT(17),
    kGeoRotation     = BIT(18),
    kGeoScale        = BIT(19),
@@ -78,6 +79,7 @@ public :
    Bool_t               IsRotation()    const {return TestBit(kGeoRotation);}
    Bool_t               IsReflection()  const {return TestBit(kGeoReflection);}
    Bool_t               IsScale()       const {return TestBit(kGeoScale);}
+   Bool_t               IsShared()      const {return TestBit(kGeoShared);}
    Bool_t               IsCombi()       const {return (TestBit(kGeoTranslation) 
                                                && TestBit(kGeoRotation));}
    Bool_t               IsGeneral()     const {return (TestBit(kGeoTranslation) 
@@ -112,6 +114,7 @@ public :
    virtual void         SetDx(Double_t) {}
    virtual void         SetDy(Double_t) {}
    virtual void         SetDz(Double_t) {}
+   void                 SetShared(Bool_t flag=kTRUE) {SetBit(kGeoShared, flag);}
    
    ClassDef(TGeoMatrix, 1)                 // base geometrical transformation class
 };
@@ -154,7 +157,7 @@ public :
    virtual void         RotateX(Double_t angle);
    virtual void         RotateY(Double_t angle);
    virtual void         RotateZ(Double_t angle);
-   virtual void         SavePrimitive(ostream &out, Option_t *option = "");
+   virtual void         SavePrimitive(std::ostream &out, Option_t *option = "");
    void                 Subtract(const TGeoTranslation *other);
    void                 SetTranslation(Double_t dx, Double_t dy, Double_t dz);
    void                 SetTranslation(const TGeoMatrix &other);
@@ -216,7 +219,7 @@ public :
    virtual void         RotateX(Double_t angle);
    virtual void         RotateY(Double_t angle);
    virtual void         RotateZ(Double_t angle);
-   virtual void         SavePrimitive(ostream &out, Option_t *option = "");
+   virtual void         SavePrimitive(std::ostream &out, Option_t *option = "");
    virtual void         ReflectX(Bool_t leftside, Bool_t rotonly=kFALSE);
    virtual void         ReflectY(Bool_t leftside, Bool_t rotonly=kFALSE);
    virtual void         ReflectZ(Bool_t leftside, Bool_t rotonly=kFALSE);
@@ -309,7 +312,7 @@ public :
    virtual void         ReflectX(Bool_t leftside, Bool_t rotonly=kFALSE);
    virtual void         ReflectY(Bool_t leftside, Bool_t rotonly=kFALSE);
    virtual void         ReflectZ(Bool_t leftside, Bool_t rotonly=kFALSE);
-   virtual void         SavePrimitive(ostream &out, Option_t *option = "");
+   virtual void         SavePrimitive(std::ostream &out, Option_t *option = "");
    virtual void         SetDx(Double_t dx) {SetTranslation(dx, fTranslation[1], fTranslation[2]);}
    virtual void         SetDy(Double_t dy) {SetTranslation(fTranslation[0], dy, fTranslation[2]);}
    virtual void         SetDz(Double_t dz) {SetTranslation(fTranslation[0], fTranslation[1], dz);}
@@ -389,7 +392,7 @@ public :
    virtual const Double_t    *GetTranslation() const {return &kNullVector[0];}
    virtual const Double_t    *GetRotationMatrix() const {return &kIdentityMatrix[0];}
    virtual const Double_t    *GetScale()       const {return &kUnitScale[0];}
-   virtual void         SavePrimitive(ostream &, Option_t * = "") {;}
+   virtual void         SavePrimitive(std::ostream &, Option_t * = "") {;}
 
    ClassDef(TGeoIdentity, 1)                 // identity transformation class
 };
@@ -439,7 +442,7 @@ public :
    virtual void         ReflectX(Bool_t leftside, Bool_t rotonly=kFALSE);
    virtual void         ReflectY(Bool_t leftside, Bool_t rotonly=kFALSE);
    virtual void         ReflectZ(Bool_t leftside, Bool_t rotonly=kFALSE);
-   virtual void         SavePrimitive(ostream &out, Option_t *option = "");
+   virtual void         SavePrimitive(std::ostream &out, Option_t *option = "");
    virtual void         SetDx(Double_t dx) {fTranslation[0] = dx; SetBit(kGeoTranslation);}
    virtual void         SetDy(Double_t dy) {fTranslation[1] = dy; SetBit(kGeoTranslation);}
    virtual void         SetDz(Double_t dz) {fTranslation[2] = dz; SetBit(kGeoTranslation);}
