@@ -1,4 +1,4 @@
-// @(#)root/geom:$Id: TGeoVolume.h 42999 2012-02-14 13:42:55Z agheata $
+// @(#)root/geom:$Id$
 // Author: Andrei Gheata   30/05/02
 
 /*************************************************************************
@@ -49,6 +49,7 @@ class TGeoMatrix;
 class TGeoPatternFinder;
 class TGeoVoxelFinder;
 class TGeoManager;
+class TGeoExtension;
 
 ////////////////////////////////////////////////////////////////////////////
 //                                                                        //
@@ -75,6 +76,8 @@ protected :
    TString            fOption;         //! option - if any
    Int_t              fNumber;         //  volume serial number in the list of volumes
    Int_t              fNtotal;         // total number of physical nodes
+   TGeoExtension     *fUserExtension;  //! Transient user-defined extension to volumes
+   TGeoExtension     *fFWExtension;    //! Transient framework-defined extension to volumes
 
    TGeoVolume(const TGeoVolume&); 
    TGeoVolume& operator=(const TGeoVolume&);
@@ -136,6 +139,12 @@ public:
    void            PrintNodes() const;
    void            PrintVoxels() const; // *MENU*
    void            ReplayCreation(const TGeoVolume *other);
+   void            SetUserExtension(TGeoExtension *ext);
+   void            SetFWExtension(TGeoExtension *ext);
+   TGeoExtension  *GetUserExtension() const {return fUserExtension;}
+   TGeoExtension  *GetFWExtension() const   {return fFWExtension;}
+   TGeoExtension  *GrabUserExtension() const;
+   TGeoExtension  *GrabFWExtension() const;
    virtual void    ExecuteEvent(Int_t event, Int_t px, Int_t py);
 
    Bool_t          IsActive() const {return TGeoAtt::IsActive();}
@@ -193,13 +202,13 @@ public:
    TGeoVolume     *MakeReflectedVolume(const char *newname="") const;
    Bool_t          OptimizeVoxels(); // *MENU*
    void            RandomPoints(Int_t npoints=1000000, Option_t *option=""); // *MENU*
-   void            RandomRays(Int_t nrays=10000, Double_t startx=0, Double_t starty=0, Double_t startz=0); // *MENU*
+   void            RandomRays(Int_t nrays=10000, Double_t startx=0, Double_t starty=0, Double_t startz=0, const char *target_vol=0, Bool_t check_norm=kFALSE); // *MENU*
    void            Raytrace(Bool_t flag=kTRUE); // *TOGGLE* *GETTER=IsRaytracing
    void            RegisterYourself(Option_t *option="");
    void            RemoveNode(TGeoNode *node);
    TGeoNode       *ReplaceNode(TGeoNode *nodeorig, TGeoShape *newshape=0, TGeoMatrix *newpos=0, TGeoMedium *newmed=0);
    void            SaveAs(const char *filename,Option_t *option="") const; // *MENU*
-   virtual void    SavePrimitive(ostream &out, Option_t *option = "");
+   virtual void    SavePrimitive(std::ostream &out, Option_t *option = "");
    void            SelectVolume(Bool_t clear = kFALSE);
    void            SetActivity(Bool_t flag=kTRUE) {TGeoAtt::SetActivity(flag);}
    void            SetActiveDaughters(Bool_t flag=kTRUE) {TGeoAtt::SetActiveDaughters(flag);}
