@@ -1,4 +1,4 @@
-// @(#):$Id: TGeoBoolNode.cxx 42999 2012-02-14 13:42:55Z agheata $
+// @(#):$Id$
 // Author: Andrei Gheata   30/05/02
 // TGeoBoolNode::Contains and parser implemented by Mihaela Gheata
 
@@ -202,7 +202,7 @@ Bool_t TGeoBoolNode::MakeBranch(const char *expr, Bool_t left)
    TString sleft, sright, stransf;
    Int_t boolop = TGeoManager::Parse(expr, sleft, sright, stransf);
    if (boolop<0) {
-      Error("MakeBranch", "invalid expresion");
+      Error("MakeBranch", "invalid expression");
       return kFALSE;
    }
    TGeoShape *shape = 0;
@@ -317,7 +317,32 @@ void TGeoBoolNode::RegisterMatrices()
 }
 
 //_____________________________________________________________________________
-void TGeoBoolNode::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
+Bool_t TGeoBoolNode::ReplaceMatrix(TGeoMatrix *mat, TGeoMatrix *newmat)
+{
+// Replace one of the matrices. Does not work with TGeoIdentity. Returns true
+// if replacement was successful.
+   if (mat==gGeoIdentity || newmat==gGeoIdentity) {
+      Error("ReplaceMatrix", "Matrices should not be gGeoIdentity. Use default matrix constructor to repersent identities.");
+      return kFALSE;
+   }
+   if (!mat || !newmat) {
+      Error("ReplaceMatrix", "Matrices should not be null pointers.");
+      return kFALSE;
+   }
+   Bool_t replaced = kFALSE;
+   if (fLeftMat == mat) {
+      fLeftMat  = newmat;
+      replaced = kTRUE;
+   }
+   if (fRightMat == mat) {
+      fRightMat  = newmat;
+      replaced = kTRUE;
+   }
+   return replaced;
+}
+
+//_____________________________________________________________________________
+void TGeoBoolNode::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
 // Save a primitive as a C++ statement(s) on output stream "out".
    fLeft->SavePrimitive(out,option);
@@ -357,8 +382,16 @@ void TGeoBoolNode::Sizeof3D() const
    fLeft->Sizeof3D();
    fRight->Sizeof3D();
 }
+
 ClassImp(TGeoUnion)
 
+//______________________________________________________________________________
+TGeoBoolNode *TGeoUnion::MakeClone() const
+{
+// Make a clone of this. Pointers are preserved.
+   return new TGeoUnion(fLeft, fRight, fLeftMat, fRightMat);
+}   
+
 //______________________________________________________________________________
 void TGeoUnion::Paint(Option_t *option)
 {
@@ -443,23 +476,18 @@ void TGeoUnion::ComputeBBox(Double_t &dx, Double_t &dy, Double_t &dz, Double_t *
    origin[1] = 0.5*(ymin+ymax);
    dz = 0.5*(zmax-zmin);
    origin[2] = 0.5*(zmin+zmax);
-}   
+}
 
 //______________________________________________________________________________
 Bool_t TGeoUnion::Contains(Double_t *point) const
 {
 // Find if a union of two shapes contains a given point
    Double_t local[3];
-   TGeoBoolNode *node = (TGeoBoolNode*)this;
    fLeftMat->MasterToLocal(point, &local[0]);
    Bool_t inside = fLeft->Contains(&local[0]);
-   if (inside) {
-      node->SetSelected(1);
-      return kTRUE;
-   }   
+   if (inside) return kTRUE;
    fRightMat->MasterToLocal(point, &local[0]);
    inside = fRight->Contains(&local[0]);
-   if (inside) node->SetSelected(2);
    return inside;
 }
 
@@ -548,9 +576,9 @@ Double_t TGeoUnion::DistFromInside(Double_t *point, Double_t *dir, Int_t iact,
    Bool_t inside2 = fRight->Contains(local);
    if (inside2) d2 = fRight->DistFromInside(local, rdir, 3);
    if (!(inside1 | inside2)) {
-   // May be a pathological case when the point is on the boundary
+   // This is a pathological case when the point is on the boundary
       d1 = fLeft->DistFromOutside(local1, ldir, 3);
-      if (d1<2.*TGeoShape::Tolerance()) {
+      if (d1<1.E-3) {
          eps = d1+TGeoShape::Tolerance();
          for (i=0; i<3; i++) local1[i] += eps*ldir[i];
          inside1 = kTRUE;
@@ -558,7 +586,7 @@ Double_t TGeoUnion::DistFromInside(Double_t *point, Double_t *dir, Int_t iact,
          d1 += eps;
       } else {      
          d2 = fRight->DistFromOutside(local, rdir, 3);
-         if (d2<2.*TGeoShape::Tolerance()) {
+         if (d2<1.E-3) {
            eps = d2+TGeoShape::Tolerance();
            for (i=0; i<3; i++) local[i] += eps*rdir[i];
            inside2 = kTRUE;
@@ -721,7 +749,7 @@ Double_t TGeoUnion::Safety(Double_t *point, Bool_t in) const
 }   
 
 //_____________________________________________________________________________
-void TGeoUnion::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
+void TGeoUnion::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
 // Save a primitive as a C++ statement(s) on output stream "out".
    TGeoBoolNode::SavePrimitive(out,option);
@@ -730,8 +758,8 @@ void TGeoUnion::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
    out << fRight->GetPointerName() << ",";
    if (!fLeftMat->IsIdentity()) out << fLeftMat->GetPointerName() << ",";
    else                         out << "0,";
-   if (!fRightMat->IsIdentity()) out << fRightMat->GetPointerName() << ");" << endl;
-   else                         out << "0);" << endl;
+   if (!fRightMat->IsIdentity()) out << fRightMat->GetPointerName() << ");" << std::endl;
+   else                         out << "0);" << std::endl;
 }   
 
 //______________________________________________________________________________
@@ -744,6 +772,13 @@ void TGeoUnion::Sizeof3D() const
 
 ClassImp(TGeoSubtraction)
 
+//______________________________________________________________________________
+TGeoBoolNode *TGeoSubtraction::MakeClone() const
+{
+// Make a clone of this. Pointers are preserved.
+   return new TGeoSubtraction(fLeft, fRight, fLeftMat, fRightMat);
+}
+
 //______________________________________________________________________________
 void TGeoSubtraction::Paint(Option_t *option)
 {
@@ -874,14 +909,11 @@ Bool_t TGeoSubtraction::Contains(Double_t *point) const
 {
 // Find if a subtraction of two shapes contains a given point
    Double_t local[3];
-   TGeoBoolNode *node = (TGeoBoolNode*)this;
    fLeftMat->MasterToLocal(point, &local[0]);
    Bool_t inside = fLeft->Contains(&local[0]);
-   if (inside) node->SetSelected(1);
-   else return kFALSE;
+   if (!inside) return kFALSE;
    fRightMat->MasterToLocal(point, &local[0]);
    inside = !fRight->Contains(&local[0]);
-   if (!inside) node->SetSelected(2);
    return inside;
 }
 
@@ -1034,7 +1066,7 @@ Double_t TGeoSubtraction::Safety(Double_t *point, Bool_t in) const
 }   
 
 //_____________________________________________________________________________
-void TGeoSubtraction::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
+void TGeoSubtraction::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
 // Save a primitive as a C++ statement(s) on output stream "out".
    TGeoBoolNode::SavePrimitive(out,option);
@@ -1043,8 +1075,8 @@ void TGeoSubtraction::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
    out << fRight->GetPointerName() << ",";
    if (!fLeftMat->IsIdentity()) out << fLeftMat->GetPointerName() << ",";
    else                         out << "0,";
-   if (!fRightMat->IsIdentity()) out << fRightMat->GetPointerName() << ");" << endl;
-   else                         out << "0);" << endl;
+   if (!fRightMat->IsIdentity()) out << fRightMat->GetPointerName() << ");" << std::endl;
+   else                         out << "0);" << std::endl;
 }   
 
 //______________________________________________________________________________
@@ -1056,6 +1088,13 @@ void TGeoSubtraction::Sizeof3D() const
 
 ClassImp(TGeoIntersection)
 
+//______________________________________________________________________________
+TGeoBoolNode *TGeoIntersection::MakeClone() const
+{
+// Make a clone of this. Pointers are preserved.
+   return new TGeoIntersection(fLeft, fRight, fLeftMat, fRightMat);
+}
+
 //______________________________________________________________________________
 void TGeoIntersection::Paint(Option_t *option)
 {
@@ -1328,10 +1367,12 @@ Double_t TGeoIntersection::DistFromOutside(Double_t *point, Double_t *dir, Int_t
    node->SetSelected(0);
    Double_t snext = 0.0;
    if (inleft && inright) {
+      // It is vey likely to have a numerical issue and the point should
+      // be logically outside one of the shapes
       d1 = fLeft->DistFromInside(lpt,ldir,3);
       d2 = fRight->DistFromInside(rpt,rdir,3);
-      if (d1<2*tol) inleft = kFALSE;
-      if (d2<2*tol) inright = kFALSE;
+      if (d1<1.E-3) inleft = kFALSE;
+      if (d2<1.E-3) inright = kFALSE;
       if (inleft && inright) return snext;
    }   
 
@@ -1435,7 +1476,7 @@ Double_t TGeoIntersection::Safety(Double_t *point, Bool_t in) const
 }   
 
 //_____________________________________________________________________________
-void TGeoIntersection::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
+void TGeoIntersection::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
 // Save a primitive as a C++ statement(s) on output stream "out".
    TGeoBoolNode::SavePrimitive(out,option);
@@ -1444,8 +1485,8 @@ void TGeoIntersection::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
    out << fRight->GetPointerName() << ",";
    if (!fLeftMat->IsIdentity()) out << fLeftMat->GetPointerName() << ",";
    else                         out << "0,";
-   if (!fRightMat->IsIdentity()) out << fRightMat->GetPointerName() << ");" << endl;
-   else                         out << "0);" << endl;
+   if (!fRightMat->IsIdentity()) out << fRightMat->GetPointerName() << ");" << std::endl;
+   else                         out << "0);" << std::endl;
 }   
 
 //______________________________________________________________________________
