@@ -1,4 +1,4 @@
-// @(#)root/geom:$Id: TGeoCompositeShape.cxx 42999 2012-02-14 13:42:55Z agheata $
+// @(#)root/geom:$Id$
 // Author: Andrei Gheata   31/01/02
 
 /*************************************************************************
@@ -58,7 +58,7 @@
 //   This represent the union of A, B and C. Both union operators are at the
 // same level. Since:
 //        A+B+C = (A+B)+C = A+(B+C)
-// the first (+) is taken as separator, hence the expression splitted:
+// the first (+) is taken as separator, hence the expression split:
 //        A and B+C
 // A Boolean node of type TGeoUnion("A", "B+C") is created. This tries to replace
 // the 2 expressions by actual pointers to corresponding shapes.
@@ -462,14 +462,14 @@ Double_t TGeoCompositeShape::Safety(Double_t *point, Bool_t in) const
 }
 
 //_____________________________________________________________________________
-void TGeoCompositeShape::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
+void TGeoCompositeShape::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
 // Save a primitive as a C++ statement(s) on output stream "out".
    if (TObject::TestBit(kGeoSavePrimitive)) return;
    if (fNode) fNode->SavePrimitive(out,option);
-   out << "   // Shape: " << GetName() << " type: " << ClassName() << endl;
-   out << "   TGeoShape *" << GetPointerName() << " = new TGeoCompositeShape(\"" << GetName() << "\", pBoolNode);" << endl;
-   if (strlen(GetTitle())) out << "   " << GetPointerName() << "->SetTitle(\"" << GetTitle() << "\");" << endl;
+   out << "   // Shape: " << GetName() << " type: " << ClassName() << std::endl;
+   out << "   TGeoShape *" << GetPointerName() << " = new TGeoCompositeShape(\"" << GetName() << "\", pBoolNode);" << std::endl;
+   if (strlen(GetTitle())) out << "   " << GetPointerName() << "->SetTitle(\"" << GetTitle() << "\");" << std::endl;
    TObject::SetBit(TGeoShape::kGeoSavePrimitive);
 }
 
