@@ -1,4 +1,4 @@
-// @(#)root/geom:$Id: TGeoShape.cxx 41891 2011-11-10 22:46:31Z pcanal $
+// @(#)root/geom:$Id$
 // Author: Andrei Gheata   31/01/02
 
 /*************************************************************************
@@ -472,6 +472,34 @@ Double_t TGeoShape::SafetyPhi(Double_t *point, Bool_t in, Double_t phi1, Double_
    return safe;   
 }        
 
+//_____________________________________________________________________________
+Double_t TGeoShape::SafetySeg(Double_t r, Double_t z, Double_t r1, Double_t z1, Double_t r2, Double_t z2, Bool_t outer)
+{
+// Compute distance from point of coordinates (r,z) to segment (r1,z1):(r2,z2)
+   Double_t crossp = (z2-z1)*(r-r1)-(z-z1)*(r2-r1);
+   crossp *= (outer) ? 1. : -1.;
+   // Positive crossp means point on the requested side of the (1,2) segment
+   if (crossp < 0) {
+      if (((z-z1)*(z2-z)) > 0) return 0;
+      return TGeoShape::Big();
+   }   
+   // Compute (1,P) dot (1,2)
+   Double_t c1 = (z-z1)*(z2-z1)+(r-r1)*(r2-r1);
+   // Negative c1 means point (1) is closest
+   if (c1<1.E-10) return TMath::Sqrt((r-r1)*(r-r1)+(z-z1)*(z-z1));
+   // Compute (2,P) dot (1,2)
+   Double_t c2 = (z-z2)*(z2-z1)+(r-r2)*(r2-r1);
+   // Positive c2 means point (2) is closest
+   if (c2>-1.E-10) return TMath::Sqrt((r-r2)*(r-r2)+(z-z2)*(z-z2));
+   // The closest point is between (1) and (2)
+   c2 = (z2-z1)*(z2-z1)+(r2-r1)*(r2-r1);
+   // projected length factor with respect to (1,2) length
+   Double_t alpha = c1/c2;
+   Double_t rp = r1 + alpha*(r2-r1);
+   Double_t zp = z1 + alpha*(z2-z1);
+   return TMath::Sqrt((r-rp)*(r-rp)+(z-zp)*(z-zp));
+}
+
 //_____________________________________________________________________________
 void TGeoShape::SetShapeBit(UInt_t f, Bool_t set)
 {
