@@ -1,4 +1,4 @@
-// @(#)root/graf2d:$Id: TGCocoa.h 44330 2012-05-29 11:30:29Z tpochep $
+// @(#)root/graf2d:$Id$
 // Author: Timur Pocheptsov   22/11/2011
 
 /*************************************************************************
@@ -16,11 +16,15 @@
 #include <utility>
 #include <vector>
 #include <memory>
+#include <string>
 #include <map>
 
 #ifndef ROOT_TVirtualX
 #include "TVirtualX.h"
 #endif
+#ifndef ROOT_X11Atoms
+#include "X11Atoms.h"
+#endif
 
 //////////////////////////////////////////////////////////////////////////
 //                                                                      //
@@ -34,23 +38,7 @@
 //                                                                      //
 //////////////////////////////////////////////////////////////////////////
 
-namespace ROOT {
-namespace Quartz {
-
-class CGStateGuard {
-public:
-   CGStateGuard(void *ctx);
-   ~CGStateGuard();
-   
-private:
-   void *fCtx;
-   
-   CGStateGuard(const CGStateGuard &rhs);
-   CGStateGuard &operator = (const CGStateGuard &rhs);
-};
-
-}
-   
+namespace ROOT {   
 namespace MacOSX {
 
 namespace X11 {
@@ -78,12 +66,14 @@ public:
    ///////////////////////////////////////
    //General.
    virtual Bool_t      Init(void *display);
-   virtual Int_t       OpenDisplay(const char *dpyName);
+   virtual Int_t       OpenDisplay(const char *displayName);
    virtual const char *DisplayName(const char *);
+   virtual Int_t       SupportsExtension(const char *extensionName)const;
    virtual void        CloseDisplay();
    virtual Display_t   GetDisplay()const;
    virtual Visual_t    GetVisual()const;
    virtual Int_t       GetScreen()const;
+   virtual UInt_t      ScreenWidthMM()const;
    virtual Int_t       GetDepth()const;
    virtual void        Update(Int_t mode);
    //End of general.
@@ -104,10 +94,8 @@ public:
    virtual void      UpdateWindow(Int_t mode);
    virtual Window_t  GetCurrentWindow() const;
    virtual void      CloseWindow();
-
-   //-"Qt ROOT".
-   virtual Int_t     AddWindow(ULong_t qwid, UInt_t w, UInt_t h);
-   virtual void      RemoveWindow(ULong_t qwid);
+   virtual Int_t     AddWindow(ULong_t qwid, UInt_t w, UInt_t h); //-"Qt ROOT".
+   virtual void      RemoveWindow(ULong_t qwid); //-"Qt ROOT".
 
 
    //-Functions used by GUI.
@@ -154,9 +142,23 @@ public:
    virtual void      SetIconName(Window_t wid, char *name);
    virtual void      SetIconPixmap(Window_t wid, Pixmap_t pix);
    virtual void      SetClassHints(Window_t wid, char *className, char *resourceName);
+   //Non-rectangular window:
+   virtual void      ShapeCombineMask(Window_t wid, Int_t x, Int_t y, Pixmap_t mask);
+
    //End window-management part.
    ///////////////////////////////////////
    
+   /////////////////////////////
+   //Set of "Window manager hints".
+   virtual void      SetMWMHints(Window_t winID, UInt_t value, UInt_t decorators, UInt_t inputMode);
+   virtual void      SetWMPosition(Window_t winID, Int_t x, Int_t y);
+   virtual void      SetWMSize(Window_t winID, UInt_t w, UInt_t h);
+   virtual void      SetWMSizeHints(Window_t winID, UInt_t wMin, UInt_t hMin, UInt_t wMax, UInt_t hMax, UInt_t wInc, UInt_t hInc);
+   virtual void      SetWMState(Window_t winID, EInitialState state);
+   virtual void      SetWMTransientHint(Window_t winID, Window_t mainWinID);
+   //"Window manager hints".
+   /////////////////////////////
+
 
    ///////////////////////////////////////
    //GUI-rendering part.
@@ -208,6 +210,18 @@ public:
    //End of pixmap management.
    /////////////////////////////
 
+
+   /////////////////////////////
+   //"Images" - emulation of XCreateImage/XPutImage etc.
+   virtual Drawable_t   CreateImage(UInt_t width, UInt_t height);
+   virtual void         GetImageSize(Drawable_t wid, UInt_t &width, UInt_t &height);
+   virtual void         PutPixel(Drawable_t wid, Int_t x, Int_t y, ULong_t pixel);
+   virtual void         PutImage(Drawable_t wid, GContext_t gc, Drawable_t img, Int_t dx, Int_t dy,
+                                 Int_t x, Int_t y, UInt_t w, UInt_t h);
+   virtual void         DeleteImage(Drawable_t img);
+   //"Images".
+   /////////////////////////////
+
    /////////////////////////////
    //Mouse (cursor, events, etc.)
    virtual void      GrabButton(Window_t wid, EMouseButton button, UInt_t modifier,
@@ -216,9 +230,20 @@ public:
    virtual void      GrabPointer(Window_t wid, UInt_t evmask, Window_t confine,
                                  Cursor_t cursor, Bool_t grab = kTRUE,
                                  Bool_t owner_events = kTRUE);
-
+   virtual void      ChangeActivePointerGrab(Window_t, UInt_t, Cursor_t);//Noop.
    //End of mouse related part.
    /////////////////////////////
+   
+   /////////////////////////////
+   //Keyboard management.
+   virtual void      SetKeyAutoRepeat(Bool_t on = kTRUE);
+   virtual void      GrabKey(Window_t wid, Int_t keycode, UInt_t modifier, Bool_t grab = kTRUE);
+   virtual Int_t     KeysymToKeycode(UInt_t keysym);
+   virtual Window_t  GetInputFocus();
+   virtual void      SetInputFocus(Window_t wid);
+   virtual void      LookupString(Event_t *event, char *buf, Int_t buflen, UInt_t &keysym);
+   //End of keyboard management.
+   /////////////////////////////
 
    /////////////////////////////
    //Font management.
@@ -265,42 +290,87 @@ public:
    //Cursors.
    virtual Cursor_t     CreateCursor(ECursor cursor);
    virtual void         SetCursor(Window_t wid, Cursor_t curid);
-   virtual void         SetCursor(Int_t win, ECursor cursor);   
+   virtual void         SetCursor(Int_t win, ECursor cursor);
+   virtual void         QueryPointer(Int_t &x, Int_t &y);
+   virtual void         QueryPointer(Window_t wid, Window_t &rootw, Window_t &childw,
+                                     Int_t &root_x, Int_t &root_y, Int_t &win_x,
+                                     Int_t &win_y, UInt_t &mask);
    //Cursors.
    /////////////////////////////
 
-   //Remaining bunch of functions is not sorted yet (and not imlemented at the moment).
 
-   virtual void      ChangeProperty(Window_t wid, Atom_t property, Atom_t type,
-                                    UChar_t *data, Int_t len);
-
-   //Set of "Window manager hints".
-   virtual void      SetMWMHints(Window_t wid, UInt_t value, UInt_t funcs, UInt_t input);
-   virtual void      SetWMPosition(Window_t wid, Int_t x, Int_t y);
-   virtual void      SetWMSize(Window_t wid, UInt_t w, UInt_t h);
-   virtual void      SetWMSizeHints(Window_t wid, UInt_t wmin, UInt_t hmin,
-                                       UInt_t wmax, UInt_t hmax, UInt_t winc, UInt_t hinc);
-   virtual void      SetWMState(Window_t wid, EInitialState state);
-   virtual void      SetWMTransientHint(Window_t wid, Window_t main_id);
-
-   //
+   /////////////////////////////
+   //OpenGL.
+   virtual Double_t  GetOpenGLScalingFactor();
    virtual Window_t  CreateOpenGLWindow(Window_t parentID, UInt_t width, UInt_t height, const std::vector<std::pair<UInt_t, Int_t> > &format);
    virtual Handle_t  CreateOpenGLContext(Window_t windowID, Handle_t sharedContext);
    virtual void      CreateOpenGLContext(Int_t wid);
-   virtual Bool_t    MakeOpenGLContextCurrent(Handle_t ctx);
+   virtual Bool_t    MakeOpenGLContextCurrent(Handle_t ctx, Window_t windowID);
+   virtual Handle_t  GetCurrentOpenGLContext();
    virtual void      FlushOpenGLBuffer(Handle_t ctx);
 
-   virtual void      DeleteOpenGLContext(Int_t wid);
+   virtual void      DeleteOpenGLContext(Int_t wid);   
+   //OpenGL.
+   /////////////////////////////
+
+   /////////////////////////////
+   //TPad's/TCanvas' specific - "double buffer" (off-screen rendering) + 'xor' mode.
+   virtual void      SetDoubleBuffer(Int_t wid, Int_t mode);
+   virtual void      SetDoubleBufferOFF();
+   virtual void      SetDoubleBufferON();
+   virtual void      SetDrawMode(EDrawMode mode);
+   //TPad's/TCanvas'.
+   /////////////////////////////
+
+   /////////////////////////////   
+   //Event management.
+   virtual void      SendEvent(Window_t wid, Event_t *ev);
+   virtual void      NextEvent(Event_t &event);
+   virtual Int_t     EventsPending();
+   virtual Bool_t    CheckEvent(Window_t wid, EGEventType type, Event_t &ev);
+   virtual Handle_t  GetNativeEvent() const;
+   //Event management.
+   /////////////////////////////
+
+   /////////////////////////////   
+   //"Drag and drop" and "Copy and paste" (quotes are intentional :)).
+   
+   //Names here are total mess, but this comes from TVirtualX interface.
+   virtual Atom_t    InternAtom(const char *atom_name, Bool_t only_if_exist);
+
+   virtual void      SetPrimarySelectionOwner(Window_t wid);
+   virtual Bool_t    SetSelectionOwner(Window_t windowID, Atom_t &selectionID);
+   virtual Window_t  GetPrimarySelectionOwner();
+
+   virtual void      ConvertPrimarySelection(Window_t wid, Atom_t clipboard, Time_t when);
+   virtual void      ConvertSelection(Window_t, Atom_t&, Atom_t&, Atom_t&, Time_t&);
+   virtual Int_t     GetProperty(Window_t, Atom_t, Long_t, Long_t, Bool_t, Atom_t,
+                                    Atom_t*, Int_t*, ULong_t*, ULong_t*, unsigned char**);
+   virtual void      GetPasteBuffer(Window_t wid, Atom_t atom, TString &text, Int_t &nchar,
+                                    Bool_t del);
+
+   virtual void      ChangeProperty(Window_t wid, Atom_t property, Atom_t type,
+                                    UChar_t *data, Int_t len);
+   virtual void      ChangeProperties(Window_t wid, Atom_t property, Atom_t type,
+                                      Int_t format, UChar_t *data, Int_t len);
+   virtual void      DeleteProperty(Window_t, Atom_t&);
+
+   virtual void      SetDNDAware(Window_t, Atom_t *);
+   virtual Bool_t    IsDNDAware(Window_t win, Atom_t *typelist);
+
+   virtual void      SetTypeList(Window_t win, Atom_t prop, Atom_t *typelist);
+   //FindRWindow is in DND part, since it looks for a DND aware window.
+   virtual Window_t  FindRWindow(Window_t win, Window_t dragwin, Window_t input, int x, int y, int maxd);
+   //"Drag and drop" and "Copy and paste".
+   /////////////////////////////
+
+   //The remaining bunch of functions is not sorted yet (and not imlemented at the moment).
 
    virtual UInt_t    ExecCommand(TGWin32Command *code);
    virtual void      GetCharacterUp(Float_t &chupx, Float_t &chupy);
 
    virtual Int_t     GetDoubleBuffer(Int_t wid);
-   virtual Handle_t  GetNativeEvent() const;
 
-
-
-   virtual void      QueryPointer(Int_t &ix, Int_t &iy);
    virtual Pixmap_t  ReadGIF(Int_t x0, Int_t y0, const char *file, Window_t wid);
    virtual Int_t     RequestLocator(Int_t mode, Int_t ctyp, Int_t &x, Int_t &y);
    virtual Int_t     RequestString(Int_t x, Int_t y, char *text);
@@ -308,10 +378,6 @@ public:
    virtual void      SetCharacterUp(Float_t chupx, Float_t chupy);
    virtual void      SetClipOFF(Int_t wid);
    virtual void      SetClipRegion(Int_t wid, Int_t x, Int_t y, UInt_t w, UInt_t h);
-   virtual void      SetDoubleBuffer(Int_t wid, Int_t mode);
-   virtual void      SetDoubleBufferOFF();
-   virtual void      SetDoubleBufferON();
-   virtual void      SetDrawMode(EDrawMode mode);
 
    virtual void      SetTextMagnitude(Float_t mgn);
 
@@ -319,14 +385,10 @@ public:
    virtual void      Warp(Int_t ix, Int_t iy, Window_t wid);
    virtual Int_t     WriteGIF(char *name);
    virtual void      WritePixmap(Int_t wid, UInt_t w, UInt_t h, char *pxname);
-   virtual Int_t     SupportsExtension(const char *ext) const;
 
    virtual Bool_t       NeedRedraw(ULong_t tgwindow, Bool_t force);
 
 
-   virtual UInt_t       ScreenWidthMM() const;
-   virtual Atom_t       InternAtom(const char *atom_name, Bool_t only_if_exist);
-
    virtual Bool_t       CreatePictureFromFile(Drawable_t wid, const char *filename,
                                               Pixmap_t &pict, Pixmap_t &pict_mask,
                                               PictureAttributes_t &attr);
@@ -336,29 +398,12 @@ public:
    virtual Bool_t       ReadPictureDataFromFile(const char *filename, char ***ret_data);
    virtual void         DeletePictureData(void *data);
    virtual void         SetDashes(GContext_t gc, Int_t offset, const char *dash_list, Int_t n);
-   virtual Int_t        EventsPending();
-   virtual void         NextEvent(Event_t &event);
+
+   
    virtual void         Bell(Int_t percent);
    
-   virtual Bool_t       CheckEvent(Window_t wid, EGEventType type, Event_t &ev);
-   virtual void         SendEvent(Window_t wid, Event_t *ev);
-   virtual void         DispatchClientMessage(UInt_t messageID);
-   virtual void         RemoveEventsForWindow(Window_t wid);
    virtual void         WMDeleteNotify(Window_t wid);
-   virtual void         SetKeyAutoRepeat(Bool_t on = kTRUE);
-   virtual void         GrabKey(Window_t wid, Int_t keycode, UInt_t modifier, Bool_t grab = kTRUE);
-   virtual Int_t        KeysymToKeycode(UInt_t keysym);
-   virtual Window_t     GetInputFocus();
-   virtual void         SetInputFocus(Window_t wid);
-   virtual Window_t     GetPrimarySelectionOwner();
-   virtual void         SetPrimarySelectionOwner(Window_t wid);
-   virtual void         ConvertPrimarySelection(Window_t wid, Atom_t clipboard, Time_t when);
-   virtual void         LookupString(Event_t *event, char *buf, Int_t buflen, UInt_t &keysym);
-   virtual void         GetPasteBuffer(Window_t wid, Atom_t atom, TString &text, Int_t &nchar,
-                                       Bool_t del);
-   virtual void         QueryPointer(Window_t wid, Window_t &rootw, Window_t &childw,
-                                     Int_t &root_x, Int_t &root_y, Int_t &win_x,
-                                     Int_t &win_y, UInt_t &mask);
+
    virtual void         SetClipRectangles(GContext_t gc, Int_t x, Int_t y, Rectangle_t *recs, Int_t n);
    virtual Region_t     CreateRegion();
    virtual void         DestroyRegion(Region_t reg);
@@ -372,29 +417,7 @@ public:
    virtual Bool_t       PointInRegion(Int_t x, Int_t y, Region_t reg);
    virtual Bool_t       EqualRegion(Region_t rega, Region_t regb);
    virtual void         GetRegionBox(Region_t reg, Rectangle_t *rect);
-   virtual Drawable_t   CreateImage(UInt_t width, UInt_t height);
-   virtual void         GetImageSize(Drawable_t wid, UInt_t &width, UInt_t &height);
-   virtual void         PutPixel(Drawable_t wid, Int_t x, Int_t y, ULong_t pixel);
-   virtual void         PutImage(Drawable_t wid, GContext_t gc, Drawable_t img, Int_t dx, Int_t dy,
-                                 Int_t x, Int_t y, UInt_t w, UInt_t h);
-   virtual void         DeleteImage(Drawable_t img);
-   virtual void         ShapeCombineMask(Window_t wid, Int_t x, Int_t y, Pixmap_t mask);
-
-   //---- Drag and Drop -----
-   virtual void         DeleteProperty(Window_t, Atom_t&);
-   virtual Int_t        GetProperty(Window_t, Atom_t, Long_t, Long_t, Bool_t, Atom_t,
-                                    Atom_t*, Int_t*, ULong_t*, ULong_t*, unsigned char**);
-   virtual void         ChangeActivePointerGrab(Window_t, UInt_t, Cursor_t);
-   virtual void         ConvertSelection(Window_t, Atom_t&, Atom_t&, Atom_t&, Time_t&);
-   virtual Bool_t       SetSelectionOwner(Window_t, Atom_t&);
-   virtual void         ChangeProperties(Window_t wid, Atom_t property, Atom_t type,
-                                         Int_t format, UChar_t *data, Int_t len);
-   virtual void         SetDNDAware(Window_t, Atom_t *);
-   virtual void         SetTypeList(Window_t win, Atom_t prop, Atom_t *typelist);
-   virtual Window_t     FindRWindow(Window_t win, Window_t dragwin, Window_t input, int x, int y, int maxd);
-   virtual Bool_t       IsDNDAware(Window_t win, Atom_t *typelist);
-
-   virtual void         BeginModalSessionFor(Window_t wid);
+   //
 
    virtual Bool_t       IsCmdThread() const { return kTRUE; }
    
@@ -409,44 +432,35 @@ public:
 protected:
    void *GetCurrentContext();
 
-   Int_t fSelectedDrawable;
+   Drawable_t fSelectedDrawable;
 
    std::auto_ptr<ROOT::MacOSX::Details::CocoaPrivate> fPimpl; //!
    Int_t fCocoaDraw;
 
    EDrawMode fDrawMode;
    bool fDirectDraw;//Primitive in canvas tries to draw into window directly.
-   
-   //TODO:
-   //There is no property support yet,
-   //only this two valus to make GUI work 
-   //(used in client messages). 
-
-public:
-
-   enum EInternAtom {
-      kIA_DELETE_WINDOW = 1,
-      kIA_ROOT_MESSAGE
-   };
 
 private:
-   bool IsDialog(Window_t wid)const;
    bool MakeProcessForeground();
+   Atom_t FindAtom(const std::string &atomName, bool addIfNotFound);
+   void SetApplicationIcon();
 
    bool fForegroundProcess;
-   std::vector<GCValues_t> fX11Contexts;
+   std::vector<GCValues_t> fX11Contexts;   
+   //
+   ROOT::MacOSX::X11::name_to_atom_map fNameToAtom;
+   std::vector<std::string> fAtomToName;
 
-   typedef std::pair<Window_t, Event_t> ClientMessage_t;
-   std::vector<UInt_t> fFreeMessageIDs;
-   UInt_t fCurrentMessageID;
-   std::map<UInt_t, ClientMessage_t> fClientMessages;
-   typedef std::map<UInt_t, ClientMessage_t>::iterator message_iterator;
+   std::map<Atom_t, Window_t> fSelectionOwners;
+   typedef std::map<Atom_t, Window_t>::iterator selection_iterator;
    
-   //Quite ugly solution for the moment.
-   std::map<Window_t, std::vector<UInt_t> > fClientMessagesToWindow;
-   typedef std::map<Window_t, std::vector<UInt_t> >::iterator message_window_iterator;
-      
-   //I'd prefere to use = delete syntax from C++0x11, but this file is processed by CINT.
+   bool fSetApp;
+
+public:
+   static Atom_t fgDeleteWindowAtom;
+
+private:
+   //I'd prefer to use = delete syntax from C++0x11, but this file is processed by CINT.
    TGCocoa(const TGCocoa &rhs);
    TGCocoa &operator = (const TGCocoa &rhs);
 
