@@ -1,4 +1,4 @@
-// @(#)root/graf2d:$Id: FontCache.mm 44201 2012-05-09 15:17:36Z tpochep $
+// @(#)root/graf2d:$Id$
 // Author: Timur Pocheptsov   19/03/2012
 
 /*************************************************************************
@@ -30,12 +30,14 @@ namespace Details {
 
 namespace {
 
-//ROOT uses indices for fonts.
-//Later, I'll find (I promise! ;) better
-//way to map font indices to actual fonts
-//(families, etc.) - I simply do not have any time now.
+//ROOT uses indices for fonts. Indices are in the range [1 .. 15],
+//12 is a symbol font (quite special thing, see the code below,
+//15 is a "symbol italic" font - shear transformation is applied.
+
+//TODO: actually, it's not good to assume I have these fonts for sure,
+//find a better way to check the available fonts and search for the best
+//match.
 
-const int fmdNOfFonts = 13;
 const CFStringRef fixedFontNames[FontCache::nPadFonts] =
                                      {
                                       CFSTR("TimesNewRomanPS-ItalicMT"),
@@ -50,7 +52,9 @@ const CFStringRef fixedFontNames[FontCache::nPadFonts] =
                                       CFSTR("Courier-Bold"),
                                       CFSTR("Courier-BoldOblique"),
                                       CFSTR("Symbol"),
-                                      CFSTR("TimesNewRomanPSMT")
+                                      CFSTR("TimesNewRomanPSMT"),
+                                      CFSTR("Wingdings"),
+                                      CFSTR("Symbol-Italic")
                                      };
 
 
@@ -203,6 +207,7 @@ void CreateXLFDString(const X11::XLFDName &xlfd, std::string &xlfdString)
 
 //_________________________________________________________________
 FontCache::FontCache()
+             : fSymbolFontRegistered(false)
 {
 }
 
@@ -396,12 +401,9 @@ void FontCache::GetFontProperties(FontStruct_t font, int &maxAscent, int &maxDes
 
    assert(fLoadedFonts.find(fontRef) != fLoadedFonts.end() && "Font was not created by font manager");
 
-   //Instead of this, use CT funtion to request ascent/descent.
    try {
-      const Quartz::TextLine textLine("LALALA", fontRef);
-      textLine.GetAscentDescent(maxAscent, maxDescent);
-      maxAscent += 1;
-      maxDescent += 1;
+      maxAscent = int(CTFontGetAscent(fontRef) + 0.5) + 2;
+      maxDescent = int(CTFontGetDescent(fontRef) + 0.5);
    } catch (const std::exception &) {
       throw;
    }
@@ -415,13 +417,12 @@ CTFontRef FontCache::SelectFont(Font_t fontIndex, Float_t fontSize)
 
    if (fontIndex > nPadFonts || !fontIndex) {
       ::Warning("FontCache::SelectFont", "Font with index %d was requested", fontIndex);
-      fontIndex = 1;
-   }
+      fontIndex = 3;//Select the Helvetica as default.
+   } else
+      fontIndex -= 1;
    
-   fontIndex -= 1;
-   
-   if (fontIndex == 11)//Special case, our own symbol.ttf file.
-      return SelectSymbolFont(fontSize);
+   if (fontIndex == 11 || fontIndex == 14)//Special case, our own symbol.ttf file.
+      return SelectSymbolFont(fontSize, fontIndex);
    
    const UInt_t fixedSize = UInt_t(fontSize);
    font_map_iterator it = fFonts[fontIndex].find(fixedSize);
@@ -446,19 +447,22 @@ CTFontRef FontCache::SelectFont(Font_t fontIndex, Float_t fontSize)
 }
 
 //_________________________________________________________________
-CTFontRef FontCache::SelectSymbolFont(Float_t fontSize)
+CTFontRef FontCache::SelectSymbolFont(Float_t fontSize, unsigned fontIndex)
 {
+   assert(fontIndex == 11 || fontIndex == 14 && "SelectSymbolFont, parameter fontIndex has invalid value");
+
    const UInt_t fixedSize = UInt_t(fontSize);
-   font_map_iterator it = fFonts[11].find(fixedSize);//In ROOT, 11 is a font from symbol.ttf.
+   font_map_iterator it = fFonts[fontIndex].find(fixedSize);//In ROOT, 11 is a font from symbol.ttf.
    
-   if (it == fFonts[11].end()) {
+   if (it == fFonts[fontIndex].end()) {
       //This GetValue + Which I took from Olivier's code.
-      const char *fontDirectoryPath = gEnv->GetValue("Root.TTFontPath","$(ROOTSYS)/fonts");//This one I do not own.
-      char *fontFileName = gSystem->Which(fontDirectoryPath, "symbol.ttf", kReadPermission);//This must be deleted.
+      const char * const fontDirectoryPath = gEnv->GetValue("Root.TTFontPath","$(ROOTSYS)/fonts");//This one I do not own.
+      char * const fontFileName = gSystem->Which(fontDirectoryPath, "symbol.ttf", kReadPermission);//This must be deleted.
+      
+      const Util::ScopedArray<char> arrayGuard(fontFileName);
 
       if (!fontFileName || fontFileName[0] == 0) {
-         ::Error("FontCache::SelectSymbolFont", "sumbol.ttf file not found");
-         delete [] fontFileName;
+         ::Error("FontCache::SelectSymbolFont", "symbol.ttf file not found");
          return 0;
       }
 
@@ -466,28 +470,45 @@ CTFontRef FontCache::SelectSymbolFont(Float_t fontSize)
          const Util::CFScopeGuard<CFStringRef> path(CFStringCreateWithCString(kCFAllocatorDefault, fontFileName, kCFURLPOSIXPathStyle));
          if (!path.Get()) {
             ::Error("FontCache::SelectSymbolFont", "CFStringCreateWithCString failed");
-            delete [] fontFileName;
+            return 0;
+         }
+
+         const Util::CFScopeGuard<CFURLRef> fontURL(CFURLCreateWithFileSystemPath(kCFAllocatorDefault, path.Get(), kCFURLPOSIXPathStyle, false));
+         if (!fontURL.Get()) {
+            ::Error("FontCache::SelectSymbolFont", "CFURLCreateWithFileSystemPath failed");
             return 0;
          }
          
-         const Util::CFScopeGuard<CFArrayRef> arr(CTFontManagerCreateFontDescriptorsFromURL(CFURLCreateWithFileSystemPath(kCFAllocatorDefault, path.Get(), kCFURLPOSIXPathStyle, false)));
+         //Try to register this font.
+         if (!fSymbolFontRegistered) {
+            CFErrorRef err = 0;
+            fSymbolFontRegistered = CTFontManagerRegisterFontsForURL(fontURL.Get(), kCTFontManagerScopeProcess, &err);
+            if (!fSymbolFontRegistered) {
+               ::Error("FontCache::SelectSymbolFont", "CTFontManagerRegisterFontsForURL failed");
+               if (err)
+                  CFRelease(err);
+               return 0;
+            }
+         }
+         
+         const Util::CFScopeGuard<CFArrayRef> arr(CTFontManagerCreateFontDescriptorsFromURL(fontURL.Get()));
          if (!arr.Get()) {
             ::Error("FontCache::SelectSymbolFont", "CTFontManagerCreateFontDescriptorsFromURL failed");
-            delete [] fontFileName;
             return 0;
          }
 
          CTFontDescriptorRef fontDesc = (CTFontDescriptorRef)CFArrayGetValueAtIndex(arr.Get(), 0);
-         const CTFontGuard_t font(CTFontCreateWithFontDescriptor(fontDesc, fixedSize, 0), false);
+         
+         const CGAffineTransform shearMatrix = {1., 0., 0.26794, 1., 0., 0.};//Yes, these are hardcoded values, taken from TPDF class.
+         const CTFontGuard_t font(CTFontCreateWithFontDescriptorAndOptions(fontDesc, fixedSize,
+                                                                           fontIndex == 11 ? &CGAffineTransformIdentity :
+                                                                           &shearMatrix, kCTFontOptionsDefault), false);
          if (!font.Get()) {
             ::Error("FontCache::SelectSymbolFont", "CTFontCreateWithFontDescriptor failed");
-            delete [] fontFileName;
             return 0;
          }
 
-         delete [] fontFileName;
-
-         fFonts[11][fixedSize] = font;//This can throw.
+         fFonts[fontIndex][fixedSize] = font;//This can throw.
          return fSelectedFont = font.Get();
       } catch (const std::exception &) {//Bad alloc.
          //RAII destructors should do their work.
