@@ -1,4 +1,4 @@
-// @(#)root/graf:$Id: TLatex.cxx 43064 2012-02-21 15:07:04Z couet $
+// @(#)root/graf:$Id$
 // Author: Nicolas Brun, Olivier Couet, Oleksandr Grebenyuk
 
 /*************************************************************************
@@ -13,16 +13,13 @@
 #include "TROOT.h"
 #include "TClass.h"
 #include "TLatex.h"
+#include "TMathText.h"
 #include "TMath.h"
 #include "TVirtualPad.h"
 #include "TVirtualPS.h"
 
-#ifdef R__SUNCCBUG
-const Double_t kPI = 3.14159265358979323846;
-#else
 const Double_t kPI = TMath::Pi();
-#endif
-const Int_t kLatex      = BIT(10);
+const Int_t kLatex = BIT(10);
 
 ClassImp(TLatex)
 
@@ -48,6 +45,7 @@ It provides several functionalities:
 <li><a href="#L11"> Character Adjustement</a></li>
 <li><a href="#L12"> Italic and Boldface</a></li>
 <li><a href="#L13"> Examples</a></li>
+<li><a href="#L14"> Interface to TMathText</a></li>
 </ul>
 
 When the font precision (see <tt>TAttText</tt>) is low (0 or 1), TLatex is
@@ -63,10 +61,16 @@ which set relative size of subscripts and superscripts, and
 subscipts and superscripts.
 <p>Examples:
 End_Html
-   x^{2y} :    Begin_Latex x^{2y}    End_Latex x_{2y} :    Begin_Latex x^{2y}    End_Latex
+   x^{2y} :    Begin_Latex x^{2y}    End_Latex x_{2y} :    Begin_Latex x_{2y}    End_Latex
    x^{y^{2}} : Begin_Latex x^{y^{2}} End_Latex x^{y_{1}} : Begin_Latex x^{y_{1}} End_Latex
    x^{y}_{1} : Begin_Latex x^{y}_{1} End_Latex x_{1}^{y} : Begin_Latex x_{1}^{y} End_Latex
 
+Begin_Html
+The best way to put the subscipts and superscripts before the character and not
+after, is to use an empty character:
+End_Html
+   {}^{40}_{20}Ca : Begin_Latex {}^{40}_{20}Ca End_Latex
+
 Begin_Html
 The subscripts and superscripts operators apply not only on one character but
 on all the "normal text" preceding them. In the following example the second
@@ -297,6 +301,15 @@ Begin_Macro(source)
    return ex3;
 }
 End_Macro
+
+Begin_Html
+<a name="L14"></a><h3>Interface to TMathText</h3>
+The class <tt>TMathText</tt> is a TeX math formulae interpreter. It uses plain
+TeX syntax and uses "\" as control instead of "#". If a piece of text containing
+"\" is given to <tt>TLatex</tt> then <tt>TMathText</tt> is automatically invoked.
+Therefore, as histograms' titles, axis titles, labels etc ... are drawn using
+<tt>TLatex</tt>, the <tt>TMathText</tt> syntax can be used for them also.
+End_Html
 */
 
 
@@ -529,9 +542,9 @@ TLatexFormSize TLatex::Analyse(Double_t x, Double_t y, TextSpec_t spec, const Ch
    Int_t opCurlyCurly    = -1;   // Position of first }{
    Int_t opSquareCurly   = -1;   // Position of first ]{
    Int_t opCloseCurly    = -2;   // Position of first }
-   Int_t opColor         = -1;   // Position of first \color
-   Int_t opFont          = -1;   // Position of first \font
-   Int_t opScale         = -1;   // Position of first \scale
+   Int_t opColor         = -1;   // Position of first #color
+   Int_t opFont          = -1;   // Position of first #font
+   Int_t opScale         = -1;   // Position of first #scale
    Int_t opGreek         = -1;   // Position of a Greek letter
    Int_t opSpec          = -1;   // position of a special character
    Int_t opAbove         = -1;   // position of a vector/overline
@@ -545,8 +558,9 @@ TLatexFormSize TLatex::Analyse(Double_t x, Double_t y, TextSpec_t spec, const Ch
    Int_t opOdot          = 0;    // position of #odot
    Int_t opHbar          = 0;    // position of #hbar
    Int_t opMp            = 0;    // position of #mp
+   Int_t opBackslash     = 0;    // position of #backslash
    Int_t opParallel      = 0;    // position of #parallel
-   Int_t opSplitLine     = -1;   // Position of first \splitline
+   Int_t opSplitLine     = -1;   // Position of first #splitline
    Int_t opKern          = -1;   // Position of first #kern
    Int_t opLower         = -1;   // Position of first #lower
    Int_t opBf            = -1;   // Position of first #bf
@@ -647,6 +661,15 @@ TLatexFormSize TLatex::Analyse(Double_t x, Double_t y, TextSpec_t spec, const Ch
                continue;
             }
          }
+         if (length>i+9) {
+            Char_t buf[10];
+            strncpy(buf,&text[i+1],9);
+            if (!opBackslash && strncmp(buf,"backslash",9)==0) {
+               opBackslash=1; opFound = kTRUE;
+               if (i>0 && opCloseCurly==-2) opCloseCurly=i-1;
+               continue;
+            }
+         }
          if (length>i+8) {
             Char_t buf[9];
             strncpy(buf,&text[i+1],8);
@@ -1102,6 +1125,26 @@ TLatexFormSize TLatex::Analyse(Double_t x, Double_t y, TextSpec_t spec, const Ch
       }
       result = fs1;
    }
+   else if (opBackslash) {
+      Double_t square = GetHeight()*spec.fSize/2;
+      if (!fShow) {
+         fs1 = Anal1(spec,text+10,length-10);
+      } else {
+         fs1 = Analyse(x+square,y,spec,text+10,length-10);
+         TText bs;
+         bs.SetTextFont(GetTextFont());
+         bs.SetTextColor(fTextColor);
+         bs.SetTextSize(spec.fSize);
+         bs.SetTextAngle(fTextAngle);
+         Double_t xOrigin = (Double_t)gPad->XtoAbsPixel(fX);
+         Double_t yOrigin = (Double_t)gPad->YtoAbsPixel(fY);
+         Double_t angle   = kPI*spec.fAngle/180.;
+         Double_t xx = gPad->AbsPixeltoX(Int_t((x-xOrigin)*TMath::Cos(angle)+(y-yOrigin)*TMath::Sin(angle)+xOrigin));
+         Double_t yy = gPad->AbsPixeltoY(Int_t((x-xOrigin)*TMath::Sin(-angle)+(y-yOrigin)*TMath::Cos(angle)+yOrigin));
+         bs.PaintText(xx,yy,"\\");
+      }
+      result = fs1 + TLatexFormSize(square,square,0);
+   }
    else if (opParallel) {
       Double_t square = GetHeight()*spec.fSize/1.4;
       if (!fShow) {
@@ -1270,10 +1313,10 @@ TLatexFormSize TLatex::Analyse(Double_t x, Double_t y, TextSpec_t spec, const Ch
                tilde.SetTextAngle(fTextAngle);
                tilde.PaintText(xx,yy,"~");
                if (saveps) {
-                  y2 -= 4*sub;
+                  gVirtualPS = saveps;
+                  if (!strstr(gVirtualPS->GetTitle(),"IMG")) y2 -= 4*sub;
                   xx  = gPad->AbsPixeltoX(Int_t((x2-xOrigin)*cosang+(y2-yOrigin)*sinang+xOrigin));
                   yy  = gPad->AbsPixeltoY(Int_t((x2-xOrigin)*-sinang+(y2-yOrigin)*cosang+yOrigin));
-                  gVirtualPS = saveps;
                   gVirtualPS->SetTextAlign(22);
                   gVirtualPS->Text(xx, yy, "~");
                }
@@ -1844,6 +1887,14 @@ void TLatex::PaintLatex(Double_t x, Double_t y, Double_t angle, Double_t size, c
       return;
    }
 
+   // Paint the text using TMathText if contains a "\"
+   if (strstr(text1,"\\")) {
+      TMathText tm;
+      tm.SetTextAlign(GetTextAlign());
+      tm.PaintMathText(x, y, angle, size, text1);
+      return;
+   }
+
    TString newText = text1;
    if( newText.Length() == 0) return;
 
@@ -1864,8 +1915,8 @@ void TLatex::PaintLatex(Double_t x, Double_t y, Double_t angle, Double_t size, c
 
    fError = 0 ;
    if (CheckLatexSyntax(newText)) {
-      cout<<"\n*ERROR<TLatex>: "<<fError<<endl;
-      cout<<"==> "<<text1<<endl;
+      std::cout<<"\n*ERROR<TLatex>: "<<fError<<std::endl;
+      std::cout<<"==> "<<text1<<std::endl;
       return ;
    }
    fError = 0 ;
@@ -1895,8 +1946,8 @@ void TLatex::PaintLatex(Double_t x, Double_t y, Double_t angle, Double_t size, c
    Short_t valign = fTextAlign - 10*halign;
    TextSpec_t newSpec = spec;
    if (fError != 0) {
-      cout<<"*ERROR<TLatex>: "<<fError<<endl;
-      cout<<"==> "<<text<<endl;
+      std::cout<<"*ERROR<TLatex>: "<<fError<<std::endl;
+      std::cout<<"==> "<<text<<std::endl;
    } else {
       fShow = kTRUE;
       newSpec.fSize = size;
@@ -2201,8 +2252,8 @@ Double_t TLatex::GetXsize()
    if( newText.Length() == 0) return 0;
    fError = 0 ;
    if (CheckLatexSyntax(newText)) {
-      cout<<"\n*ERROR<TLatex>: "<<fError<<endl;
-      cout<<"==> "<<GetTitle()<<endl;
+      std::cout<<"\n*ERROR<TLatex>: "<<fError<<std::endl;
+      std::cout<<"==> "<<GetTitle()<<std::endl;
       return 0;
    }
    fError = 0 ;
@@ -2226,8 +2277,8 @@ void TLatex::GetBoundingBox(UInt_t &w, UInt_t &h, Bool_t angle)
    if( newText.Length() == 0) return;
    fError = 0 ;
    if (CheckLatexSyntax(newText)) {
-      cout<<"\n*ERROR<TLatex>: "<<fError<<endl;
-      cout<<"==> "<<GetTitle()<<endl;
+      std::cout<<"\n*ERROR<TLatex>: "<<fError<<std::endl;
+      std::cout<<"==> "<<GetTitle()<<std::endl;
       return;
    }
    fError = 0 ;
@@ -2275,8 +2326,8 @@ Double_t TLatex::GetYsize()
    if( newText.Length() == 0) return 0;
    fError = 0 ;
    if (CheckLatexSyntax(newText)) {
-      cout<<"\n*ERROR<TLatex>: "<<fError<<endl;
-      cout<<"==> "<<GetTitle()<<endl;
+      std::cout<<"\n*ERROR<TLatex>: "<<fError<<std::endl;
+      std::cout<<"==> "<<GetTitle()<<std::endl;
       return 0;
    }
    fError = 0 ;
@@ -2323,25 +2374,29 @@ void TLatex::Savefs(TLatexFormSize *fs)
 
 
 //______________________________________________________________________________
-void TLatex::SavePrimitive(ostream &out, Option_t * /*= ""*/)
+void TLatex::SavePrimitive(std::ostream &out, Option_t * /*= ""*/)
 {
    // Save primitive as a C++ statement(s) on output stream out
 
    char quote = '"';
+
    if (gROOT->ClassSaved(TLatex::Class())) {
       out<<"   ";
    } else {
       out<<"   TLatex *";
    }
+
    TString s = GetTitle();
+
+   s.ReplaceAll("\\","\\\\");
    s.ReplaceAll("\"","\\\"");
-   out<<"   tex = new TLatex("<<fX<<","<<fY<<","<<quote<<s.Data()<<quote<<");"<<endl;
-   if (TestBit(kTextNDC)) out<<"tex->SetNDC();"<<endl;
+   out<<"   tex = new TLatex("<<fX<<","<<fY<<","<<quote<<s.Data()<<quote<<");"<<std::endl;
+   if (TestBit(kTextNDC)) out<<"tex->SetNDC();"<<std::endl;
 
    SaveTextAttributes(out,"tex",11,0,1,62,0.05);
    SaveLineAttributes(out,"tex",1,1,1);
 
-   out<<"   tex->Draw();"<<endl;
+   out<<"   tex->Draw();"<<std::endl;
 }
 
 
