@@ -1,4 +1,4 @@
-// @(#)root/graf:$Id: TLegend.cxx 44072 2012-05-02 15:10:27Z couet $
+// @(#)root/graf:$Id$
 // Author: Matthew.Adam.Dobbs   06/09/99
 
 /*************************************************************************
@@ -299,6 +299,11 @@ TLegendEntry *TLegend::AddEntry(const char *name, const char *label, Option_t *o
    </ul>
    End_Html */
 
+   if (!gPad) {
+      Error("AddEntry", "need to create a canvas first");
+      return 0;
+   }
+
    TObject *obj = gPad->FindObject(name);
 
    // If the object "name" has not been found, the following code tries to
@@ -312,12 +317,12 @@ TLegendEntry *TLegend::AddEntry(const char *name, const char *label, Option_t *o
             if ( o->InheritsFrom(TMultiGraph::Class() ) ) {
                TList * grlist = ((TMultiGraph *)o)->GetListOfGraphs();
                obj = grlist->FindObject(name);
-               if (obj) continue;
+               if (obj) break;
             }
             if ( o->InheritsFrom(THStack::Class() ) ) {
                TList * hlist = ((THStack *)o)->GetHists();
                obj = hlist->FindObject(name);
-               if (obj) continue;
+               if (obj) break;
             }
          }
       }
@@ -444,6 +449,11 @@ TLegendEntry *TLegend::GetEntry() const
    This method is mostly a tool for other methods inside this class.
    End_Html */
 
+   if (!gPad) {
+      Error("GetEntry", "need to create a canvas first");
+      return 0;
+   }
+   
    Int_t nRows = GetNRows();
    if ( nRows == 0 ) return 0;
 
@@ -502,6 +512,11 @@ void TLegend::InsertEntry( const char* objectName, const char* label, Option_t*
    Add a new entry before the entry at the mouse position.
    End_Html */
 
+   if (!gPad) {
+      Error("InsertEntry", "need to create a canvas first");
+      return;
+   }
+   
    TLegendEntry* beforeEntry = GetEntry();   // get entry pointed by the mouse
    TObject *obj = gPad->FindObject( objectName );
 
