@@ -1868,15 +1868,10 @@ gdk_window_set_icon(GdkWindow * window,
    icon_info.hbmMask = hbitmap_mask;
    icon_info.hbmColor = (HBITMAP) GDK_DRAWABLE_XID(pixmap);
 
-   hicon = (HICON)GetClassLong((HWND)GDK_DRAWABLE_XID(window), GCL_HICONSM);
-   if (hicon) DestroyIcon(hicon);
-   hicon = (HICON)GetClassLong((HWND)GDK_DRAWABLE_XID(window), GCL_HICON);
-   if (hicon) DestroyIcon(hicon);
-
    hicon = CreateIconIndirect(&icon_info);
 
-   SetClassLong((HWND)GDK_DRAWABLE_XID(window), GCL_HICONSM, (LPARAM)CopyIcon(hicon));
-   SetClassLong((HWND)GDK_DRAWABLE_XID(window), GCL_HICON, (LPARAM)CopyIcon(hicon));
+   SendMessage((HWND)GDK_DRAWABLE_XID(window), WM_SETICON, ICON_BIG, (LPARAM)CopyIcon(hicon));
+   SendMessage((HWND)GDK_DRAWABLE_XID(window), WM_SETICON, ICON_SMALL, (LPARAM)CopyIcon(hicon));
 
    DestroyIcon(hicon);
    DeleteObject(hbitmap_mask);
