@@ -1,4 +1,4 @@
-// @(#)root/gui:$Id: TGFileDialog.cxx 43842 2012-04-19 13:09:11Z bellenot $
+// @(#)root/gui:$Id$
 // Author: Fons Rademakers   20/01/98
 
 /*************************************************************************
@@ -321,7 +321,7 @@ TGFileDialog::TGFileDialog(const TGWindow *p, const TGWindow *main,
    const char *wname = (dlg_type == kFDSave) ? "Save As..." : "Open";
    SetWindowName(wname);
    SetIconName(wname);
-   SetClassHints("FileDialog", "FileDialog");
+   SetClassHints("ROOT", "FileDialog");
 
    SetMWMHints(kMWMDecorAll | kMWMDecorResizeH  | kMWMDecorMaximize |
                               kMWMDecorMinimize | kMWMDecorMenu,
