@@ -1,4 +1,4 @@
-// @(#)root/gui:$Id: TGListTree.cxx 41891 2011-11-10 22:46:31Z pcanal $
+// @(#)root/gui:$Id$
 // Author: Fons Rademakers   25/02/98
 
 /*************************************************************************
@@ -57,6 +57,7 @@
 #include "TGDNDManager.h"
 #include "TBufferFile.h"
 #include "Riostream.h"
+#include "RConfigure.h"
 
 Pixel_t          TGListTree::fgGrayPixel = 0;
 const TGFont    *TGListTree::fgDefaultFont = 0;
@@ -1479,6 +1480,7 @@ void TGListTree::DrawItem(Handle_t id, TGListTreeItem *item, Int_t x, Int_t y,
 
    // Compute the height of this line
    height = FontHeight();
+
    xline = 0;
    xpic1 = x;
    xtext = x + fHspacing + (Int_t)item->GetPicWidth();
@@ -1502,6 +1504,9 @@ void TGListTree::DrawItem(Handle_t id, TGListTreeItem *item, Int_t x, Int_t y,
          height = pic1->GetHeight();
          ypic1 = y;
       } else {
+#ifdef R__HAS_COCOA
+         if (!pic2)//DO NOT MODIFY ytext, it WAS ADJUSTED already!
+#endif
          ytext = y;
          ypic1 = y + (Int_t)((height - pic1->GetHeight()) >> 1);
       }
@@ -1587,8 +1592,13 @@ void TGListTree::DrawOutline(Handle_t id, TGListTreeItem *item, Pixel_t col,
    }
    else
       gVirtualX->SetForeground(fDrawGC, col);
+
+#ifdef R__HAS_COCOA
+   gVirtualX->DrawRectangle(id, fDrawGC, 1, item->fY - pos.fY, dim.fWidth-2, item->fHeight + 1);
+#else
    gVirtualX->DrawRectangle(id, fDrawGC, 1, item->fYtext-pos.fY-2, 
                             dim.fWidth-3, FontHeight()+4);
+#endif
    gVirtualX->SetForeground(fDrawGC, fgBlackPixel);
 }
 
@@ -1603,8 +1613,13 @@ void TGListTree::DrawActive(Handle_t id, TGListTreeItem *item)
 
    width = dim.fWidth-2;
    gVirtualX->SetForeground(fDrawGC, item->GetActiveColor());
+
+#ifdef R__HAS_COCOA
+   gVirtualX->FillRectangle(id, fDrawGC, 1, item->fY - pos.fY, width, item->fHeight + 1);
+#else
    gVirtualX->FillRectangle(id, fDrawGC, 1, item->fYtext-pos.fY-1, width, 
                             FontHeight()+3);
+#endif
    gVirtualX->SetForeground(fDrawGC, fgBlackPixel);
    gVirtualX->DrawString(id, fActiveGC, item->fXtext, 
                          item->fYtext - pos.fY + FontAscent(),
@@ -1623,7 +1638,7 @@ void TGListTree::DrawItemName(Handle_t id, TGListTreeItem *item)
       DrawActive(id, item);
    }
    else { // if (!item->IsActive() && (item != fSelected)) {
-      gVirtualX->FillRectangle(id, fHighlightGC, item->fXtext, 
+      gVirtualX->FillRectangle(id, fHighlightGC, item->fXtext,
                        item->fYtext-pos.fY, dim.fWidth-item->fXtext-2,
                        FontHeight()+1);
       gVirtualX->DrawString(id, fDrawGC,
@@ -2578,13 +2593,13 @@ const TGPicture *TGListTree::GetUncheckedPic()
 }
 
 //______________________________________________________________________________
-void TGListTree::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
+void TGListTree::SavePrimitive(std::ostream &out, Option_t *option /*= ""*/)
 {
    // Save a list tree widget as a C++ statements on output stream out.
 
    if (fBackground != GetWhitePixel()) SaveUserColor(out, option);
 
-   out << endl << "   // list tree" << endl;
+   out << std::endl << "   // list tree" << std::endl;
    out << "   TGListTree *";
 
    if ((fParent->GetParent())->InheritsFrom(TGCanvas::Class())) {
@@ -2596,37 +2611,37 @@ void TGListTree::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
 
    if (fBackground == GetWhitePixel()) {
       if (GetOptions() == kSunkenFrame) {
-         out <<");" << endl;
+         out <<");" << std::endl;
       } else {
-         out << "," << GetOptionString() <<");" << endl;
+         out << "," << GetOptionString() <<");" << std::endl;
       }
    } else {
-      out << "," << GetOptionString() << ",ucolor);" << endl;
+      out << "," << GetOptionString() << ",ucolor);" << std::endl;
    }
    if (option && strstr(option, "keep_names"))
-      out << "   " << GetName() << "->SetName(\"" << GetName() << "\");" << endl;
+      out << "   " << GetName() << "->SetName(\"" << GetName() << "\");" << std::endl;
 
-   out << endl;
+   out << std::endl;
 
    static Int_t n = 0;
 
    TGListTreeItem *current;
    current = GetFirstItem();
 
-   out << "   const TGPicture *popen;       //used for list tree items" << endl;
-   out << "   const TGPicture *pclose;      //used for list tree items" << endl;
-   out << endl;
+   out << "   const TGPicture *popen;       //used for list tree items" << std::endl;
+   out << "   const TGPicture *pclose;      //used for list tree items" << std::endl;
+   out << std::endl;
 
    while (current) {
       out << "   TGListTreeItem *item" << n << " = " << GetName() << "->AddItem(";
       current->SavePrimitive(out, TString::Format("%d",n), n);
       if (current->IsOpen())
-         out << "   " << GetName() << "->OpenItem(item" << n << ");" << endl;
+         out << "   " << GetName() << "->OpenItem(item" << n << ");" << std::endl;
       else
-         out << "   " << GetName() << "->CloseItem(item" << n << ");" << endl;
+         out << "   " << GetName() << "->CloseItem(item" << n << ");" << std::endl;
 
       if (current == fSelected)
-         out << "   " << GetName() << "->SetSelected(item" << n << ");" << endl;
+         out << "   " << GetName() << "->SetSelected(item" << n << ");" << std::endl;
 
       n++;
       if (current->fFirstchild) {
@@ -2635,11 +2650,11 @@ void TGListTree::SavePrimitive(ostream &out, Option_t *option /*= ""*/)
       current = current->fNextsibling;
    }
 
-   out << endl;
+   out << std::endl;
 }
 
 //______________________________________________________________________________
-void TGListTree::SaveChildren(ostream &out, TGListTreeItem *item, Int_t &n)
+void TGListTree::SaveChildren(std::ostream &out, TGListTreeItem *item, Int_t &n)
 {
    // Save child items as a C++ statements on output stream out.
 
@@ -2656,7 +2671,7 @@ void TGListTree::SaveChildren(ostream &out, TGListTreeItem *item, Int_t &n)
 }
 
 //______________________________________________________________________________
-void TGListTreeItemStd::SavePrimitive(ostream &out, Option_t *option, Int_t n)
+void TGListTreeItemStd::SavePrimitive(std::ostream &out, Option_t *option, Int_t n)
 {
    // Save a list tree item attributes as a C++ statements on output stream.
 
@@ -2679,50 +2694,50 @@ void TGListTreeItemStd::SavePrimitive(ostream &out, Option_t *option, Int_t n)
    text.ReplaceAll('\\', "\\\\");
    text.ReplaceAll("\"", "\\\"");
    out << quote << text << quote;
-   out << ");" << endl;
+   out << ");" << std::endl;
 
    if (oldopen != fOpenPic) {
       oldopen = fOpenPic;
       out << "   popen = gClient->GetPicture(" << quote
           << gSystem->ExpandPathName(gSystem->UnixPathName(fOpenPic->GetName()))
-          << quote << ");" << endl;
+          << quote << ");" << std::endl;
    }
    if (oldclose != fClosedPic) {
       oldclose = fClosedPic;
       out << "   pclose = gClient->GetPicture(" << quote
           << gSystem->ExpandPathName(gSystem->UnixPathName(fClosedPic->GetName()))
-          << quote << ");" << endl;
+          << quote << ");" << std::endl;
    }
-   out << "   item" << s.Data() << "->SetPictures(popen, pclose);" << endl;
+   out << "   item" << s.Data() << "->SetPictures(popen, pclose);" << std::endl;
    if (HasCheckBox()) {
       if (fCheckedPic && makecheck) {
-         out << "   const TGPicture *pcheck;        //used for checked items" << endl;
+         out << "   const TGPicture *pcheck;        //used for checked items" << std::endl;
          makecheck = kFALSE;
       }
       if (fUncheckedPic && makeuncheck) {
-         out << "   const TGPicture *puncheck;      //used for unchecked items" << endl;
+         out << "   const TGPicture *puncheck;      //used for unchecked items" << std::endl;
          makeuncheck = kFALSE;
       }
-      out << "   item" << s.Data() << "->CheckItem();" << endl;
+      out << "   item" << s.Data() << "->CheckItem();" << std::endl;
       if (fCheckedPic && oldcheck != fCheckedPic) {
          oldcheck = fCheckedPic;
          out << "   pcheck = gClient->GetPicture(" << quote
              << gSystem->ExpandPathName(gSystem->UnixPathName(fCheckedPic->GetName()))
-             << quote << ");" << endl;
+             << quote << ");" << std::endl;
       }
       if (fUncheckedPic && olduncheck != fUncheckedPic) {
          olduncheck = fUncheckedPic;
          out << "   puncheck = gClient->GetPicture(" << quote
              << gSystem->ExpandPathName(gSystem->UnixPathName(fUncheckedPic->GetName()))
-             << quote << ");" << endl;
+             << quote << ");" << std::endl;
       }
-      out << "   item" << s.Data() << "->SetCheckBoxPictures(pcheck, puncheck);" << endl;
-      out << "   item" << s.Data() << "->SetCheckBox(kTRUE);" << endl;
+      out << "   item" << s.Data() << "->SetCheckBoxPictures(pcheck, puncheck);" << std::endl;
+      out << "   item" << s.Data() << "->SetCheckBox(kTRUE);" << std::endl;
    }
    if (fHasColor) {
       if (oldcolor != fColor) {
          oldcolor = fColor;
-         out << "   item" << s.Data() << "->SetColor(" << fColor << ");" << endl;
+         out << "   item" << s.Data() << "->SetColor(" << fColor << ");" << std::endl;
       }
    }
    if (fTipText.Length() > 0) {
@@ -2731,7 +2746,7 @@ void TGListTreeItemStd::SavePrimitive(ostream &out, Option_t *option, Int_t n)
       tiptext.ReplaceAll("\n", "\\n");
       tiptext.ReplaceAll("\"", "\\\"");
       out << "   item" << s.Data() << "->SetTipText(" << quote
-          << tiptext << quote << ");" << endl;
+          << tiptext << quote << ");" << std::endl;
    }
 
 }
