@@ -1,4 +1,4 @@
-// @(#)root/gui:$Id: TRootCanvas.cxx 44173 2012-05-08 11:59:51Z bellenot $
+// @(#)root/gui:$Id$
 // Author: Fons Rademakers   15/01/98
 
 /*************************************************************************
@@ -578,7 +578,7 @@ void TRootCanvas::CreateCanvas(const char *name)
    SetWindowName(name);
    SetIconName(name);
    fIconPic = SetIconPixmap("macro_s.xpm");
-   SetClassHints("Canvas", "Canvas");
+   SetClassHints("ROOT", "Canvas");
 
    SetEditDisabled(kEditDisable);
    MapSubwindows();
