@@ -1,4 +1,4 @@
-// @(#)root/hist:$Id: TH1.h 44121 2012-05-04 14:44:59Z moneta $
+// @(#)root/hist:$Id$
 // Author: Rene Brun   26/12/94
 
 /*************************************************************************
@@ -289,6 +289,8 @@ public:
    virtual Int_t    GetQuantiles(Int_t nprobSum, Double_t *q, const Double_t *probSum=0);
    virtual Double_t GetRandom() const;
    virtual void     GetStats(Double_t *stats) const;
+           Double_t GetStdDev(Int_t axis=1) const { return GetRMS(axis); }                  
+           Double_t GetStdDevError(Int_t axis=1) const { return GetRMSError(axis); }
    virtual Double_t GetSumOfWeights() const;
    virtual TArrayD *GetSumw2() {return &fSumw2;}
    virtual const TArrayD *GetSumw2() const {return &fSumw2;}
@@ -384,7 +386,7 @@ public:
    virtual void     Smooth(Int_t ntimes=1, Option_t *option=""); // *MENU*
    static  void     SmoothArray(Int_t NN, Double_t *XX, Int_t ntimes=1);
    static  void     StatOverflows(Bool_t flag=kTRUE);
-   virtual void     Sumw2();
+   virtual void     Sumw2(Bool_t flag = kTRUE);
    void             UseCurrentStyle();
    static  TH1     *TransformHisto(TVirtualFFT *fft, TH1* h_output,  Option_t *option);
 
