@@ -1,4 +1,4 @@
-// @(#)root/hist:$Id: TGraph.cxx 44216 2012-05-10 12:51:36Z moneta $
+// @(#)root/hist:$Id$
 // Author: Rene Brun, Olivier Couet   12/12/94
 
 /*************************************************************************
@@ -197,10 +197,30 @@ TGraph& TGraph::operator=(const TGraph &gr)
 
       fNpoints = gr.fNpoints;
       fMaxSize = gr.fMaxSize;
+
+      // delete list of functions and their contents before copying it
+      if (fFunctions) { 
+         // delete previous lists of functions
+         if (!fFunctions->IsEmpty()) {
+            fFunctions->SetBit(kInvalidObject);
+            // use TList::Remove to take into account the case the same object is 
+            // added multiple times in the list
+            TObject *obj;
+            while ((obj  = fFunctions->First())) {
+               while (fFunctions->Remove(obj)) { }
+               delete obj;
+            }
+         }
+         delete fFunctions;
+      }
+
       if (gr.fFunctions) fFunctions = (TList*)gr.fFunctions->Clone();
       else fFunctions = new TList;
+
+      if (fHistogram) delete fHistogram; 
       if (gr.fHistogram) fHistogram = new TH1F(*(gr.fHistogram));
       else fHistogram = 0;
+
       fMinimum = gr.fMinimum;
       fMaximum = gr.fMaximum;
       if (fX) delete [] fX;
@@ -1098,7 +1118,7 @@ TFitResultPtr TGraph::Fit(TF1 *f1, Option_t *option, Option_t *goption, Axis_t r
    //             = "C" In case of linear fitting, do not calculate the chisquare
    //                    (saves time)
    //             = "F" If fitting a polN, use the minuit fitter
-   //             = "EX0" When fitting a TGraphErrors do not consider errors in the coordinate
+   //             = "EX0" When fitting a TGraphErrors or TGraphAsymErrors do not consider errors in the coordinate
    //             = "ROB" In case of linear fitting, compute the LTS regression
    //                     coefficients (robust (resistant) regression), using
    //                     the default fraction of good points
