@@ -1,4 +1,4 @@
-// @(#)root/hist:$Id: TProfile.cxx 44273 2012-05-16 13:04:57Z moneta $
+// @(#)root/hist:$Id$
 // Author: Rene Brun   29/09/95
 
 /*************************************************************************
@@ -424,20 +424,27 @@ void TProfile::Copy(TObject &obj) const
 //*-*-*-*-*-*-*-*Copy a Profile histogram to a new profile histogram*-*-*-*-*
 //*-*            ===================================================
 
-   TH1D::Copy(((TProfile&)obj));
-   fBinEntries.Copy(((TProfile&)obj).fBinEntries);
-   fBinSumw2.Copy(((TProfile&)obj).fBinSumw2);
-   for (int bin=0;bin<fNcells;bin++) {
-      ((TProfile&)obj).fArray[bin]        = fArray[bin];
-      ((TProfile&)obj).fSumw2.fArray[bin] = fSumw2.fArray[bin];
+   try { 
+      TProfile & pobj = dynamic_cast<TProfile&>(obj);
+      TH1D::Copy(pobj);
+      fBinEntries.Copy(pobj.fBinEntries);
+      fBinSumw2.Copy(pobj.fBinSumw2);
+      for (int bin=0;bin<fNcells;bin++) {
+         pobj.fArray[bin]        = fArray[bin];
+         pobj.fSumw2.fArray[bin] = fSumw2.fArray[bin];
+      }
+      
+      pobj.fYmin = fYmin;
+      pobj.fYmax = fYmax;
+      pobj.fScaling   = fScaling;
+      pobj.fErrorMode = fErrorMode;
+      pobj.fTsumwy    = fTsumwy;
+      pobj.fTsumwy2   = fTsumwy2;
+      
+   } catch(...) {
+      Fatal("Copy","Cannot copy a TProfile in a %s",obj.IsA()->GetName());
    }
-
-   ((TProfile&)obj).fYmin = fYmin;
-   ((TProfile&)obj).fYmax = fYmax;
-   ((TProfile&)obj).fScaling   = fScaling;
-   ((TProfile&)obj).fErrorMode = fErrorMode;
-   ((TProfile&)obj).fTsumwy    = fTsumwy;
-   ((TProfile&)obj).fTsumwy2   = fTsumwy2;
+      
 }
 
 
@@ -1772,15 +1779,16 @@ void TProfile::Streamer(TBuffer &R__b)
    }
 }
 //______________________________________________________________________________
-void TProfile::Sumw2()
+void TProfile::Sumw2(Bool_t flag)
 {
-   // Create structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
+   // Create/delete structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
    //   This is needed to compute  the correct statistical quantities  
    //    of a profile filled with weights 
    //  
    //
    //  This function is automatically called when the histogram is created
    //  if the static function TH1::SetDefaultSumw2 has been called before.
+   // If flag is false the structure is deleted
 
-   TProfileHelper::Sumw2(this);
+   TProfileHelper::Sumw2(this, flag);
 }
