@@ -1,4 +1,4 @@
-// @(#)root/hist:$Id: TProfile2D.cxx 44273 2012-05-16 13:04:57Z moneta $
+// @(#)root/hist:$Id$
 // Author: Rene Brun   16/04/2000
 
 /*************************************************************************
@@ -370,19 +370,27 @@ void TProfile2D::Copy(TObject &obj) const
 //*-*-*-*-*-*-*-*Copy a Profile2D histogram to a new profile2D histogram*-*-*-*
 //*-*            =======================================================
 
-   TH2D::Copy(((TProfile2D&)obj));
-   fBinEntries.Copy(((TProfile2D&)obj).fBinEntries);
-   fBinSumw2.Copy(((TProfile2D&)obj).fBinSumw2);
-   for (int bin=0;bin<fNcells;bin++) {
-      ((TProfile2D&)obj).fArray[bin]        = fArray[bin];
-      ((TProfile2D&)obj).fSumw2.fArray[bin] = fSumw2.fArray[bin];
+   try { 
+      TProfile2D & pobj = dynamic_cast<TProfile2D&>(obj);
+
+      TH2D::Copy(pobj);
+      fBinEntries.Copy(pobj.fBinEntries);
+      fBinSumw2.Copy(pobj.fBinSumw2);
+      for (int bin=0;bin<fNcells;bin++) {
+         pobj.fArray[bin]        = fArray[bin];
+         pobj.fSumw2.fArray[bin] = fSumw2.fArray[bin];
+      }
+      pobj.fZmin = fZmin;
+      pobj.fZmax = fZmax;
+      pobj.fScaling   = fScaling;
+      pobj.fErrorMode = fErrorMode;
+      pobj.fTsumwz    = fTsumwz;
+      pobj.fTsumwz2   = fTsumwz2;
+      
+   } catch(...) {
+      Fatal("Copy","Cannot copy a TProfile2D in a %s",obj.IsA()->GetName());
    }
-   ((TProfile2D&)obj).fZmin = fZmin;
-   ((TProfile2D&)obj).fZmax = fZmax;
-   ((TProfile2D&)obj).fScaling   = fScaling;
-   ((TProfile2D&)obj).fErrorMode = fErrorMode;
-   ((TProfile2D&)obj).fTsumwz    = fTsumwz;
-   ((TProfile2D&)obj).fTsumwz2   = fTsumwz2;
+
 }
 
 
@@ -1834,15 +1842,16 @@ void TProfile2D::Streamer(TBuffer &R__b)
 }
 
 //______________________________________________________________________________
-void TProfile2D::Sumw2()
+void TProfile2D::Sumw2(Bool_t flag)
 {
-   // Create structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
+   // Create/Delete structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
    //   This is needed to compute  the correct statistical quantities  
    //    of a profile filled with weights 
    //  
    //
    //  This function is automatically called when the histogram is created
    //  if the static function TH1::SetDefaultSumw2 has been called before.
+   //  If flag is false the structure is deleted 
 
-   TProfileHelper::Sumw2(this);
+   TProfileHelper::Sumw2(this, flag);
 }
