@@ -1,4 +1,4 @@
-// @(#)root/hist:$Id: TProfile3D.cxx 44273 2012-05-16 13:04:57Z moneta $
+// @(#)root/hist:$Id$
 // Author: Rene Brun   17/05/2006
 
 /*************************************************************************
@@ -343,21 +343,27 @@ void TProfile3D::Copy(TObject &obj) const
 //*-*-*-*-*-*-*-*Copy a Profile3D histogram to a new profile2D histogram*-*-*-*
 //*-*            =======================================================
 
-   TH3D::Copy(((TProfile3D&)obj));
-   fBinEntries.Copy(((TProfile3D&)obj).fBinEntries);
-   fBinSumw2.Copy(((TProfile3D&)obj).fBinSumw2);
-   for (int bin=0;bin<fNcells;bin++) {
-      ((TProfile3D&)obj).fArray[bin]        = fArray[bin];
-      ((TProfile3D&)obj).fSumw2.fArray[bin] = fSumw2.fArray[bin];
-   }
-   ((TProfile3D&)obj).fTmin = fTmin;
-   ((TProfile3D&)obj).fTmax = fTmax;
-   ((TProfile3D&)obj).fScaling = fScaling;
-   ((TProfile3D&)obj).fErrorMode = fErrorMode;
-   ((TProfile3D&)obj).fTsumwt  = fTsumwt;
-   ((TProfile3D&)obj).fTsumwt2 = fTsumwt2;
-}
+   try { 
+      TProfile3D & pobj = dynamic_cast<TProfile3D&>(obj);
 
+      TH3D::Copy(pobj);
+      fBinEntries.Copy(pobj.fBinEntries);
+      fBinSumw2.Copy(pobj.fBinSumw2);
+      for (int bin=0;bin<fNcells;bin++) {
+         pobj.fArray[bin]        = fArray[bin];
+         pobj.fSumw2.fArray[bin] = fSumw2.fArray[bin];
+      }
+      pobj.fTmin = fTmin;
+      pobj.fTmax = fTmax;
+      pobj.fScaling = fScaling;
+      pobj.fErrorMode = fErrorMode;
+      pobj.fTsumwt  = fTsumwt;
+      pobj.fTsumwt2 = fTsumwt2;
+
+   } catch(...) {
+      Fatal("Copy","Cannot copy a TProfile3D in a %s",obj.IsA()->GetName());
+   }
+}
 
 //______________________________________________________________________________
 Bool_t TProfile3D::Divide(TF1 *, Double_t )
@@ -1314,15 +1320,16 @@ void TProfile3D::SetErrorOption(Option_t *option)
 }
 
 //______________________________________________________________________________
-void TProfile3D::Sumw2()
+void TProfile3D::Sumw2(Bool_t flag)
 {
-   // Create structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
+   // Create/Delete structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
    //   This is needed to compute  the correct statistical quantities  
    //    of a profile filled with weights 
    //  
    //
    //  This function is automatically called when the histogram is created
    //  if the static function TH1::SetDefaultSumw2 has been called before.
+   //  If flag = false the structure is deleted
 
-   TProfileHelper::Sumw2(this);
+   TProfileHelper::Sumw2(this, flag);
 }
