@@ -1,4 +1,4 @@
-// @(#)root/hist:$Id: TProfileHelper.h 43761 2012-04-16 13:36:17Z moneta $
+// @(#)root/hist:$Id$
 // Author: David Gonzalez Maline   18/01/2008
 
 /*************************************************************************
@@ -46,7 +46,7 @@ public:
    static void Scale(T* p, Double_t c1, Option_t * option);
 
    template <typename T> 
-   static void Sumw2(T* p);
+   static void Sumw2(T* p, Bool_t flag );
 
    template <typename T>
    static void LabelsDeflate(T* p, Option_t *);
@@ -171,6 +171,7 @@ Long64_t TProfileHelper::Merge(T* p, TCollection *li) {
    Bool_t initialLimitsFound = kFALSE;
    Bool_t allSameLimits = kTRUE;
    Bool_t allHaveLimits = kTRUE;
+   Bool_t firstNonEmptyHist = kTRUE;
 
    TIter next(&inlist);
    T* h = p;
@@ -185,6 +186,24 @@ Long64_t TProfileHelper::Merge(T* p, TCollection *li) {
 
       if (hasLimits) {
          h->BufferEmpty();
+
+         // this is done in case the first histograms are empty and 
+         // the histogram have different limits
+         if (firstNonEmptyHist ) { 
+            // set axis limits in the case the first histogram was empty
+            if (h != p ) { 
+               if (!p->SameLimitsAndNBins(p->fXaxis, *(h->GetXaxis())) ) 
+                  p->fXaxis.Set(h->GetXaxis()->GetNbins(), h->GetXaxis()->GetXmin(),h->GetXaxis()->GetXmax());
+               if (!p->SameLimitsAndNBins(p->fYaxis, *(h->GetYaxis())) ) 
+                  p->fYaxis.Set(h->GetYaxis()->GetNbins(), h->GetYaxis()->GetXmin(),h->GetYaxis()->GetXmax());
+               if (!p->SameLimitsAndNBins(p->fZaxis, *(h->GetZaxis())) ) 
+                  p->fZaxis.Set(h->GetZaxis()->GetNbins(), h->GetZaxis()->GetXmin(),h->GetZaxis()->GetXmax());
+            }
+            firstNonEmptyHist = kFALSE;     
+         }
+
+         // this is executed the first time an histogram with limits is found
+         // to set some initial values on the new axis
          if (!initialLimitsFound) {
             initialLimitsFound = kTRUE;
             newXAxis.Set(h->GetXaxis()->GetNbins(), h->GetXaxis()->GetXmin(),
@@ -454,15 +473,21 @@ void TProfileHelper::Scale(T* p, Double_t c1, Option_t *)
 }
 
 template <typename T>
-void TProfileHelper::Sumw2(T* p)
+void TProfileHelper::Sumw2(T* p, Bool_t flag)
 {
-   // Create structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
+   // Create/Delete structure to store sum of squares of weights per bin  *-*-*-*-*-*-*-*
    //   This is needed to compute  the correct statistical quantities  
    //    of a profile filled with weights 
    //  
    //
    //  This function is automatically called when the histogram is created
    //  if the static function TH1::SetDefaultSumw2 has been called before.
+   
+   if (!flag) { 
+      // clear array if existing or do nothing 
+      if (p->fBinSumw2.fN > 0 ) p->fBinSumw2.Set(0);
+      return;
+   }
 
    if ( p->fBinSumw2.fN == p->fNcells) {
       if (!p->fgDefaultSumw2) 
