@@ -1,4 +1,4 @@
-// @(#)root/io:$Id: TDirectoryFile.cxx 43952 2012-04-25 21:19:23Z pcanal $
+// @(#)root/io:$Id$
 // Author: Rene Brun   22/01/2007
 
 /*************************************************************************
@@ -184,6 +184,18 @@ TDirectoryFile::~TDirectoryFile()
 
    CleanTargets();
 
+   // Delete our content before we become somewhat invalid
+   // since some those objects (TTree for example) needs information
+   // from this object.  Note that on some platform after the end
+   // of the body (i.e. thus during ~TDirectory which is also 
+   // contains this code) the exeuction of 'this->GetFile()' fails
+   // to return the 'proper' value (because it uses the wrong 
+   // virtual function).
+   if (fList) {
+      fList->Delete("slow");
+      SafeDelete(fList);
+   }
+
    if (gDebug) {
       Info("~TDirectoryFile", "dtor called for %s", GetName());
    }
@@ -474,25 +486,20 @@ TDirectory *TDirectoryFile::GetDirectory(const char *apath,
       delete [] path; return result;
    }
 
-   TObject *obj;
+   TDirectoryFile *obj;
    char *slash = (char*)strchr(path,'/');
    if (!slash) {                     // we are at the lowest level
       if (!strcmp(path, "..")) {
          result = GetMotherDir();
          delete [] path; return result;
       }
-      obj = Get(path);
+      GetObject(path,obj);
       if (!obj) {
          if (printError) Error(funcname,"Unknown directory %s", path);
          delete [] path; return 0;
       }
 
-      //Check return object is a directory
-      if (!obj->InheritsFrom(TDirectoryFile::Class())) {
-         if (printError) Error(funcname,"Object %s is not a directory", path);
-         delete [] path; return 0;
-      }
-      delete [] path; return (TDirectory*)obj;
+      delete [] path; return obj;
    }
 
    TString subdir(path);
@@ -505,17 +512,12 @@ TDirectory *TDirectoryFile::GetDirectory(const char *apath,
          result = mom->GetDirectory(slash+1,printError,funcname);
       delete [] path; return result;
    }
-   obj = Get(subdir);
+   GetObject(subdir,obj);
    if (!obj) {
       if (printError) Error(funcname,"Unknown directory %s", subdir.Data());
       delete [] path; return 0;
    }
 
-   //Check return object is a directory
-   if (!obj->InheritsFrom(TDirectoryFile::Class())) {
-      if (printError) Error(funcname,"Object %s is not a directory", subdir.Data());
-      delete [] path; return 0;
-   }
    result = ((TDirectory*)obj)->GetDirectory(slash+1,printError,funcname);
    delete [] path; return result;
 }
@@ -848,17 +850,17 @@ TObject *TDirectoryFile::Get(const char *namecycle)
    char     name[kMaxLen];
 
    DecodeNameCycle(namecycle, name, cycle);
-   char *namobj = name;
    Int_t nch = strlen(name);
    for (Int_t i = nch-1; i > 0; i--) {
       if (name[i] == '/') {
          name[i] = 0;
          TDirectory* dirToSearch=GetDirectory(name);
-         namobj = name + i + 1;
+         const char *subnamecycle = namecycle + i + 1;
          name[i] = '/';
-         return dirToSearch?dirToSearch->Get(namobj):0;
+         return dirToSearch?dirToSearch->Get(subnamecycle):0;
       }
    }
+   const char *namobj = name;
 
 //*-*---------------------Case of Object in memory---------------------
 //                        ========================
@@ -948,21 +950,21 @@ void *TDirectoryFile::GetObjectChecked(const char *namecycle, const TClass* expe
    char     name[kMaxLen];
 
    DecodeNameCycle(namecycle, name, cycle);
-   char *namobj = name;
    Int_t nch = strlen(name);
    for (Int_t i = nch-1; i > 0; i--) {
       if (name[i] == '/') {
          name[i] = 0;
          TDirectory* dirToSearch=GetDirectory(name);
-         namobj = name + i + 1;
+         const char *subnamecycle = namecycle + i + 1;
          name[i] = '/';
          if (dirToSearch) {
-            return dirToSearch->GetObjectChecked(namobj, expectedClass);
+            return dirToSearch->GetObjectChecked(subnamecycle, expectedClass);
          } else {
             return 0;
          }
       }
    }
+   const char *namobj = name;
 
 //*-*---------------------Case of Object in memory---------------------
 //                        ========================
@@ -1124,8 +1126,12 @@ TDirectory *TDirectoryFile::mkdir(const char *name, const char *title)
       char *workname = new char[size+1];
       strncpy(workname, name, size);
       workname[size] = 0;
-      TDirectoryFile *tmpdir = (TDirectoryFile*)mkdir(workname,title);
-      if (!tmpdir) return 0;
+      TDirectoryFile *tmpdir;
+      GetObject(workname,tmpdir);
+      if (!tmpdir) {
+         tmpdir = (TDirectoryFile*)mkdir(workname,title);
+         if (!tmpdir) return 0;
+      }
       if (!newdir) newdir = tmpdir;
       tmpdir->mkdir(slash+1);
       delete[] workname;
