@@ -1,4 +1,4 @@
-// @(#)root/io:$Id: TFileCacheRead.cxx 43149 2012-02-27 21:09:07Z pcanal $
+// @(#)root/io:$Id$
 // Author: Rene Brun   18/05/2006
 
 /*************************************************************************
@@ -137,41 +137,17 @@ TFileCacheRead::TFileCacheRead(TFile *file, Int_t buffersize, TObject *tree)
    //initialise the prefetch object and set the cache directory
    // start the thread only if the file is not local  
    fEnablePrefetching = gEnv->GetValue("TFile.AsyncPrefetching", 0);
+   
    if (fEnablePrefetching && strcmp(file->GetEndpointUrl()->GetProtocol(), "file")){
-      fPrefetch = new TFilePrefetch(file);
-      const char* cacheDir = gEnv->GetValue("Cache.Directory", "");
-      if (strcmp(cacheDir, ""))
-        if (!fPrefetch->SetCache((char*) cacheDir))
-           fprintf(stderr, "Error while trying to set the cache directory.\n");
-      if (fPrefetch->ThreadStart()){
-         fprintf(stderr,"Error stating prefetching thread. Disabling prefetching.\n");
-         fEnablePrefetching = 0;
-      }
+      SetEnablePrefetchingImpl(true);
    }
-   else //disable the async pref for local files
-      fEnablePrefetching = 0;
-
-   //environment variable used to switch to the new method of reading asynchronously
-   if (fEnablePrefetching){
-      fAsyncReading = kFALSE;
+   else {//disable the async pref for local files
+      SetEnablePrefetchingImpl(false);
    }
-   else {
-      fAsyncReading = gEnv->GetValue("TFile.AsyncReading", 0);
-      if (fAsyncReading) {
-         // Check if asynchronous reading is supported by this TFile specialization
-         fAsyncReading = kFALSE;
-         if (file && !(file->ReadBufferAsync(0, 0)))
-            fAsyncReading = kTRUE;
-      }
-      if (!fAsyncReading) {
-         // we use sync primitives, hence we need the local buffer
-         fBuffer = new char[fBufferSize];
-      }
-   }  
 
-   fIsSorted    = kFALSE;
-   fIsTransferred = kFALSE;
-   fBIsSorted = kFALSE;
+   fIsSorted       = kFALSE;
+   fIsTransferred  = kFALSE;
+   fBIsSorted      = kFALSE;
    fBIsTransferred = kFALSE;
 
    if (file) file->SetCacheRead(this, tree);
@@ -191,7 +167,8 @@ TFileCacheRead::~TFileCacheRead()
    delete [] fSeekSortLen;
    delete [] fSeekPos;
    delete [] fLen;
-   delete [] fBuffer;
+   if (fBuffer)
+      delete [] fBuffer;
    delete [] fBSeek;
    delete [] fBSeekIndex;
    delete [] fBSeekSort;
@@ -202,6 +179,21 @@ TFileCacheRead::~TFileCacheRead()
    delete [] fBLen;
 }
 
+//_____________________________________________________________________________
+void TFileCacheRead::Close(Option_t * /* opt = "" */)
+{
+   // Close out any threads or asynchronous fetches used by the underlying 
+   // implementation.
+   // This is called by TFile::Close to prevent usage of the file handles
+   // after the closing of the file.
+   
+   if (fPrefetch) {
+      delete fPrefetch;
+      fPrefetch = 0;
+   }
+
+}
+
 //_____________________________________________________________________________
 void TFileCacheRead::Prefetch(Long64_t pos, Int_t len)
 {
@@ -391,7 +383,7 @@ Int_t TFileCacheRead::ReadBufferExt(char *buf, Long64_t pos, Int_t len, Int_t &l
 //_____________________________________________________________________________
 Int_t TFileCacheRead::ReadBufferExtPrefetch(char *buf, Long64_t pos, Int_t len, Int_t &loc)
 {
-   //prefetch the first block                                                                                                                                
+   //prefetch the first block                                                                                                                            
    if (fNseek > 0 && !fIsSorted) {
       Sort();
       loc = -1;
@@ -425,8 +417,8 @@ Int_t TFileCacheRead::ReadBufferExtPrefetch(char *buf, Long64_t pos, Int_t len,
       if (buf && fPrefetch){
          //prefetch with the new method                  
          fPrefetch->ReadBuffer(buf, pos, len);
+         return 1;
       }
-      return 1;
    }
    else if (buf && fPrefetch){
       //try to preferch from the second block                                                                                                               
@@ -534,7 +526,7 @@ Int_t TFileCacheRead::ReadBufferExtNormal(char *buf, Long64_t pos, Int_t len, In
 }
 
 //_____________________________________________________________________________
-void TFileCacheRead::SetFile(TFile *file)
+void TFileCacheRead::SetFile(TFile *file, TFile::ECacheAction action)
 {
    // Set the file using this cache and reset the current blocks (if any).
 
@@ -549,10 +541,12 @@ void TFileCacheRead::SetFile(TFile *file)
       }
    }
 
-   Prefetch(0,0);
+   if (action == TFile::kDisconnect)
+      Prefetch(0,0);
    
    if (fPrefetch) {
-      SecondPrefetch(0, 0);
+      if (action == TFile::kDisconnect)
+         SecondPrefetch(0, 0);
       fPrefetch->SetFile(file);
    }
 }
@@ -677,3 +671,71 @@ TFilePrefetch* TFileCacheRead::GetPrefetchObj(){
   
    return this->fPrefetch;
 }
+
+
+//_____________________________________________________________________________
+void TFileCacheRead::WaitFinishPrefetch()
+{
+   if ( fEnablePrefetching && fPrefetch ) {
+      fPrefetch->WaitFinishPrefetch();
+   }
+}           
+
+
+//______________________________________________________________________________
+void TFileCacheRead::SetEnablePrefetching(Bool_t setPrefetching) 
+{
+   // Set the prefetching mode of this file.
+   // if 'setPrefetching', enable the asynchronous prefetching
+   // (using TFilePrefetch) and if the gEnv and rootrc 
+   // variable Cache.Directory is set, also enable the local
+   // caching of the prefetched blocks.
+   // if 'setPrefetching', the old prefetcher is enabled is
+   // the gEnv and rootrc variable is TFile.AsyncReading
+   
+   SetEnablePrefetchingImpl(setPrefetching);
+}
+
+//______________________________________________________________________________
+void TFileCacheRead::SetEnablePrefetchingImpl(Bool_t setPrefetching) 
+{
+   // TFileCacheRead implementation of SetEnablePrefetching.
+   //
+   // This function is called from the constructor and should not be virtual.
+      
+   fEnablePrefetching = setPrefetching;
+   
+   if (!fPrefetch && fEnablePrefetching) {
+      fPrefetch = new TFilePrefetch(fFile);
+      const char* cacheDir = gEnv->GetValue("Cache.Directory", "");
+      if (strcmp(cacheDir, ""))
+        if (!fPrefetch->SetCache((char*) cacheDir))
+           fprintf(stderr, "Error while trying to set the cache directory.\n");
+      if (fPrefetch->ThreadStart()){
+         fprintf(stderr,"Error stating prefetching thread. Disabling prefetching.\n");
+         fEnablePrefetching = 0;
+      }
+   } else if (fPrefetch && !fEnablePrefetching) {
+       SafeDelete(fPrefetch);
+       fPrefetch = NULL;
+   }
+
+   //environment variable used to switch to the new method of reading asynchronously
+   if (fEnablePrefetching) {
+      fAsyncReading = kFALSE;
+   }
+   else {
+      fAsyncReading = gEnv->GetValue("TFile.AsyncReading", 0);
+      if (fAsyncReading) {
+         // Check if asynchronous reading is supported by this TFile specialization
+         fAsyncReading = kFALSE;
+         if (fFile && !(fFile->ReadBufferAsync(0, 0)))
+            fAsyncReading = kTRUE;
+         }
+      if (!fAsyncReading && fBuffer == 0) {
+         // we use sync primitives, hence we need the local buffer
+         fBuffer = new char[fBufferSize];
+      }
+   }
+}
+
