@@ -1,4 +1,4 @@
-// @(#)root/sql:$Id: TBufferSQL2.cxx 43518 2012-03-28 01:04:07Z pcanal $
+// @(#)root/sql:$Id$
 // Author: Sergey Linev  20/11/2005
 
 /*************************************************************************
@@ -1543,6 +1543,22 @@ void TBufferSQL2::ReadFastArrayFloat16(Float_t  *f, Int_t n, TStreamerElement *
    TBufferSQL2_ReadFastArray(f);
 }
 
+//______________________________________________________________________________
+void TBufferSQL2::ReadFastArrayWithFactor(Float_t  *f, Int_t n, Double_t /* factor */, Double_t /* minvalue */)
+{
+   // read array of Float16_t from buffer
+
+   TBufferSQL2_ReadFastArray(f);
+}
+
+//______________________________________________________________________________
+void TBufferSQL2::ReadFastArrayWithNbits(Float_t  *f, Int_t n, Int_t /*nbits*/)
+{
+   // read array of Float16_t from buffer
+
+   TBufferSQL2_ReadFastArray(f);
+}
+
 //______________________________________________________________________________
 void TBufferSQL2::ReadFastArrayDouble32(Double_t  *d, Int_t n, TStreamerElement * /*ele*/)
 {
@@ -1551,6 +1567,21 @@ void TBufferSQL2::ReadFastArrayDouble32(Double_t  *d, Int_t n, TStreamerElement
    TBufferSQL2_ReadFastArray(d);
 }
 
+//______________________________________________________________________________
+void TBufferSQL2::ReadFastArrayWithFactor(Double_t  *d, Int_t n, Double_t /* factor */, Double_t /* minvalue */)
+{
+   // read array of Double32_t from buffer
+
+   TBufferSQL2_ReadFastArray(d);
+}
+//______________________________________________________________________________
+void TBufferSQL2::ReadFastArrayWithNbits(Double_t  *d, Int_t n, Int_t /*nbits*/)
+{
+   // read array of Double32_t from buffer
+
+   TBufferSQL2_ReadFastArray(d);
+}
+
 //______________________________________________________________________________
 void TBufferSQL2::ReadFastArray(void  *start, const TClass *cl, Int_t n, TMemberStreamer *streamer, const TClass* onFileClass )
 {
