@@ -1,4 +1,4 @@
-// @(#)root/mathcore:$Id: TMath.h 41837 2011-11-09 16:04:19Z rdm $
+// @(#)root/mathcore:$Id$
 // Authors: Rene Brun, Anna Kreshuk, Eddy Offermann, Fons Rademakers   29/07/95
 
 /*************************************************************************
@@ -302,6 +302,9 @@ namespace TMath {
    template <typename T> Double_t RMS(Long64_t n, const T *a);
    template <typename Iterator> Double_t RMS(Iterator first, Iterator last);
 
+   template <typename T> Double_t StdDev(Long64_t n, const T *a) { return RMS<T>(n,a); }
+   template <typename Iterator> Double_t StdDev(Iterator first, Iterator last) { return RMS<Iterator>(first,last); }
+
    template <typename T> Double_t Median(Long64_t n, const T *a,  const Double_t *w=0, Long64_t *work=0);
 
    //k-th order statistic
@@ -502,7 +505,7 @@ inline Int_t TMath::Finite(Double_t x)
 #endif
 
 inline Int_t TMath::IsNaN(Double_t x)
-#if (defined(R__ANSISTREAM) || (defined(R__MACOSX) && defined(__arm__))) && !defined(_AIX)
+#if (defined(R__ANSISTREAM) || (defined(R__MACOSX) && defined(__arm__))) && !defined(_AIX) && !defined(__CUDACC__) 
 #if defined(isnan) || defined(R__SOLARIS_CC50) || defined(__INTEL_COMPILER)
    // from math.h
   { return ::isnan(x); }
