@@ -1,4 +1,4 @@
-// @(#)root/minuit:$Id: TMinuitMinimizer.cxx 43901 2012-04-23 11:09:34Z moneta $
+// @(#)root/minuit:$Id$
 // Author: L. Moneta Wed Oct 25 16:28:55 2006
 
 /**********************************************************************
@@ -297,30 +297,20 @@ bool TMinuitMinimizer::SetLimitedVariable(unsigned int ivar, const std::string &
    int iret = fMinuit->DefineParameter(ivar, name.c_str(), val, step, lower, upper ); 
    return (iret == 0); 
 }
-#ifdef LATER
-bool Minuit2Minimizer::SetLowerLimitedVariable(unsigned int ivar , const std::string & name , double val , double step , double lower ) {
-    // add a lower bounded variable as a double bound one, using a very large number for the upper limit
 
-   if (fMinuit == 0) { 
-      Error("SetVariable","invalid TMinuit pointer. Need to call first SetFunction"); 
-      return false; 
-   }
-
-   fUsed = fgUsed; 
-
-   // clear after minimization when setting params
-   if (fUsed) DoClear(); 
-
-   // check if parameter was defined and in case it was fixed, release it 
-   DoReleaseFixParameter(ivar);
-
-   double s = val-lower; 
-   double upper = s*1.0E15; 
-   if (s != 0)  upper = 1.0E15;
-   return SetLimitedVariable(ivar, name, val, step, lower,upper);
+bool TMinuitMinimizer::SetLowerLimitedVariable(unsigned int  ivar , const std::string & name , double val , double step , double lower ) { 
+   // set a lower limited variable
+   // since is not supported in TMinuit , just use a artificial large value 
+   Warning("TMinuitMinimizer::SetLowerLimitedVariable","not implemented - use as upper limit 1.E7 instead of +inf"); 
+   return SetLimitedVariable(ivar, name, val , step, lower, lower+ 1.E7);  // use 1.E7 which will make TMinuit happy
 }
-#endif
 
+bool TMinuitMinimizer::SetUpperLimitedVariable(unsigned int  ivar , const std::string & name , double val , double step , double upper ) { 
+   // set a upper limited variable
+   // since is not supported in TMinuit , just use a artificial large negative value 
+   Warning("TMinuitMinimizer::SetUpperLimitedVariable","not implemented - use as lower limit -1.E7 instead of -inf"); 
+   return SetLimitedVariable(ivar, name, val , step, upper -1.E7, upper);
+}
 
 bool TMinuitMinimizer::SetFixedVariable(unsigned int ivar, const std::string & name, double val) { 
    // set a fixed variable.
@@ -384,17 +374,29 @@ bool TMinuitMinimizer::Minimize() {
 
 
    if (fMinuit == 0) { 
-      Error("Minimize","invalid TMinuit pointer. Need to call first SetFunction and SetVariable"); 
+      Error("TMinuitMinimizer::Minimize","invalid TMinuit pointer. Need to call first SetFunction and SetVariable"); 
       return false; 
    }
 
 
    // total number of parameter defined in Minuit is fNu
    if (fMinuit->fNu <  static_cast<int>(fDim) ) { 
-      Error("Minimize","The total number of defined parameters is different than the function dimension, npar = %d, dim = %d",fMinuit->fNu, fDim);
+      Error("TMinuitMinimizer::Minimize","The total number of defined parameters is different than the function dimension, npar = %d, dim = %d",fMinuit->fNu, fDim);
       return false; 
    }
 
+   int printlevel = PrintLevel(); 
+
+   // total number of free parameter is 0
+   if (fMinuit->fNpar <= 0) { 
+      // retrieve parameters values  from TMinuit
+      RetrieveParams(); 
+      fMinuit->fAmin = (*fgFunc)(&fParams.front());
+      if (printlevel > 0) Info("TMinuitMinimizer::Minimize","There are no free parameter - just compute the function value");
+      return true; 
+   }
+
+ 
    double arglist[10]; 
    int ierr = 0; 
 
@@ -403,7 +405,6 @@ bool TMinuitMinimizer::Minimize() {
    arglist[0] = ErrorDef(); 
    fMinuit->mnexcm("SET Err",arglist,1,ierr);
 
-   int printlevel = PrintLevel(); 
    arglist[0] = printlevel - 1;
    fMinuit->mnexcm("SET PRINT",arglist,1,ierr);
 
@@ -465,13 +466,13 @@ bool TMinuitMinimizer::Minimize() {
    fStatus = ierr; 
    int minErrStatus = ierr;
 
-   if (printlevel>2) Info("Minimize","Finished to run MIGRAD - status %d",ierr);
+   if (printlevel>2) Info("TMinuitMinimizer::Minimize","Finished to run MIGRAD - status %d",ierr);
 
    // run improved if needed
    if (ierr == 0 && fType == ROOT::Minuit::kMigradImproved) {
       fMinuit->mnexcm("IMPROVE",arglist,1,ierr);
       fStatus += 1000*ierr; 
-      if (printlevel>2) Info("Minimize","Finished to run IMPROVE - status %d",ierr);
+      if (printlevel>2) Info("TMinuitMinimizer::Minimize","Finished to run IMPROVE - status %d",ierr);
    }
 
 
@@ -481,7 +482,7 @@ bool TMinuitMinimizer::Minimize() {
    if (minErrStatus == 0  && (IsValidError() || ( strategy >=1 && CovMatrixStatus() < 3) ) ) { 
       fMinuit->mnexcm("HESSE",arglist,1,ierr);
       fStatus += 100*ierr; 
-      if (printlevel>2) Info("Minimize","Finished to run HESSE - status %d",ierr);
+      if (printlevel>2) Info("TMinuitMinimizer::Minimize","Finished to run HESSE - status %d",ierr);
    }
 
    // retrieve parameters and errors  from TMinuit
@@ -590,7 +591,7 @@ bool TMinuitMinimizer::GetCovMatrix(double * cov) const {
    // get covariance matrix
    int covStatus = CovMatrixStatus(); 
    if ( fCovar.size() != fDim*fDim || covStatus < 2) { 
-      Error("GetHessianMatrix","Hessian matrix has not been computed - status %d",covStatus);
+      Error("TMinuitMinimizer::GetHessianMatrix","Hessian matrix has not been computed - status %d",covStatus);
       return false;
    }
    std::copy(fCovar.begin(), fCovar.end(), cov);
@@ -604,7 +605,7 @@ bool TMinuitMinimizer::GetHessianMatrix(double * hes) const {
    // but need to get the compact form to avoid the zero for the fixed parameters 
    int covStatus = CovMatrixStatus(); 
    if ( fCovar.size() != fDim*fDim || covStatus < 2) { 
-      Error("GetHessianMatrix","Hessian matrix has not been computed - status %d",covStatus);
+      Error("TMinuitMinimizer::GetHessianMatrix","Hessian matrix has not been computed - status %d",covStatus);
       return false;
    }
    // case of fixed params need to take care 
@@ -666,33 +667,35 @@ bool TMinuitMinimizer::GetMinosError(unsigned int i, double & errLow, double & e
    // Perform Minos analysis for the given parameter  i 
 
    if (fMinuit == 0) { 
-      Error("GetMinosError","invalid TMinuit pointer. Need to call first SetFunction and SetVariable"); 
+      Error("TMinuitMinimizer::GetMinosError","invalid TMinuit pointer. Need to call first SetFunction and SetVariable"); 
       return false; 
    }
 
    double arglist[2];
    int ierr = 0; 
 
-   // if Minos is not run run it 
-   if (!fMinosRun) { 
-
-      // set error and print level 
+   // set error and print level, precision and stratgey if needed
+   if (fMinuit->fUp != ErrorDef() ) { 
       arglist[0] = ErrorDef(); 
       fMinuit->mnexcm("SET Err",arglist,1,ierr);
+   }
       
+   if (fMinuit->fISW[4] != (PrintLevel()-1) )  { 
       arglist[0] = PrintLevel()-1; 
       fMinuit->mnexcm("SET PRINT",arglist,1,ierr);
-
       // suppress warning in case Printlevel() == 0 
       if (PrintLevel() == 0)    fMinuit->mnexcm("SET NOW",arglist,0,ierr);
-
-      // set precision if needed
-      if (Precision() > 0)  { 
-         arglist[0] = Precision();
-         fMinuit->mnexcm("SET EPS",arglist,1,ierr);
-      }
-
    }
+   if (fMinuit->fIstrat != Strategy() ) { 
+      arglist[0] = Strategy(); 
+      fMinuit->mnexcm("SET STR",arglist,1,ierr);
+   }
+
+   if (Precision() > 0 &&  fMinuit->fEpsma2 != Precision() ) { 
+      arglist[0] = Precision();
+      fMinuit->mnexcm("SET EPS",arglist,1,ierr);
+   }
+   
 
    // syntax of MINOS is MINOS [maxcalls] [parno]
    // if parno = 0 all parameters are done 
@@ -701,6 +704,18 @@ bool TMinuitMinimizer::GetMinosError(unsigned int i, double & errLow, double & e
    
    int nargs = 2; 
    fMinuit->mnexcm("MINOS",arglist,nargs,ierr);
+   bool isValid = (ierr == 0); 
+   // check also the status from fCstatu
+   if (isValid && fMinuit->fCstatu != "SUCCESSFUL") {
+      if (fMinuit->fCstatu == "FAILURE" ) { 
+         // in this case MINOS failed on all prameter, so it is not valid ! 
+         ierr = 5; 
+         isValid = false; 
+      }   
+      if (fMinuit->fCstatu == "PROBLEMS") ierr = 6; 
+      ierr = 7;  // this should be the case UNCHANGED
+   }
+
    fStatus += 10*ierr;
 
    fMinosRun = true; 
@@ -710,8 +725,8 @@ bool TMinuitMinimizer::GetMinosError(unsigned int i, double & errLow, double & e
    // what returns if parameter fixed or constant or at limit ? 
    fMinuit->mnerrs(i,errUp,errLow, errParab, gcor); 
 
-   if (fStatus%100 != 0 ) return false; 
-   return true; 
+   // do not flag errors case of PROBLEMS or UNCHANGED (
+   return isValid; 
 
 }
 
@@ -801,21 +816,20 @@ bool TMinuitMinimizer::Contour(unsigned int ipar, unsigned int jpar, unsigned in
 
 
    if (npoints < 4) { 
-      Error("Contour","Cannot make contour with so few points");
+      Error("TMinuitMinimizer::Contour","Cannot make contour with so few points");
       return false; 
    }
    int npfound = 0; 
-   npoints -= 1;   // remove always one point in TMinuit
    // parameter numbers in mncont start from zero
    fMinuit->mncont( ipar,jpar,npoints, x, y,npfound); 
    if (npfound<4) {
       // mncont did go wrong
-      Error("Contour","Cannot find more than 4 points");
+      Error("TMinuitMinimizer::Contour","Cannot find more than 4 points");
       return false;
    }
    if (npfound!=(int)npoints) {
       // mncont did go wrong
-      Warning("Contour","Returning only %d points ",npfound);
+      Warning("TMinuitMinimizer::Contour","Returning only %d points ",npfound);
       npoints = npfound;
    }
    return true; 
@@ -898,7 +912,7 @@ bool TMinuitMinimizer::Hesse() {
    // perform calculation of Hessian
 
    if (fMinuit == 0) { 
-      Error("Hesse","invalid TMinuit pointer. Need to call first SetFunction and SetVariable"); 
+      Error("TMinuitMinimizer::Hesse","invalid TMinuit pointer. Need to call first SetFunction and SetVariable"); 
       return false; 
    }
 
