@@ -1,4 +1,4 @@
-// @(#)root/net:$Id: TWebFile.cxx 44247 2012-05-15 09:12:06Z rdm $
+// @(#)root/net:$Id$
 // Author: Fons Rademakers   17/01/97
 
 /*************************************************************************
@@ -354,6 +354,8 @@ void TWebFile::CheckProxy()
          return;
       }
       fProxy = p;
+      if (gDebug > 0)
+         Info("CheckProxy", "using HTTP proxy %s", fProxy.GetUrl());
    }
 }
 
@@ -644,6 +646,9 @@ Int_t TWebFile::GetFromWeb10(char *buf, Int_t len, const TString &msg)
       return -1;
    }
 
+   if (gDebug > 0)
+      Info("GetFromWeb10", "sending HTTP request:\n%s", msg.Data());
+   
    if (fSocket->SendRaw(msg.Data(), msg.Length()) == -1) {
       Error("GetFromWeb10", "error sending command to host %s", fUrl.GetHost());
       return -1;
@@ -890,7 +895,7 @@ Int_t TWebFile::GetHead()
       fMsgGetHead += "\r\n\r\n";
    }
    TString msg = fMsgGetHead;
-
+ 
    TUrl connurl;
    if (fProxy.IsValid())
       connurl = fProxy;
@@ -925,6 +930,11 @@ Int_t TWebFile::GetHead()
    if (!s)
       return -1;
 
+   if (gDebug > 0) {
+      Info("GetHead", "connected to host %s", connurl.GetHost());
+      Info("GetHead", "sending HTTP request:\n%s", msg.Data());
+   }
+
    if (s->SendRaw(msg.Data(), msg.Length()) == -1) {
       Error("GetHead", "error sending command to host %s", fUrl.GetHost());
       delete s;
@@ -955,6 +965,7 @@ Int_t TWebFile::GetHead()
          Info("GetHead", "header: %s", line);
 
       TString res = line;
+      ProcessHttpHeader(res);
       if (res.BeginsWith("HTTP/1.")) {
          if (res.BeginsWith("HTTP/1.1")) {
             if (!fHTTP11) {
@@ -1240,6 +1251,13 @@ const char *TWebFile::GetProxy()
    return "";
 }
 
+//______________________________________________________________________________
+void TWebFile::ProcessHttpHeader(const TString&)
+{
+   // Process the HTTP header in the argument. This method is intended to be
+   // overwritten by subclasses that exploit the information contained in the
+   // HTTP headers.
+}
 
 //______________________________________________________________________________
 TWebSystem::TWebSystem() : TSystem("-http", "HTTP Helper System")
@@ -1350,7 +1368,7 @@ Bool_t TWebSystem::AccessPathName(const char *path, EAccessMode)
 //______________________________________________________________________________
 Int_t TWebSystem::Unlink(const char *)
 {
-   // Unlink, i.e. remove, a file or directory. Returns 0 when succesfull,
+   // Unlink, i.e. remove, a file or directory. Returns 0 when successful,
    // -1 in case of failure. Not supported for httpd.
 
    return -1;
