@@ -1,4 +1,4 @@
-// @(#)root/proof:$Id: TProofBench.cxx 44016 2012-04-30 08:23:40Z ganis $
+// @(#)root/proof:$Id$
 // Author: G.Ganis, S.Ryu Feb 2011
 
 /*************************************************************************
@@ -110,6 +110,33 @@ TProofBench::TProofBench(const char *url, const char *outfile, const char *proof
       Error("TProofBench", "could not open a valid PROOF session - cannot continue");
       return;
    }
+   // Get the size of the cluster
+   fNumWrkMax = fProof->GetParallel();
+   if (fProof->UseDynamicStartup() && TProof::GetEnvVars()) {
+      // It must be passed as PROOF option 'workers=N' and recorded in the envs vars
+      TNamed *n = (TNamed *) TProof::GetEnvVars()->FindObject("PROOF_NWORKERS");
+      if (!n) {
+         Error("TProofBench", "dynamic mode: you must specify the max number of workers");
+         fProof->Close();
+         SafeDelete(fProof);
+         return;
+      }
+      TString sn(n->GetTitle());
+      if (sn.IsDigit()) fNumWrkMax = sn.Atoi();
+      if (!sn.IsDigit()) {
+         Error("TProofBench", "dynamic mode: wrong specification of the max number of"
+                              " workers ('%s')", n->GetTitle());
+         fProof->Close();
+         SafeDelete(fProof);
+         return;
+      }
+   }
+   if (fNumWrkMax <= 0) {
+      Error("TProofBench", "wrong max number of workers ('%d')", fNumWrkMax);
+      fProof->Close();
+      SafeDelete(fProof);
+      return;
+   }
    // By default we use the same instance for dataset actions
    fProofDS = fProof;
    // The object is now valid
@@ -119,7 +146,7 @@ TProofBench::TProofBench(const char *url, const char *outfile, const char *proof
    TString host(TString::Format("PROOF at %s", u.GetHost()));
    if (!strcmp(u.GetProtocol(), "lite")) host.Form("PROOF-Lite on %s", gSystem->HostName());
    fDescription = new TNamed("PB_description",
-                             TString::Format("%s, %d workers", host.Data(), fProof->GetParallel()).Data());
+                             TString::Format("%s, %d workers", host.Data(), fNumWrkMax).Data());
    Printf(" Run description: %s", fDescription->GetTitle());
    // Set output file
    if (SetOutFile(outfile, kFALSE) != 0)
@@ -193,7 +220,7 @@ Int_t TProofBench::SetOutFile(const char *outfile, Bool_t verbose)
       TDatime dat;
       const char *lite = (fProof->IsLite()) ? "-lite" : "";
       fOutFileName.Form("proofbench-%s%s-%dw-%d-%.2d%.2d.root",
-                        fProof->GetMaster(), lite, fProof->GetParallel(),
+                        fProof->GetMaster(), lite, fNumWrkMax,
                         dat.GetDate(), dat.GetHour(), dat.GetMinute());
       Info("SetOutFile", "using default output file: '%s'", fOutFileName.Data());
       fUnlinkOutfile = kTRUE;
@@ -344,6 +371,8 @@ void TProofBench::DrawCPU(const char *outfile, const char *opt, Bool_t verbose,
    Int_t nbins = pf->GetNbinsX();
    TGraphErrors *gr = new TGraphErrors(nbins);
    Double_t xx, ex, yy, ey, ymi = pf->GetBinContent(1), ymx = ymi;
+   Double_t xlow = pf->GetBinCenter(1) - pf->GetBinWidth(1)/2. ;
+   Double_t xhigh = pf->GetBinCenter(nbins) + pf->GetBinWidth(nbins)/2. ;
    Int_t k =1, kmx = -1;
    for (;k <= nbins; k++) {
       xx = pf->GetBinCenter(k);
@@ -372,7 +401,7 @@ void TProofBench::DrawCPU(const char *outfile, const char *opt, Bool_t verbose,
    gr->SetMarkerStyle(21);
    gr->SetMarkerSize(1.2);
 
-   TH1F *hgr = new TH1F("Graph-CPU"," CPU speed-up", nbins*4,0,nbins+1);
+   TH1F *hgr = new TH1F("Graph-CPU"," CPU speed-up", nbins*4, xlow, xhigh);
    hgr->SetMaximum(ymx + (ymx-ymi)*0.2);
    hgr->SetMinimum(0);
    hgr->SetDirectory(0);
@@ -736,7 +765,7 @@ void TProofBench::DrawDataSet(const char *outfile,
       hprofn.Form("Prof_x_DataRead_%s", typ.Data());
    } else if (!strcmp(opt, "norm:")) {
       hprofn.Form("Norm_DataRead_%s", typ.Data());
-   } else if (!strcmp(opt, "absx:")) {
+   } else if (!strcmp(opt, "normx:")) {
       hprofn.Form("Norm_x_DataRead_%s", typ.Data());
    } else {
       ::Error("DrawDataSet", "unknown option '%s'", opt);
@@ -763,6 +792,8 @@ void TProofBench::DrawDataSet(const char *outfile,
    Int_t nbins = pf->GetNbinsX();
    TGraphErrors *gr = new TGraphErrors(nbins);
    Double_t xx, ex, yy, ey, ymi = pf->GetBinContent(1), ymx = ymi;
+   Double_t xlow = pf->GetBinCenter(1) - pf->GetBinWidth(1)/2. ;
+   Double_t xhigh = pf->GetBinCenter(nbins) + pf->GetBinWidth(nbins)/2. ;
    Int_t k =1;
    for (;k <= nbins; k++) {
       xx = pf->GetBinCenter(k);
@@ -791,7 +822,7 @@ void TProofBench::DrawDataSet(const char *outfile,
    gr->SetMarkerStyle(21);
    gr->SetMarkerSize(1.2);
 
-   TH1F *hgr = new TH1F("Graph-DataSet"," Data Read speed-up", nbins*4,0,nbins+1);
+   TH1F *hgr = new TH1F("Graph-DataSet"," Data Read speed-up", nbins*4, xlow, xhigh);
    hgr->SetMaximum(ymx + (ymx-ymi)*0.2);
    hgr->SetMinimum(0);
    hgr->SetDirectory(0);
