@@ -1,4 +1,4 @@
-// @(#)root/proofd:$Id: XrdProofdManager.cxx 44523 2012-06-04 15:17:29Z ganis $
+// @(#)root/proofd:$Id$
 // Author: G. Ganis June 2007
 
 /*************************************************************************
@@ -123,6 +123,7 @@ XrdProofdManager::XrdProofdManager(XrdProtocol_Config *pi, XrdSysError *edest)
    fPort = XPD_DEF_PORT;
    fImage = "";        // image name for these servers
    fSockPathDir = "";
+   fStageReqRepo = "";
    fTMPdir = "/tmp";
    fWorkDir = "";
    fMUWorkDir = "";
@@ -137,6 +138,7 @@ XrdProofdManager::XrdProofdManager(XrdProtocol_Config *pi, XrdSysError *edest)
    // Data dir
    fDataDir = "";        // Default <workdir>/<user>/data
    fDataDirOpts = "";    // Default: no action
+   fDataDirUrlOpts = ""; // Default: none
 
    // Rootd file serving enabled by default in readonly mode
    fRootdExe = "<>";
@@ -338,7 +340,7 @@ int XrdProofdManager::CheckUser(const char *usr, const char *grp,
       // Policy: check first the general directive for groups; a user of a specific group
       // (both UNIX or PROOF groups) can be rejected by prefixing a '-'.
       // The group check fails if active (the allowedgroups directive has entries) and at
-      // least of the two groups (UNIX or PROOF) are explicitely denied.
+      // least of the two groups (UNIX or PROOF) are explicitly denied.
       // The result of the group check is superseeded by any explicit speicification in the
       // allowedusers, either positive or negative.
       //
@@ -352,13 +354,13 @@ int XrdProofdManager::CheckUser(const char *usr, const char *grp,
       //         User 'katy' is allowed because part of PROOF group 'student'; 
       //         users 'jack' and 'john' are denied because not part of PROOF group 'student' (no 'allowedusers' directive)
       //   3.    xpd.allowedgroups alfa,-student
-      //         User 'katy' is denied because part of PROOF group 'student' which is explicitely denied;
+      //         User 'katy' is denied because part of PROOF group 'student' which is explicitly denied;
       //         users 'jack' and 'john' are allowed becasue part of UNIX group 'alfa' (no 'allowedusers' directive)
       //   4.    xpd.allowedgroups alfa,-student
       //         xpd.allowedusers katy,-jack
-      //         User 'katy' is allowed because explicitely allowed by the 'allowedusers' directive;
-      //         user 'jack' is denied because explicitely denied by the 'allowedusers' directive;
-      //         user 'john' is allowed because part of 'alfa' and not explicitely denied by the 'allowedusers' directive
+      //         User 'katy' is allowed because explicitly allowed by the 'allowedusers' directive;
+      //         user 'jack' is denied because explicitly denied by the 'allowedusers' directive;
+      //         user 'john' is allowed because part of 'alfa' and not explicitly denied by the 'allowedusers' directive
       //         (the allowedgroups directive is in this case ignored for users 'katy' and 'jack').
 
       bool grpok = 1;
@@ -428,7 +430,7 @@ int XrdProofdManager::CheckUser(const char *usr, const char *grp,
          usrok = 1;
          e = "";
       }
-      // We fail if either the user is explicitely denied or it is not explicitely allowed
+      // We fail if either the user is explicitly denied or it is not explicitly allowed
       // and the group is denied
       if (usrok == -1 || (!grpok && usrok != 1)) return -1;
    }
@@ -750,20 +752,23 @@ int XrdProofdManager::Config(bool rcf)
 
    // Data directory, if specified
    if (fDataDir.length() > 0) {
-      // Make sure it exists
-      if (XrdProofdAux::AssertDir(fDataDir.c_str(), ui, fChangeOwn) != 0) {
-         XPDERR("unable to assert data dir: " << fDataDir);
-         return -1;
-      }
-      // Get the right privileges now
-      XrdSysPrivGuard pGuard((uid_t)ui.fUid, (gid_t)ui.fGid);
-      if (XpdBadPGuard(pGuard, ui.fUid)) {
-         TRACE(XERR, "could not get privileges to set/change ownership of " << fDataDir);
-         return -1;
-      }
-      if (chmod(fDataDir.c_str(), 0777) != 0) {
-         XPDERR("problems setting permissions 0777 data dir: " << fDataDir);
-         return -1;
+      if (fDataDir.endswith('/')) fDataDir.erasefromend(1);
+      if (fDataDirOpts.length() > 0) {
+         // Make sure it exists
+         if (XrdProofdAux::AssertDir(fDataDir.c_str(), ui, fChangeOwn) != 0) {
+            XPDERR("unable to assert data dir: " << fDataDir << " (opts: "<<fDataDirOpts<<")");
+            return -1;
+         }
+         // Get the right privileges now
+         XrdSysPrivGuard pGuard((uid_t)ui.fUid, (gid_t)ui.fGid);
+         if (XpdBadPGuard(pGuard, ui.fUid)) {
+            TRACE(XERR, "could not get privileges to set/change ownership of " << fDataDir);
+            return -1;
+         }
+         if (chmod(fDataDir.c_str(), 0777) != 0) {
+            XPDERR("problems setting permissions 0777 data dir: " << fDataDir);
+            return -1;
+         }
       }
       TRACE(ALL, "data directories under: " << fDataDir);
    }
@@ -807,18 +812,25 @@ int XrdProofdManager::Config(bool rcf)
       std::list<XrdProofdDSInfo *>::iterator ii = fDataSetSrcs.begin();
       bool goodsrc = 0;
       for (ii = fDataSetSrcs.begin(); ii != fDataSetSrcs.end();) {
-         if (!(goodsrc = ValidateLocalDataSetSrc((*ii)->fUrl, (*ii)->fLocal))) {
-            XPDERR("source " << (*ii)->fUrl << " could not be validated");
-            ii = fDataSetSrcs.erase(ii);
+         TRACE(ALL, ">> Defined dataset: " << (*ii)->ToString());
+         if ((*ii)->fType == "file") {
+            if (!(goodsrc = ValidateLocalDataSetSrc((*ii)->fUrl, (*ii)->fLocal))) {
+               XPDERR("source " << (*ii)->fUrl << " could not be validated");
+               ii = fDataSetSrcs.erase(ii);
+            } else {
+               // Check next
+               ii++;
+            }
          } else {
-            // Check next
+            // Validate only "file" datasets
+            TRACE(ALL, "Skipping validation (no \"file\" type dataset source)");
             ii++;
          }
       }
       if (fDataSetSrcs.size() > 0) {
          TRACE(ALL, fDataSetSrcs.size() << " dataset sources defined");
          for (ii = fDataSetSrcs.begin(); ii != fDataSetSrcs.end(); ii++) {
-            TRACE(ALL, " url:" << (*ii)->fUrl << ", local:" << (*ii)->fLocal << ", rw:" << (*ii)->fRW);
+            TRACE(ALL, ">> Valid dataset: " << (*ii)->ToString());
             if ((*ii)->fLocal && (*ii)->fRW) {
                if (fDataSetExp.length() > 0) fDataSetExp += ",";
                fDataSetExp += ((*ii)->fUrl).c_str();
@@ -938,7 +950,7 @@ int XrdProofdManager::Config(bool rcf)
 
    // Groups
    if (!fGroupsMgr)
-      // Create default group, if none explicitely requested
+      // Create default group, if none explicitly requested
       fGroupsMgr = new XrdProofGroupMgr;
 
    if (fGroupsMgr)
@@ -1113,6 +1125,7 @@ bool XrdProofdManager::ValidateLocalDataSetSrc(XrdOucString &url, bool &local)
                if (!flck) {
                   TRACE(XERR, "Cannot open file '" << fnpath << "' with the lock file path; errno: " << errno);
                } else {
+                  errno = 0;
                   off_t ofs = lseek(fileno(flck), 0, SEEK_CUR);
                   if (ofs == 0) {
                      // New file: write the default lock file path
@@ -1123,13 +1136,15 @@ bool XrdProofdManager::ValidateLocalDataSetSrc(XrdOucString &url, bool &local)
                      fprintf(flck, "%s\n", fnlock.c_str());
                      if (fclose(flck) != 0)
                         TRACE(XERR, "Problems closing file '" << fnpath << "'; errno: " << errno);
+                     flck = 0;
                      if (XrdProofdAux::ChangeOwn(fnpath.c_str(), ui) != 0) {
                         TRACE(XERR, "Problems asserting ownership of " << fnpath);
                      }
-                  } else if (ofs != (off_t)(-1)) {
+                  } else if (ofs == (off_t)(-1)) {
                      TRACE(XERR, "Problems getting current position on file '" << fnpath << "'; errno: " << errno);
                   }
-                  fclose(flck);
+                  if (flck && fclose(flck) != 0)
+                     TRACE(XERR, "Problems closing file '" << fnpath << "'; errno: " << errno);
                }
             }
             // Make sure that everybody can modify the file for updates
@@ -1139,6 +1154,9 @@ bool XrdProofdManager::ValidateLocalDataSetSrc(XrdOucString &url, bool &local)
          }
       }
    }
+   else {
+      TRACE(ALL, "New dataset with no URL!");
+   }
    // Done
    return goodsrc;
 }
@@ -1173,6 +1191,7 @@ void XrdProofdManager::RegisterDirectives()
    Register("image", new XrdProofdDirective("image", (void *)&fImage, &DoDirectiveString));
    Register("workdir", new XrdProofdDirective("workdir", (void *)&fWorkDir, &DoDirectiveString));
    Register("sockpathdir", new XrdProofdDirective("sockpathdir", (void *)&fSockPathDir, &DoDirectiveString));
+   Register("stagereqrepo", new XrdProofdDirective("stagereqrepo", (void *)&fStageReqRepo, &DoDirectiveString));
 }
 
 //______________________________________________________________________________
@@ -1187,6 +1206,7 @@ int XrdProofdManager::ResolveKeywords(XrdOucString &s, XrdProofdClient *pcl)
    //     <group>            user group
    //     <uid>              user ID
    //     <gid>              user group ID
+   //     <effuser>          effective user name (for multiuser or user mapping modes)
    // Return the number of keywords resolved.
    XPDLOC(ALL, "Manager::ResolveKeywords")
 
@@ -1214,6 +1234,15 @@ int XrdProofdManager::ResolveKeywords(XrdOucString &s, XrdProofdClient *pcl)
          nk++;
    }
 
+   // Parse <effuser> of the process
+   if (s.find("<effuser>") != STR_NPOS) {
+      XrdProofUI eui;
+      if (XrdProofdAux::GetUserInfo(geteuid(), eui) == 0) {
+         if (s.replace("<effuser>", eui.fUser.c_str()))
+            nk++;
+      }
+   }
+
    // Parse <user>
    if (pcl)
       if (s.replace("<user>", pcl->User()))
@@ -1633,7 +1662,7 @@ int XrdProofdManager::DoDirectiveDataSetSrc(char *val, XrdOucStream *cfg, bool)
       return -1;
 
    // URL for this source
-   XrdOucString type(val), url, opts;
+   XrdOucString type(val), url, opts, obscure;
    bool rw = 0, local = 0, goodsrc = 1;
    char *nxt = 0;
    while ((nxt = cfg->GetWord())) {
@@ -1645,6 +1674,9 @@ int XrdProofdManager::DoDirectiveDataSetSrc(char *val, XrdOucStream *cfg, bool)
          if (u.Proto == "" && u.HostWPort == "") local = 1;
       } else if (!strncmp(nxt, "opt:", 4)) {
          opts = nxt + 4;
+      } else {
+         obscure += nxt;
+         obscure += " ";
       }
    }
 
@@ -1663,10 +1695,12 @@ int XrdProofdManager::DoDirectiveDataSetSrc(char *val, XrdOucStream *cfg, bool)
       if (opts.length() <= 0) {
          opts = rw ? "Ar:Av:" : "-Ar:-Av:";
       }
+      XrdProofdDSInfo *dsi = new XrdProofdDSInfo(type.c_str(), url.c_str(),
+         local, rw, opts.c_str(), obscure.c_str());
       if (haslocal || !local) {
-         fDataSetSrcs.push_back(new XrdProofdDSInfo(type.c_str(), url.c_str(), local, rw, opts.c_str()));
+         fDataSetSrcs.push_back(dsi);
       } else {
-         fDataSetSrcs.push_front(new XrdProofdDSInfo(type.c_str(), url.c_str(), local, rw, opts.c_str()));
+         fDataSetSrcs.push_front(dsi);
       }
    }
    return 0;
@@ -1683,12 +1717,20 @@ int XrdProofdManager::DoDirectiveDataDir(char *val, XrdOucStream *cfg, bool)
 
    // Data directory and write permissions
    fDataDir = val;
+   fDataDirOpts = "";
+   fDataDirUrlOpts = "";
    XrdOucString opts;
    char *nxt = 0;
    while ((nxt = cfg->GetWord()) && (opts.length() == 0)) {
       opts = nxt;
    }
    if (opts.length() > 0) fDataDirOpts = opts;
+   // Check if URL type options have been spcified in the main url
+   int iq = STR_NPOS;
+   if ((iq = fDataDir.rfind('?')) != STR_NPOS) {
+      fDataDirUrlOpts.assign(fDataDir, iq + 1);
+      fDataDir.erase(iq);
+   }
 
    // Done
    return 0;
