@@ -1,4 +1,4 @@
-// @(#)root/proofplayer:$Id: TPacketizerUnit.cxx 44010 2012-04-29 23:07:48Z ganis $
+// @(#)root/proofplayer:$Id$
 // Author: Long Tran-Thanh    22/07/07
 // Revised: G. Ganis, May 2011
 
@@ -94,6 +94,7 @@ TPacketizerUnit::TSlaveStat::TSlaveStat(TSlave *slave, TList *input)
 
    // Initialize the circularity ntple for speed calculations
    fCircNtp = new TNtupleD("Speed Circ Ntp", "Circular process info","tm:ev");
+   fCircNtp->SetDirectory(0);
    TProof::GetParameter(input, "PROOF_TPacketizerUnitCircularity", fCircLvl);
    fCircLvl = (fCircLvl > 0) ? fCircLvl : 5;
    fCircNtp->SetCircular(fCircLvl);
@@ -106,7 +107,6 @@ TPacketizerUnit::TSlaveStat::~TSlaveStat()
 {
    // Destructor
 
-   fCircNtp->SetDirectory(0);
    SafeDelete(fCircNtp);
 }
 
@@ -214,6 +214,7 @@ TPacketizerUnit::TPacketizerUnit(TList *slaves, Long64_t num, TList *input,
    
    fProcessing = 0;
    fAssigned = 0;
+   fPacketSeq = 0;
 
    fStopwatch = new TStopwatch();
 
@@ -491,15 +492,19 @@ TDSetElement *TPacketizerUnit::GetNextPacket(TSlave *sl, TMessage *r)
    fProcessing = (num < (fTotalEntries - fAssigned)) ? num
                                                      : (fTotalEntries - fAssigned);
 
-   // Set the informations of the current slave
+   // Set the information of the current slave
    slstat->fLastProcessed = fProcessing;
    // Set the start time of the current packet
    slstat->fTimeInstant = cTime;
+   
+   // Update the sequential number
+   fPacketSeq++;
+   TString sseq = TString::Format("p%lld", fPacketSeq);
 
    PDB(kPacketizer,2)
       Info("GetNextPacket", "worker-%s: num %lld, processing %lld, remaining %lld",sl->GetOrdinal(),
                             num, fProcessing, (fTotalEntries - fAssigned - fProcessing));
-   TDSetElement *elem = new TDSetElement("", "", "", fAssigned, fProcessing);
+   TDSetElement *elem = new TDSetElement(sseq, sseq, "", fAssigned, fProcessing);
    elem->SetBit(TDSetElement::kEmpty);
 
    // Update the total counter
