@@ -1,4 +1,4 @@
-// @(#)root/proofplayer:$Id: TProofPlayerLite.cxx 44523 2012-06-04 15:17:29Z ganis $
+// @(#)root/proofplayer:$Id$
 // Author: G. Ganis Mar 2008
 
 /*************************************************************************
@@ -387,6 +387,7 @@ Long64_t TProofPlayerLite::Finalize(Bool_t force, Bool_t sync)
          fInput->Remove(fSelector);
          fOutput->Remove(fSelector);
          if (output) output->Remove(fSelector);
+         fSelector = 0;
       }
 
       // We have transferred copy of the output objects in TQueryResult,
@@ -404,7 +405,8 @@ Long64_t TProofPlayerLite::Finalize(Bool_t force, Bool_t sync)
       // Cleanup
       fOutput->SetOwner();
       SafeDelete(fSelector);
-   }
+      if (!fCreateSelObj) fSelector = 0;
+    }
 
    PDB(kGlobal,1) Info("Finalize","exit");
    return rv;
