@@ -1,4 +1,4 @@
-// @(#)root/proofx:$Id: TXProofMgr.cxx 44523 2012-06-04 15:17:29Z ganis $
+// @(#)root/proofx:$Id$
 // Author: Gerardo Ganis  12/12/2005
 
 /*************************************************************************
@@ -427,7 +427,7 @@ TList *TXProofMgr::QuerySessions(Option_t *opt)
             Int_t id = -1, st = -1;
             TString al, tg, tk;
             Ssiz_t from = 0;
-            while ((to->GetString()[from] == ' ')) { from++; }
+            while (to->GetString()[from] == ' ') { from++; }
             if (!to->GetString().Tokenize(tk, from, " ") || !tk.IsDigit()) continue;
             id = tk.Atoi();
             if (!to->GetString().Tokenize(tg, from, " ")) continue;
