@@ -1,4 +1,4 @@
-// @(#)root/proofx:$Id: TXProofServ.cxx 44523 2012-06-04 15:17:29Z ganis $
+// @(#)root/proofx:$Id$
 // Author: Gerardo Ganis  12/12/2005
 
 /*************************************************************************
@@ -674,11 +674,30 @@ Int_t TXProofServ::Setup()
    fWorkDir = gEnv->GetValue("ProofServ.Sandbox", Form("~/%s", kPROOF_WorkDir));
 
    // Get Session tag
-   if ((fTopSessionTag = gEnv->GetValue("ProofServ.SessionTag", "-1")) == "-1") {
+   if ((fSessionTag = gEnv->GetValue("ProofServ.SessionTag", "-1")) == "-1") {
       Error("Setup", "Session tag missing");
       return -1;
    }
-   fSessionTag = fTopSessionTag;
+   // Get top session tag, i.e. the tag of the PROOF session
+   if ((fTopSessionTag = gEnv->GetValue("ProofServ.TopSessionTag", "-1")) == "-1") {
+      fTopSessionTag = "";
+      // Try to extract it from log file path (for backward compatibility)
+      if (gSystem->Getenv("ROOTPROOFLOGFILE")) {
+         fTopSessionTag = gSystem->DirName(gSystem->Getenv("ROOTPROOFLOGFILE"));
+         Ssiz_t lstl;
+         if ((lstl = fTopSessionTag.Last('/')) != kNPOS) fTopSessionTag.Remove(0, lstl + 1);
+         if (fTopSessionTag.BeginsWith("session-")) {
+            fTopSessionTag.Remove(0, strlen("session-"));
+         } else {
+            fTopSessionTag = "";
+         }
+      }
+      if (fTopSessionTag.IsNull()) {
+         Error("Setup", "top session tag missing");
+         return -1;
+      }
+   }
+
    // Make sure the process ID is in the tag
    TString spid = Form("-%d", gSystem->GetPid());
    if (!fSessionTag.EndsWith(spid)) {
@@ -821,35 +840,42 @@ TProofServ::EQueryAction TXProofServ::GetWorkers(TList *workers,
                SafeDelete(master);
             }
             // Now the workers
-            while (fl.Tokenize(tok, from, "&") && (nwrks == -1 || nwrks > 0)) {
+            while (fl.Tokenize(tok, from, "&")) {
                if (!tok.IsNull()) {
-                  // We have the minimal set of information to start
-                  rc = kQueryOK;
-                  if (pernode && nodecnt) {
-                     TProofNodeInfo *ni = new TProofNodeInfo(tok);
-                     TParameter<Int_t> *p = 0;
-                     Int_t nw = 0;
-                     if (!(p = (TParameter<Int_t> *) nodecnt->FindObject(ni->GetNodeName().Data()))) {
-                        p = new TParameter<Int_t>(ni->GetNodeName().Data(), nw);
-                        nodecnt->Add(p);
-                     }
-                     nw = p->GetVal();
-                     if (gDebug > 0)
-                        Info("GetWorkers","%p: name: %s (%s) val: %d (nwrks: %d)",
-                                          p, p->GetName(), ni->GetNodeName().Data(),  nw, nwrks);
-                     if (nw < nwrks) {
-                        if (workers) workers->Add(ni);
-                        nw++;
-                        p->SetVal(nw);
+                  if (nwrks == -1 || nwrks > 0) {
+                     // We have the minimal set of information to start
+                     rc = kQueryOK;
+                     if (pernode && nodecnt) {
+                        TProofNodeInfo *ni = new TProofNodeInfo(tok);
+                        TParameter<Int_t> *p = 0;
+                        Int_t nw = 0;
+                        if (!(p = (TParameter<Int_t> *) nodecnt->FindObject(ni->GetNodeName().Data()))) {
+                           p = new TParameter<Int_t>(ni->GetNodeName().Data(), nw);
+                           nodecnt->Add(p);
+                        }
+                        nw = p->GetVal();
+                        if (gDebug > 0)
+                           Info("GetWorkers","%p: name: %s (%s) val: %d (nwrks: %d)",
+                                             p, p->GetName(), ni->GetNodeName().Data(),  nw, nwrks);
+                        if (nw < nwrks) {
+                           if (workers) workers->Add(ni);
+                           nw++;
+                           p->SetVal(nw);
+                        } else {
+                           // Two many workers on this machine already
+                           SafeDelete(ni);
+                        }
                      } else {
-                        // Two many workers on this machine already
-                        SafeDelete(ni);
+                        if (workers)
+                           workers->Add(new TProofNodeInfo(tok));
+                        // Count down
+                        if (nwrks != -1) nwrks--;
                      }
                   } else {
-                     if (workers)
-                        workers->Add(new TProofNodeInfo(tok));
-                     // Count down
-                     if (nwrks != -1) nwrks--;
+                     // Release this worker (to cleanup the session list in the coordinator and get a fresh
+                     // and correct list next call) 
+                     TProofNodeInfo *ni = new TProofNodeInfo(tok);
+                     ReleaseWorker(ni->GetOrdinal().Data());
                   }
                }
             }
@@ -1149,7 +1175,7 @@ void TXProofServ::ReleaseWorker(const char *ord)
    // Send message to intermediate coordinator to release worker of last ordinal
    // ord.
 
-   Info("ReleaseWorker","releasing: %s", ord);
+   if (gDebug > 2) Info("ReleaseWorker","releasing: %s", ord);
 
    ((TXSocket *)fSocket)->SendCoordinator(kReleaseWorker, ord);
 }
