@@ -1,3 +1,12 @@
+// @(#)root/roostats:$Id$
+// Author: George Lewis, Kyle Cranmer
+/*************************************************************************
+ * Copyright (C) 1995-2008, Rene Brun and Fons Rademakers.               *
+ * All rights reserved.                                                  *
+ *                                                                       *
+ * For the licensing terms see $ROOTSYS/LICENSE.                         *
+ * For the list of contributors see $ROOTSYS/README/CREDITS.             *
+ *************************************************************************/
 
 #ifndef HISTFACTORY_DATA_H
 #define HISTFACTORY_DATA_H
@@ -17,8 +26,11 @@ class Data {
 public:
   //friend class Channel;
 
-  Data() {;}
+  Data();
   Data( std::string HistoName, std::string InputFile, std::string HistoPath="" );
+
+  std::string GetName() { return fName; }
+  void SetName(const std::string& name) { fName=name; }
   
   void SetInputFile(const std::string& InputFile) { fInputFile = InputFile; }
   std::string GetInputFile() { return fInputFile; }
@@ -37,6 +49,8 @@ public:
   void SetHisto(TH1* Hist) { fhData = Hist; fHistoName=Hist->GetName(); }
   
 protected:
+
+  std::string fName;
   
   std::string fInputFile;
   std::string fHistoName;
