@@ -3,7 +3,12 @@
 #include "RooStats/HistFactory/Data.h"
 
 
-RooStats::HistFactory::Data::Data( std::string HistoName, std::string InputFile, std::string HistoPath ) :
+RooStats::HistFactory::Data::Data() : fName("") {
+  ;
+}
+
+RooStats::HistFactory::Data::Data( std::string HistoName, std::string InputFile, 
+				   std::string HistoPath ) :
   fInputFile( InputFile ), fHistoName( HistoName ), fHistoPath( HistoPath ) {;}
 
 TH1* RooStats::HistFactory::Data::GetHisto() {
@@ -24,15 +29,18 @@ void RooStats::HistFactory::Data::Print( std::ostream& stream ) {
 void RooStats::HistFactory::Data::writeToFile( std::string OutputFileName, std::string DirName ) {
 
   TH1* histData = GetHisto();
-  histData->Write();
-  
-  // Set the location of the data
-  // in the output measurement
-  
-  fInputFile = OutputFileName;
-  fHistoName = histData->GetName();
-  fHistoPath = DirName;
 
-  return;
+  if( histData != NULL) {
+
+    histData->Write();
+  
+    // Set the location of the data
+    // in the output measurement
+  
+    fInputFile = OutputFileName;
+    fHistoName = histData->GetName();
+    fHistoPath = DirName;
+
+  }
 
 }
