@@ -40,11 +40,13 @@
 #include "RooWorkspace.h"
 #include "RooBinning.h"
 
+using namespace std;
+
 ClassImp(ParamHistFunc);
 
 
 //_____________________________________________________________________________
-ParamHistFunc::ParamHistFunc() : _numBins(0)
+ParamHistFunc::ParamHistFunc() : _numBins(0), _Normalized(false)
 {
   ;
 }
@@ -55,9 +57,10 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
 			     const RooArgList& vars, const RooArgList& paramSet) :
   RooAbsReal(name, title),
   _dataVars("!dataVars","data Vars",       this),
-  _paramSet("!paramSet","bin paramaters",  this),
+  _paramSet("!paramSet","bin parameters",  this),
   _numBins(0),
-  _dataSet( (string(name)+"_dataSet").c_str(), "", vars) 
+  _dataSet( (string(name)+"_dataSet").c_str(), "", vars),
+  _Normalized( false )
 {
   
   // Create a function which returns binewise-values
@@ -79,13 +82,13 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
   // Set the binning
   // //_binning = var.getBinning().clone() ;
   
-  // Create the set of paramaters
+  // Create the set of parameters
   // controlling the height of each bin
 
   // Get the number of bins
   _numBins = GetNumBins( vars );
 
-  // Add the paramaters (with checking)
+  // Add the parameters (with checking)
   addVarSet( vars );
   addParamSet( paramSet );
 
@@ -100,9 +103,10 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
   RooAbsReal(name, title),
   //  _dataVar("!dataVar","data Var", this, (RooRealVar&) var),
   _dataVars("!dataVars","data Vars",       this),
-  _paramSet("!paramSet","bin paramaters",  this),
+  _paramSet("!paramSet","bin parameters",  this),
   _numBins(0),
-  _dataSet( (string(name)+"_dataSet").c_str(), "", vars, Hist) 
+  _dataSet( (string(name)+"_dataSet").c_str(), "", vars, Hist),
+  _Normalized( false )
 {
 
   // Create a function which returns binewise-values
@@ -120,7 +124,7 @@ ParamHistFunc::ParamHistFunc(const char* name, const char* title,
   // Get the number of bins
   _numBins = GetNumBins( vars );
 
-  // Add the paramaters (with checking)
+  // Add the parameters (with checking)
   addVarSet( vars );
   addParamSet( paramSet );
  
@@ -163,7 +167,8 @@ ParamHistFunc::ParamHistFunc(const ParamHistFunc& other, const char* name) :
   _paramSet("!paramSet", this, other._paramSet),
   _numBins( other._numBins ),
   _binMap( other._binMap ),
-  _dataSet( other._dataSet )
+  _dataSet( other._dataSet ),
+  _Normalized( other._Normalized )
 {
   ;
   // Copy constructor
@@ -274,8 +279,6 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
       VarNameStream << Prefix << "_bin_" << i;
       std::string VarName = VarNameStream.str();
 
-
-
       RooRealVar gamma( VarName.c_str(), VarName.c_str(), 1.0 ); 
       // "Hard-Code" a minimum of 0.0
       gamma.setMin( 0.0 );
@@ -289,7 +292,6 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
     }
   }
 
-
   else if( numVars == 2 ) {
  
     // Create a vector of indices
@@ -363,8 +365,6 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
     }
   }
 
-    
-
   else {
  
     cout << " Error: ParamHistFunc doesn't support dimensions > 3D " <<  endl;
@@ -428,7 +428,6 @@ RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Pre
 RooArgList ParamHistFunc::createParamSet(RooWorkspace& w, const std::string& Prefix, const RooArgList& vars, 
 					 Double_t gamma_min, Double_t gamma_max) {
 
-
   RooArgList params = ParamHistFunc::createParamSet( w, Prefix, vars );
 
   RooFIter paramIter = params.fwdIterator() ;
@@ -516,7 +515,6 @@ RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBin
 
   // Get the number of bins
   // in the nominal histogram
- 
 
   RooArgList paramSet;
 
@@ -539,7 +537,6 @@ RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBin
     gamma_nominal = gamma_max;
   }
 
-
   // For each bin, create a RooRealVar
   for( Int_t i = 0; i < numBins; ++i) {
 
@@ -554,7 +551,6 @@ RooArgList ParamHistFunc::createParamSet(const std::string& Prefix, Int_t numBin
 
   }
 
-
   return paramSet;
 
 }
@@ -587,7 +583,6 @@ Int_t ParamHistFunc::addVarSet( const RooArgList& vars ) {
 
   }
 
-
   Int_t numBinsX = 1;
   Int_t numBinsY = 1;
   Int_t numBinsZ = 1;
@@ -634,7 +629,6 @@ Int_t ParamHistFunc::addVarSet( const RooArgList& vars ) {
       }
     }
   }
-
   
   return 0;
 
@@ -654,7 +648,7 @@ Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
 
   if( numVarBins != numElements ) {
     std::cout << "ParamHistFunc::addParamSet - ERROR - " 
-	      << "Supplied list of paramaters " << params.GetName()
+	      << "Supplied list of parameters " << params.GetName()
 	      << " has " << numElements << " elements but the ParamHistFunc"
 	      << GetName() << " has " << numVarBins << " bins."
 	      << std::endl;
@@ -672,7 +666,7 @@ Int_t ParamHistFunc::addParamSet( const RooArgList& params ) {
   while((comp = (RooAbsArg*) paramIter.next())) {
     if (!dynamic_cast<RooRealVar*>(comp)) {
       coutE(InputArguments) << "ParamHistFunc::(" << GetName() << ") ERROR: component " << comp->GetName() 
-			    << " in paramater list is not of type RooRealVar" << std::endl ;
+			    << " in parameter list is not of type RooRealVar" << std::endl ;
       RooErrorHandler::softAbort() ;
       return 1;
     }
@@ -702,14 +696,14 @@ Double_t ParamHistFunc::evaluate() const
 
   Double_t value = param->getVal();
 
-  /*
-  cout << "ParamHistFunc - Current Bin: " << currentBin
-       << " param pointer: " << param
-       << " param val: " << paramVal
-       << " val: " << value
-       << endl;
-  */
-  return value;
+  // If we don't require the function to
+  // be normalized, return right away
+  if( !_Normalized ) return value;
+  
+  // Else, we divide the value by the integral
+  // which effectively normalizes the function over bins
+  Double_t scale = 1.0 / analyticalIntegralWN(0, NULL, NULL);
+  return scale*value;
   
 }
 
@@ -724,14 +718,12 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
   if (allVars.getSize()==0) return 0 ;
   if (_forceNumInt) return 0 ;
 
-
   // Select subset of allVars that are actual dependents
   analVars.add(allVars) ;
   //  RooArgSet* normSet = normSet2 ? getObservables(normSet2) : 0 ;
   //  RooArgSet* normSet = getObservables();
   //  RooArgSet* normSet = 0;
 
-
   // Check if this configuration was created before
   Int_t sterileIdx(-1) ;
   CacheElem* cache = (CacheElem*) _normIntMgr.getObj(normSet,&analVars,&sterileIdx,(const char*)0) ;
@@ -747,7 +739,6 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
 
   return code+1 ; 
 
-
   // Make list of function projection and normalization integrals 
   //  RooAbsReal* param ;
   // RooAbsReal *func ;
@@ -759,7 +750,6 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
   RooAbsReal* funcInt = func->createIntegral(analVars) ;
   cache->_funcIntList.addOwned(*funcInt) ;
 
-
   // Implement integration here
 
   // do variations
@@ -786,8 +776,6 @@ Int_t ParamHistFunc::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& anal
 }
 
 
-
-
 //_____________________________________________________________________________
 Double_t ParamHistFunc::analyticalIntegralWN(Int_t /*code*/, const RooArgSet* /*normSet2*/,const char* /*rangeName*/) const 
 {
