@@ -62,7 +62,7 @@ public:
 
   // Return value and unit accessors
   inline Double_t getVal(const RooArgSet* set=0) const { 
-/*    if (_fast && !_inhibitDirty) cout << "RooAbsReal::getVal(" << GetName() << ") CLEAN value = " << _value << endl ;  */
+/*     if (_fast && !_inhibitDirty && std::string("RooHistFunc")==IsA()->GetName()) std::cout << "RooAbsReal::getVal(" << GetName() << ") CLEAN value = " << _value << std::endl ;  */
 #ifndef _WIN32
     return (_fast && !_inhibitDirty) ? _value : getValV(set) ; 
 #else
@@ -77,6 +77,9 @@ public:
 
   Bool_t operator==(Double_t value) const ;
   virtual Bool_t operator==(const RooAbsArg& other) ;
+  virtual Bool_t isIdentical(const RooAbsArg& other, Bool_t assumeSameType=kFALSE)  ;
+
+
   inline const Text_t *getUnit() const { 
     // Return string with unit description
     return _unit.Data(); 
@@ -108,6 +111,7 @@ public:
     // and all integrals are calculated numerically
     _forceNumInt = flag ; 
   }
+  Bool_t getForceNumInt() const { return _forceNumInt ; }
 
   // Chi^2 fits to histograms
   virtual RooFitResult* chi2FitTo(RooDataHist& data, const RooCmdArg& arg1=RooCmdArg::none(),  const RooCmdArg& arg2=RooCmdArg::none(),  
@@ -235,12 +239,12 @@ public:
 			    Bool_t correctForBinVolume=kFALSE, Bool_t showProgress=kFALSE) const ;
 
   // I/O streaming interface (machine readable)
-  virtual Bool_t readFromStream(istream& is, Bool_t compact, Bool_t verbose=kFALSE) ;
-  virtual void writeToStream(ostream& os, Bool_t compact) const ;
+  virtual Bool_t readFromStream(std::istream& is, Bool_t compact, Bool_t verbose=kFALSE) ;
+  virtual void writeToStream(std::ostream& os, Bool_t compact) const ;
 
   // Printing interface (human readable)
-  virtual void printValue(ostream& os) const ;
-  virtual void printMultiline(ostream& os, Int_t contents, Bool_t verbose=kFALSE, TString indent="") const ;
+  virtual void printValue(std::ostream& os) const ;
+  virtual void printMultiline(std::ostream& os, Int_t contents, Bool_t verbose=kFALSE, TString indent="") const ;
 
   static void setCacheCheck(Bool_t flag) ;
 
@@ -260,7 +264,7 @@ public:
   static void setEvalErrorLoggingMode(ErrorLoggingMode m) ;
   void logEvalError(const char* message, const char* serverValueString=0) const ;
   static void logEvalError(const RooAbsReal* originator, const char* origName, const char* message, const char* serverValueString=0) ;
-  static void printEvalErrors(ostream&os=std::cout, Int_t maxPerNode=10000000) ;
+  static void printEvalErrors(std::ostream&os=std::cout, Int_t maxPerNode=10000000) ;
   static Int_t numEvalErrors() ;
   static Int_t numEvalErrorItems() ;
 
@@ -300,6 +304,13 @@ public:
 
   virtual Bool_t setData(RooAbsData& /*data*/, Bool_t /*cloneData*/=kTRUE) { return kTRUE ; }
 
+  virtual void enableOffsetting(Bool_t) {} ;
+  virtual Bool_t isOffsetting() const { return kFALSE ; }
+  virtual Double_t offset() const { return 0 ; }
+  
+  static void setHideOffset(Bool_t flag);
+  static Bool_t hideOffset() ;
+
 protected:
 
   // PlotOn with command list
@@ -319,7 +330,7 @@ protected:
 
   Bool_t isSelectedComp() const ;
 
-
+  
  public:
   const RooAbsReal* createPlotProjection(const RooArgSet& depVars, const RooArgSet& projVars) const ;
   const RooAbsReal* createPlotProjection(const RooArgSet& depVars, const RooArgSet& projVars, RooArgSet*& cloneSet) const ;
@@ -374,6 +385,7 @@ protected:
   virtual void setTreeBranchStatus(TTree& t, Bool_t active) ;
   virtual void fillTreeBranch(TTree& t) ;
 
+  friend class RooRealBinding ;
   Double_t _plotMin ;       // Minimum of plot range
   Double_t _plotMax ;       // Maximum of plot range
   Int_t    _plotBins ;      // Number of plot bins
@@ -384,6 +396,7 @@ protected:
 
   mutable Float_t _floatValue ; //! Transient cache for floating point values from tree branches 
   mutable Int_t   _intValue   ; //! Transient cache for integer values from tree branches 
+  mutable Bool_t  _boolValue  ; //! Transient cache for bool values from tree branches 
   mutable UChar_t _byteValue  ; //! Transient cache for byte values from tree branches 
   mutable Char_t  _sbyteValue ; //! Transient cache for signed byte values from tree branches 
   mutable UInt_t  _uintValue  ; //! Transient cache for unsigned integer values from tree branches 
@@ -405,7 +418,7 @@ protected:
    PlotOpt() : drawOptions("L"), scaleFactor(1.0), stype(Relative), projData(0), binProjData(kFALSE), projSet(0), precision(1e-3), 
                shiftToZero(kFALSE),projDataSet(0),normRangeName(0),rangeLo(0),rangeHi(0),postRangeFracScale(kFALSE),wmode(RooCurve::Extended),
                projectionRangeName(0),curveInvisible(kFALSE), curveName(0),addToCurveName(0),addToWgtSelf(1.),addToWgtOther(1.),
-               numCPU(1),interleave(kTRUE),curveNameSuffix(""), numee(10), eeval(0), doeeval(kFALSE), progress(kFALSE) {} ;
+               numCPU(1),interleave(RooFit::Interleave),curveNameSuffix(""), numee(10), eeval(0), doeeval(kFALSE), progress(kFALSE) {} ;
    Option_t* drawOptions ;
    Double_t scaleFactor ;	 
    ScaleType stype ;
@@ -427,7 +440,7 @@ protected:
    Double_t addToWgtSelf ;
    Double_t addToWgtOther ;
    Int_t    numCPU ;
-   Bool_t interleave ;
+   RooFit::MPSplit interleave ;
    const char* curveNameSuffix ; 
    Int_t    numee ;
    Double_t eeval ;
@@ -463,7 +476,7 @@ protected:
   static Bool_t _globalSelectComp ;  // Global activation switch for component selection
 
   mutable RooArgSet* _lastNSet ; //!
-
+  static Bool_t _hideOffset ; // Offset hiding flag
 
   ClassDef(RooAbsReal,2) // Abstract real-valued variable
 };
