@@ -1,7 +1,7 @@
 /******************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooAbsArg.cxx 42171 2011-11-22 16:38:52Z moneta $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -76,9 +76,12 @@ ClassImp(RooAbsArg)
 
 Bool_t RooAbsArg::_verboseDirty(kFALSE) ;
 Bool_t RooAbsArg::_inhibitDirty(kFALSE) ;
-
 Bool_t RooAbsArg::inhibitDirty() { return _inhibitDirty ; }
 
+std::map<RooAbsArg*,TRefArray*> RooAbsArg::_ioEvoList ;
+
+
+
 //_____________________________________________________________________________
 RooAbsArg::RooAbsArg() :
   TNamed(),
@@ -88,7 +91,8 @@ RooAbsArg::RooAbsArg() :
   _ownedComponents(0),
   _prohibitServerRedirect(kFALSE),
   _eocache(0),
-  _namePtr(0)
+  _namePtr(0),
+  _isConstant(kFALSE)
 {
   // Default constructor
 
@@ -111,7 +115,8 @@ RooAbsArg::RooAbsArg(const char *name, const char *title) :
   _ownedComponents(0),
   _prohibitServerRedirect(kFALSE),
   _eocache(0),
-  _namePtr(0)
+  _namePtr(0),
+  _isConstant(kFALSE)
 {
   // Create an object with the specified name and descriptive title.
   // The newly created object has no clients or servers and has its
@@ -137,7 +142,8 @@ RooAbsArg::RooAbsArg(const RooAbsArg& other, const char* name)
     _ownedComponents(0),
     _prohibitServerRedirect(kFALSE),
     _eocache(other._eocache),
-    _namePtr(other._namePtr)
+    _namePtr(other._namePtr),
+    _isConstant(other._isConstant)
 {
   // Copy constructor transfers all boolean and string properties of the original
   // object. Transient properties and client-server links are not copied
@@ -247,12 +253,14 @@ void RooAbsArg::setAttribute(const Text_t* name, Bool_t value)
 {
   // Set (default) or clear a named boolean attribute of this object.
 
+  // Preserve backward compatibility - any strong 
+  if(string("Constant")==name) {
+    _isConstant = value ;
+  }
+  
   if (value) {
-
     _boolAttrib.insert(name) ;
-
-  } else {
-
+  } else {    
     set<string>::iterator iter = _boolAttrib.find(name) ;
     if (iter != _boolAttrib.end()) {
       _boolAttrib.erase(iter) ;
@@ -499,6 +507,7 @@ void RooAbsArg::treeNodeServerList(RooAbsCollection* list, const RooAbsArg* arg,
   if ((doBranch&&doLeaf) ||
       (doBranch&&arg->isDerived()) ||
       (doLeaf&&arg->isFundamental()&&(!(recurseFundamental&&arg->isDerived()))) ) {
+
     list->add(*arg,kTRUE) ;
   }
 
@@ -1200,7 +1209,11 @@ void RooAbsArg::registerProxy(RooListProxy& proxy)
   }
 
   // Register proxy itself
+  Int_t nProxyOld = _proxyList.GetEntries() ;
   _proxyList.Add(&proxy) ;
+  if (_proxyList.GetEntries()!=nProxyOld+1) {
+    cout << "RooAbsArg::registerProxy(" << GetName() << ") proxy registration failure! nold=" << nProxyOld << " nnew=" << _proxyList.GetEntries() << endl ;
+  }
 }
 
 
@@ -2287,7 +2300,111 @@ void RooAbsArg::Streamer(TBuffer &R__b)
    if (R__b.IsReading()) {
       R__b.ReadClassBuffer(RooAbsArg::Class(),this);
       _namePtr = (TNamed*) RooNameReg::instance().constPtr(GetName()) ;  
+      _isConstant = getAttribute("Constant") ;
    } else {
       R__b.WriteClassBuffer(RooAbsArg::Class(),this);
    }
 }
+
+//______________________________________________________________________________
+void RooAbsArg::ioStreamerPass2() 
+{
+  // Method called by workspace container to finalize schema evolution issues
+  // that cannot be handled in a single ioStreamer pass.
+  //
+  // A second pass is typically needed when evolving data member of RooAbsArg-derived
+  // classes that are container classes with references to other members, which may
+  // not yet be 'live' in the first ioStreamer() evolution pass.
+  //
+  // Classes may overload this function, but must call the base method in the
+  // overloaded call to ensure base evolution is handled properly
+
+
+  // Handling of v5-v6 migration (TRefArray _proxyList --> RooRefArray _proxyList)
+  map<RooAbsArg*,TRefArray*>::iterator iter = _ioEvoList.find(this) ;
+  if (iter != _ioEvoList.end()) {
+
+    // Transfer contents of saved TRefArray to RooRefArray now
+    for (int i=0 ; i < iter->second->GetEntries() ; i++) {
+      _proxyList.Add(iter->second->At(i)) ;
+    }
+    // Delete TRefArray and remove from list
+    delete iter->second ;
+    _ioEvoList.erase(iter) ;
+  }
+}
+
+
+
+
+//______________________________________________________________________________
+void RooAbsArg::ioStreamerPass2Finalize() 
+{
+  // Method called by workspace container to finalize schema evolution issues
+  // that cannot be handled in a single ioStreamer pass. This static finalize method
+  // is called after ioStreamerPass2() is called on each directly listed object 
+  // in the workspace. It's purpose is to complete schema evolution of any
+  // objects in the workspace that are not directly listed as content elements
+  // (e.g. analytical convolution tokens )
+  
+
+  // Handling of v5-v6 migration (TRefArray _proxyList --> RooRefArray _proxyList)
+  map<RooAbsArg*,TRefArray*>::iterator iter = _ioEvoList.begin() ;
+  while (iter != _ioEvoList.end()) {
+    
+    // Transfer contents of saved TRefArray to RooRefArray now
+    for (int i=0 ; i < iter->second->GetEntries() ; i++) {
+      iter->first->_proxyList.Add(iter->second->At(i)) ;
+    }
+
+    // Save iterator position for deletion after increment
+    map<RooAbsArg*,TRefArray*>::iterator iter_tmp = iter ;
+    
+    iter++ ;
+    
+    // Delete TRefArray and remove from list
+    delete iter_tmp->second ;
+    _ioEvoList.erase(iter_tmp) ;
+    
+  }
+
+}
+
+
+//______________________________________________________________________________
+void RooRefArray::Streamer(TBuffer &R__b)
+{
+   // Stream an object of class RooRefArray.
+
+   UInt_t R__s, R__c;
+   if (R__b.IsReading()) {
+
+      Version_t R__v = R__b.ReadVersion(&R__s, &R__c); if (R__v) { }      
+
+      // Make temporary refArray and read that from the streamer 
+      TRefArray refArray ;
+      refArray.Streamer(R__b) ;
+      R__b.CheckByteCount(R__s, R__c, refArray.IsA());
+
+      // Transfer contents to ourselves
+      TIterator* iter = refArray.MakeIterator() ; 
+      TObject* tmpObj ; while ((tmpObj = iter->Next())) { Add(tmpObj) ; } 
+      delete iter ; 
+      
+   } else {
+
+     R__c = R__b.WriteVersion(RooRefArray::IsA(), kTRUE);
+
+     // Make a temporary refArray and write that to the streamer
+     TRefArray refArray ;
+     TIterator* iter = MakeIterator() ; 
+     TObject* tmpObj ; while ((tmpObj = iter->Next())) { refArray.Add(tmpObj) ; } 
+     delete iter ; 
+
+     refArray.Streamer(R__b) ;
+     R__b.SetByteCount(R__c, kTRUE) ;
+     
+   }
+}
+
+
