@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooAbsCollection.cxx 43071 2012-02-21 17:01:01Z axel $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -215,6 +215,7 @@ RooAbsCollection* RooAbsCollection::snapshot(Bool_t deepCopy) const
     delete output ;
     return 0 ;
   }
+  output->setHashTableSize(0) ;
   return output ;
 }
 
@@ -623,8 +624,13 @@ Bool_t RooAbsCollection::remove(const RooAbsArg& var, Bool_t , Bool_t matchByNam
       anyFound=kTRUE ;
     } else if (matchByNameOnly) {
       if (!name.CompareTo(arg->GetName())) {
+	TObject* contObj = _list.FindObject(arg) ;	  
 	_list.Remove(arg) ;
 	anyFound=kTRUE ;
+	if (_ownCont && contObj) {
+	  //cout << "RooAbsCollection::remove() deleting instance " << contObj << " named " << contObj->GetName() << endl ;
+	  delete contObj ;
+	}
       }
     }
   }
