@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooAbsData.cxx 44447 2012-06-01 09:58:22Z moneta $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -42,6 +42,7 @@
 #include "RooAbsDataStore.h"
 #include "RooVectorDataStore.h"
 #include "RooTreeDataStore.h"
+#include "RooDataHist.h"
 #include "RooCompositeDataStore.h"
 #include "RooCategory.h"
 
@@ -58,6 +59,8 @@
 #include "TH3.h"
 
 
+using namespace std;
+
 ClassImp(RooAbsData)
 ;
 
@@ -138,7 +141,7 @@ RooAbsData::RooAbsData(const char *name, const char *title, const RooArgSet& var
   }
   delete iter ;
 
-  // reconnect any paramaterized ranges to internal dataset observables
+  // reconnect any parameterized ranges to internal dataset observables
   iter = _vars.createIterator() ;
   while((0 != (var= (RooAbsArg*)iter->Next()))) {
     var->attachDataSet(*this) ;
@@ -163,7 +166,7 @@ RooAbsData::RooAbsData(const RooAbsData& other, const char* newname) :
   claimVars(this) ;
   _vars.addClone(other._vars) ;
 
-  // reconnect any paramaterized ranges to internal dataset observables
+  // reconnect any parameterized ranges to internal dataset observables
   TIterator* iter = _vars.createIterator() ;
   RooAbsArg* var ;
   while((0 != (var= (RooAbsArg*)iter->Next()))) {
@@ -810,7 +813,9 @@ Roo1DTable* RooAbsData::table(const RooArgSet& catSet, const char* cuts, const c
   RooAbsArg* arg ;
   while((arg=(RooAbsArg*)iter->Next())) {
     if (dynamic_cast<RooAbsCategory*>(arg)) {
-      catSet2.add(*arg) ;
+      RooAbsCategory* varsArg = dynamic_cast<RooAbsCategory*>(_vars.find(arg->GetName())) ;
+      if (varsArg != 0) catSet2.add(*varsArg) ;
+      else catSet2.add(*arg) ;
       if (prodName.length()>1) {
 	prodName += " x " ;
       }
@@ -1605,7 +1610,7 @@ TList* RooAbsData::split(const RooAbsCategory& splitCat, Bool_t createEmptyDataS
   // variable exists (can happen with composite datastores)
   Bool_t addWV(kFALSE) ;
   RooRealVar newweight("weight","weight",-1e9,1e9) ;
-  if (isWeighted()) {
+  if (isWeighted() && !IsA()->InheritsFrom(RooDataHist::Class())) {
     subsetVars.add(newweight) ;
     addWV = kTRUE ;
   }
