@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooAbsOptTestStatistic.cxx 43190 2012-03-01 12:09:43Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -61,6 +61,8 @@
 #include "RooTrace.h"
 #include "RooVectorDataStore.h" 
 
+using namespace std;
+
 ClassImp(RooAbsOptTestStatistic)
 ;
 
@@ -93,7 +95,7 @@ RooAbsOptTestStatistic:: RooAbsOptTestStatistic()
 //_____________________________________________________________________________
 RooAbsOptTestStatistic::RooAbsOptTestStatistic(const char *name, const char *title, RooAbsReal& real, RooAbsData& indata,
 					       const RooArgSet& projDeps, const char* rangeName, const char* addCoefRangeName,
-					       Int_t nCPU, Bool_t interleave, Bool_t verbose, Bool_t splitCutRange, Bool_t /*cloneInputData*/) : 
+					       Int_t nCPU, RooFit::MPSplit interleave, Bool_t verbose, Bool_t splitCutRange, Bool_t /*cloneInputData*/) : 
   RooAbsTestStatistic(name,title,real,indata,projDeps,rangeName, addCoefRangeName, nCPU, interleave, verbose, splitCutRange),
   _projDeps(0),
   _sealed(kFALSE), 
@@ -277,10 +279,6 @@ void RooAbsOptTestStatistic::initSlave(RooAbsReal& real, RooAbsData& indata, con
   }
   _ownData = kTRUE ;  
  
-  _dataClone->attachBuffers(*_funcObsSet) ;
-  setEventCount(_dataClone->numEntries()) ;
-
-
 
   // ******************************************************************
   // *** PART 3 *** Make adjustments for fit ranges, if specified     *
@@ -290,7 +288,7 @@ void RooAbsOptTestStatistic::initSlave(RooAbsReal& real, RooAbsData& indata, con
   RooArgSet* dataObsSet = (RooArgSet*) _dataClone->get() ;
   if (rangeName && strlen(rangeName)) {    
     cxcoutI(Fitting) << "RooAbsOptTestStatistic::ctor(" << GetName() << ") constructing test statistic for sub-range named " << rangeName << endl ;    
-//     cout << "now adjusting observable ranges to requested fit range" << endl ;
+    //cout << "now adjusting observable ranges to requested fit range" << endl ;
 
     // Adjust FUNC normalization ranges to requested fitRange, store original ranges for RooAddPdf coefficient interpretation
     iter = _funcObsSet->fwdIterator() ;
@@ -308,8 +306,8 @@ void RooAbsOptTestStatistic::initSlave(RooAbsReal& real, RooAbsData& indata, con
 
 	// Adjust range of function observable to those of given named range
 	realObs->setRange(realObs->getMin(rangeName),realObs->getMax(rangeName)) ;	
-// 	cout << "RAOTS::ctor() setting normalization range on observable " 
-// 	     << realObs->GetName() << " to [" << realObs->getMin() << "," << realObs->getMax() << "]" << endl ;
+//  	cout << "RAOTS::ctor() setting normalization range on observable " 
+//  	     << realObs->GetName() << " to [" << realObs->getMin() << "," << realObs->getMax() << "]" << endl ;
 
 	// Adjust range of data observable to those of given named range
 	RooRealVar* dataObs = (RooRealVar*) dataObsSet->find(realObs->GetName()) ;
@@ -359,6 +357,12 @@ void RooAbsOptTestStatistic::initSlave(RooAbsReal& real, RooAbsData& indata, con
   }
 
 
+  // This is deferred from part 2 - but must happen after part 3 - otherwise invalid bins cannot be properly marked in cacheValidEntries
+  _dataClone->attachBuffers(*_funcObsSet) ;
+  setEventCount(_dataClone->numEntries()) ;
+
+
+
 
   // *********************************************************************
   // *** PART 4 *** Adjust normalization range for projected observables *
@@ -369,17 +373,17 @@ void RooAbsOptTestStatistic::initSlave(RooAbsReal& real, RooAbsData& indata, con
 
     _projDeps = (RooArgSet*) projDeps.snapshot(kFALSE) ;
     
-    RooArgSet* tobedel = (RooArgSet*) _normSet->selectCommon(*_projDeps) ;
+    //RooArgSet* tobedel = (RooArgSet*) _normSet->selectCommon(*_projDeps) ;
     _normSet->remove(*_projDeps,kTRUE,kTRUE) ;
 
-    // Delete owned projected dependent copy in _normSet
-    TIterator* ii = tobedel->createIterator() ;
-    RooAbsArg* aa ;
-    while((aa=(RooAbsArg*)ii->Next())) {
-      delete aa ;
-    }
-    delete ii ;
-    delete tobedel ;
+//     // Delete owned projected dependent copy in _normSet
+//     TIterator* ii = tobedel->createIterator() ;
+//     RooAbsArg* aa ;
+//     while((aa=(RooAbsArg*)ii->Next())) {
+//       delete aa ;
+//     }
+//     delete ii ;
+//     delete tobedel ;
 
     // Mark all projected dependents as such
     RooArgSet *projDataDeps = (RooArgSet*) _funcObsSet->selectCommon(*_projDeps) ;
@@ -448,7 +452,7 @@ Double_t RooAbsOptTestStatistic::combinedValue(RooAbsReal** array, Int_t n) cons
   Double_t sum(0) ;
   Int_t i ;
   for (i=0 ; i<n ; i++) {
-    Double_t tmp = array[i]->getVal() ;
+    Double_t tmp = array[i]->getValV() ;
     // if (tmp==0) return 0 ; WVE no longer needed
     sum += tmp ;
   }
@@ -639,6 +643,22 @@ void RooAbsOptTestStatistic::optimizeConstantTerms(Bool_t activate, Bool_t apply
 	      RooAbsArg* parg ;
 	      while ((parg=piter.next())) {
 		parg->setAttribute("CacheAndTrack") ;
+		RooArgSet* pdf_nset = prod->findPdfNSet((RooAbsPdf&)(*parg)) ;
+		
+		if (pdf_nset) {
+		  // Check if conditional normalization is specified		  
+		  if (string("nset")==pdf_nset->GetName() && pdf_nset->getSize()>0) {
+		    RooNameSet n(*pdf_nset) ;
+		    parg->setStringAttribute("CATNormSet",n.content()) ;
+		  }
+		  if (string("cset")==pdf_nset->GetName()) {
+		    RooNameSet c(*pdf_nset) ;
+		    parg->setStringAttribute("CATCondSet",c.content()) ;
+		  }
+		} else {
+		  coutW(Optimization) << "RooAbsOptTestStatistic::optimizeConstantTerms(" << GetName() << ") WARNING RooProdPdf::" << prod->GetName() 
+				      << " does not specify a normalization set for component " << parg->GetName() << endl ;
+		}
 		trackNodes.add(*aarg) ;
 	      }
 	    } else {
