@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooAbsTestStatistic.cxx 43190 2012-03-01 12:09:43Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -47,9 +47,12 @@
 #include "RooRealMPFE.h"
 #include "RooErrorHandler.h"
 #include "RooMsgService.h"
+#include "TTimeStamp.h"
 
 #include <string>
 
+using namespace std;
+
 ClassImp(RooAbsTestStatistic)
 ;
 
@@ -66,15 +69,19 @@ RooAbsTestStatistic::RooAbsTestStatistic()
   _mpfeArray = 0 ;
   _projDeps = 0 ;
   _gofOpMode = Slave ;
-  _mpinterl = kFALSE ;
+  _mpinterl = RooFit::BulkPartition ;
   _nCPU = 1 ;
   _nEvents = 0 ; 
   _nGof = 0 ;
   _numSets = 0 ;
   _setNum = 0 ;
+  _extSet = 0 ;
   _simCount = 0 ;
   _splitRange = 0 ;
   _verbose = kFALSE ;
+  _doOffset = kFALSE ;
+  _offset = 0 ;
+  _offsetCarry = 0;
 }
 
 
@@ -82,7 +89,7 @@ RooAbsTestStatistic::RooAbsTestStatistic()
 //_____________________________________________________________________________
 RooAbsTestStatistic::RooAbsTestStatistic(const char *name, const char *title, RooAbsReal& real, RooAbsData& data,
 					 const RooArgSet& projDeps, const char* rangeName, const char* addCoefRangeName,
-					 Int_t nCPU, Bool_t interleave, Bool_t verbose, Bool_t splitCutRange) :
+					 Int_t nCPU, RooFit::MPSplit interleave, Bool_t verbose, Bool_t splitCutRange) :
   RooAbsReal(name,title),
   _paramSet("paramSet","Set of parameters",this),
   _func(&real),
@@ -97,7 +104,10 @@ RooAbsTestStatistic::RooAbsTestStatistic(const char *name, const char *title, Ro
   _gofArray(0),
   _nCPU(nCPU),
   _mpfeArray(0),
-  _mpinterl(interleave)
+  _mpinterl(interleave),
+  _doOffset(kFALSE),
+  _offset(0),
+  _offsetCarry(0)
 {
   // Constructor taking function (real), a dataset (data), a set of projected observables (projSet). If
   // rangeName is not null, only events in the dataset inside the range will be used in the test
@@ -112,7 +122,6 @@ RooAbsTestStatistic::RooAbsTestStatistic(const char *name, const char *title, Ro
   // If splitCutRange is true, a different rangeName constructed as rangeName_{catName} will be used
   // as range definition for each index state of a RooSimultaneous
 
-
   // Register all parameters as servers
   RooArgSet* params = real.getParameters(&data) ;
   _paramSet.add(*params) ;
@@ -139,6 +148,7 @@ RooAbsTestStatistic::RooAbsTestStatistic(const char *name, const char *title, Ro
   }
 
   _setNum = 0 ;
+  _extSet = 0 ;
   _numSets = 1 ;
   _init = kFALSE ;
   _nEvents = data.numEntries() ;
@@ -160,9 +170,13 @@ RooAbsTestStatistic::RooAbsTestStatistic(const RooAbsTestStatistic& other, const
   _verbose(other._verbose),
   _nGof(0),
   _gofArray(0),
+  _gofSplitMode(other._gofSplitMode),
   _nCPU(other._nCPU),
   _mpfeArray(0),
-  _mpinterl(other._mpinterl)
+  _mpinterl(other._mpinterl),
+  _doOffset(other._doOffset),
+  _offset(other._offset),
+  _offsetCarry(other._offsetCarry)
 {
   // Copy constructor
 
@@ -190,6 +204,7 @@ RooAbsTestStatistic::RooAbsTestStatistic(const RooAbsTestStatistic& other, const
   }
 
   _setNum = 0 ;
+  _extSet = 0 ;
   _numSets = 1 ;
   _init = kFALSE ;
   _nEvents = _data->numEntries() ;
@@ -223,6 +238,7 @@ RooAbsTestStatistic::~RooAbsTestStatistic()
   if (_projDeps) {
     delete _projDeps ;
   }
+
 }
 
 
@@ -244,49 +260,85 @@ Double_t RooAbsTestStatistic::evaluate() const
   if (_gofOpMode==SimMaster) {
 
     // Evaluate array of owned GOF objects
-    Double_t ret = combinedValue((RooAbsReal**)_gofArray,_nGof) ;
+    Double_t ret(0) ;
+
+    if (_mpinterl == RooFit::BulkPartition || _mpinterl == RooFit::Interleave ) {
+      
+      ret = combinedValue((RooAbsReal**)_gofArray,_nGof) ;
+
+    } else {
+
+      Double_t sum(0) ;
+      Int_t i ;
+      for (i=0 ; i<_nGof ; i++) {
+	if (i % _numSets == _setNum || (_mpinterl==RooFit::Hybrid && _gofSplitMode[i] != RooFit::SimComponents )) {
+	  Double_t tmp = _gofArray[i]->getValV() ;
+	  sum += tmp ;
+	}
+      }
+      ret = sum ;
+    }
 
     // Only apply global normalization if SimMaster doesn't have MP master
     if (numSets()==1) {
-//       cout << "RooAbsTestStatistic::evaluate(" << GetName() << ") A dividing ret= " << ret << " by globalNorm of " << globalNormalization() << endl ;
       ret /= globalNormalization() ;
     }
 
     return ret ;
 
   } else if (_gofOpMode==MPMaster) {
-
+    
     // Start calculations in parallel
     Int_t i ;
     for (i=0 ; i<_nCPU ; i++) {
       _mpfeArray[i]->calculate() ;
     }
-    Double_t ret = combinedValue((RooAbsReal**)_mpfeArray,_nCPU)/globalNormalization() ;
+
+
+    Double_t sum(0) ;
+    for (i=0 ; i<_nCPU ; i++) {
+      Double_t tmp = _mpfeArray[i]->getValV() ;
+      sum += tmp ;
+    }
+
+    Double_t ret = sum ;
     return ret ;
 
   } else {
 
     // Evaluate as straight FUNC
-    Int_t nFirst, nLast, nStep ;
-    if (_mpinterl) {
-      nFirst = _setNum ;
-      nLast  = _nEvents ;
-      nStep  = _numSets ;
-    } else {
+    Int_t nFirst(0), nLast(_nEvents), nStep(1) ;
+    
+    switch (_mpinterl) {
+    case RooFit::BulkPartition:
       nFirst = _nEvents * _setNum / _numSets ;
       nLast  = _nEvents * (_setNum+1) / _numSets ;
       nStep  = 1 ;
+      break;
+      
+    case RooFit::Interleave:
+      nFirst = _setNum ;
+      nLast  = _nEvents ;
+      nStep  = _numSets ;
+      break ;
+      
+    case RooFit::SimComponents:
+      nFirst = 0 ;
+      nLast  = _nEvents ;
+      nStep  = 1 ;
+      break ;
+      
+    case RooFit::Hybrid:
+      throw(std::string("this should never happen")) ;
+      break ;
     }
 
-
-    //cout << "nCPU = " << _nCPU << (_mpinterl?"INTERLEAVE":"BULK") << " nFirst = " << nFirst << " nLast = " << nLast << " nStep = " << nStep << endl ;
-
     Double_t ret =  evaluatePartition(nFirst,nLast,nStep) ;
+
     if (numSets()==1) {
-//       cout << "RooAbsTestStatistic::evaluate(" << GetName() << ") B dividing ret= " << ret << " by globalNorm of " << globalNormalization() << endl ;
       ret /= globalNormalization() ;
     }
-
+    
     return ret ;
 
   }
@@ -379,7 +431,11 @@ void RooAbsTestStatistic::constOptimizeTestStatistic(ConstOpCode opcode, Bool_t
   if (_gofOpMode==SimMaster) {
     // Forward to slaves
     for (i=0 ; i<_nGof ; i++) {
-      if (_gofArray[i]) _gofArray[i]->constOptimizeTestStatistic(opcode,doAlsoTrackingOpt) ;
+      // In SimComponents Splitting strategy only constOptimize the terms that are actually used
+      RooFit::MPSplit effSplit = (_mpinterl!=RooFit::Hybrid) ? _mpinterl : _gofSplitMode[i] ;
+      if ( (i % _numSets == _setNum) || (effSplit != RooFit::SimComponents) ) {
+	if (_gofArray[i]) _gofArray[i]->constOptimizeTestStatistic(opcode,doAlsoTrackingOpt) ;
+      }
     }
   } else if (_gofOpMode==MPMaster) {
     for (i=0 ; i<_nCPU ; i++) {
@@ -396,6 +452,8 @@ void RooAbsTestStatistic::setMPSet(Int_t inSetNum, Int_t inNumSets)
   // Set MultiProcessor set number identification of this instance
 
   _setNum = inSetNum ; _numSets = inNumSets ;
+  _extSet = _mpinterl==RooFit::SimComponents ? _setNum : crc32(Form("%sXXXX",GetName()))%_numSets ;
+  
   if (_gofOpMode==SimMaster) {
     // Forward to slaves
     initialize() ;
@@ -430,10 +488,14 @@ void RooAbsTestStatistic::initMPMode(RooAbsReal* real, RooAbsData* data, const R
     Bool_t doInline = (i==_nCPU-1) ;
     if (!doInline) coutI(Eval) << "RooAbsTestStatistic::initMPMode: starting remote server process #" << i << endl ;
     _mpfeArray[i] = new RooRealMPFE(Form("%s_%lx_MPFE%d",GetName(),(ULong_t)this,i),Form("%s_%lx_MPFE%d",GetTitle(),(ULong_t)this,i),*gof,doInline) ;
+    //_mpfeArray[i]->setVerbose(kTRUE,kTRUE) ;
     _mpfeArray[i]->initialize() ;
     if (doInline) {
       _mpfeArray[i]->addOwnedComponents(*gof) ;
     }
+    if (i>0) {
+      _mpfeArray[i]->followAsSlave(*_mpfeArray[0]) ;
+    }
   }
   //cout << "initMPMode --- done" << endl ;
   return ;
@@ -479,6 +541,7 @@ void RooAbsTestStatistic::initSimMode(RooSimultaneous* simpdf, RooAbsData* data,
 
   // Allocate arrays
   _gofArray = new pRooAbsTestStatistic[_nGof] ;
+  _gofSplitMode.resize(_nGof) ;
 
   // Create array of regular fit contexts, containing subset of data and single fitCat PDF
   catIter->Reset() ;
@@ -501,8 +564,28 @@ void RooAbsTestStatistic::initSimMode(RooSimultaneous* simpdf, RooAbsData* data,
       }
       _gofArray[n]->setSimCount(_nGof) ;
 
+      // Fill per-component split mode with Bulk Partition for now so that Auto will map to bulk-splitting of all components
+      if (_mpinterl==RooFit::Hybrid) {
+	if (dset->numEntries()<10) {
+	  //cout << "RAT::initSim("<< GetName() << ") MP mode is auto, setting split mode for component "<< n << " to SimComponents"<< endl ;
+	  _gofSplitMode[n] = RooFit::SimComponents ;
+	  _gofArray[n]->_mpinterl = RooFit::SimComponents ;
+	} else {
+	  //cout << "RAT::initSim("<< GetName() << ") MP mode is auto, setting split mode for component "<< n << " to BulkPartition"<< endl ;
+	  _gofSplitMode[n] = RooFit::BulkPartition ;
+	  _gofArray[n]->_mpinterl = RooFit::BulkPartition ;
+	}
+      }
+
       // Servers may have been redirected between instantiation and (deferred) initialization
-      _gofArray[n]->recursiveRedirectServers(_paramSet) ;
+      RooArgSet* actualParams = pdf->getParameters(dset) ;
+      RooArgSet* selTargetParams = (RooArgSet*) _paramSet.selectCommon(*actualParams) ;
+
+      _gofArray[n]->recursiveRedirectServers(*selTargetParams) ;
+
+      delete selTargetParams ;
+      delete actualParams ;
+
       n++ ;
     } else {
       if ((!dset || (dset->sumEntries()==0. && !processEmptyDataSets()) ) && pdf) {
@@ -536,6 +619,12 @@ Bool_t RooAbsTestStatistic::setData(RooAbsData& indata, Bool_t cloneData)
   // a range specification on the data, the cloneData argument is ignore and
   // the data is always cloned.
 
+  // Trigger refresh of likelihood offsets 
+  if (isOffsetting()) {
+    enableOffsetting(kFALSE) ;
+    enableOffsetting(kTRUE) ;
+  }
+
   switch(operMode()) {
 
   case Slave:
@@ -579,8 +668,44 @@ Bool_t RooAbsTestStatistic::setData(RooAbsData& indata, Bool_t cloneData)
     break ;
   }
 
+
   return kTRUE ;
 }
 
 
 
+void RooAbsTestStatistic::enableOffsetting(Bool_t flag) 
+{
+  // Apply internal value offsetting to control numeric precision
+  if (!_init) {
+    const_cast<RooAbsTestStatistic*>(this)->initialize() ;
+  }
+  
+  switch(operMode()) {
+    
+  case Slave:
+    _doOffset = flag ;
+    // Clear offset if feature is disabled to that it is recalculated next time it is enabled
+    if (!_doOffset) {
+      _offset = 0 ;
+      _offsetCarry = 0;
+    }
+    setValueDirty() ;
+    break ;
+
+  case SimMaster:
+    _doOffset = flag ;
+    for (Int_t i=0 ; i<_nGof ; i++) {
+      _gofArray[i]->enableOffsetting(flag) ;      
+    }
+    break ;
+
+  case MPMaster:    
+    _doOffset = flag ;
+    for (Int_t i=0 ; i<_nCPU ; i++) {
+      _mpfeArray[i]->enableOffsetting(flag) ;
+    }
+  }
+}
+
+
