@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooDataHist.cxx 42984 2012-02-13 15:30:31Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -167,7 +167,7 @@ RooDataHist::RooDataHist(const char *name, const char *title, const RooArgList&
   _dstore = (defaultStorageType==Tree) ? ((RooAbsDataStore*) new RooTreeDataStore(name,title,_vars)) : 
                                          ((RooAbsDataStore*) new RooVectorDataStore(name,title,_vars)) ;
   
-  importTH1Set(vars, indexCat, histMap, wgt, kTRUE) ;
+  importTH1Set(vars, indexCat, histMap, wgt, kFALSE) ;
 
   _dstore->setExternalWeightArray(_wgt,_errLo,_errHi,_sumw2) ;
 }
@@ -219,7 +219,7 @@ RooDataHist::RooDataHist(const char *name, const char *title, const RooArgList&
     assert(0) ; 
   }
 
-  importTH1(vars,*const_cast<TH1*>(hist),wgt, kTRUE) ;
+  importTH1(vars,*const_cast<TH1*>(hist),wgt, kFALSE) ;
 
   _dstore->setExternalWeightArray(_wgt,_errLo,_errHi,_sumw2) ;
 }
@@ -328,7 +328,7 @@ RooDataHist::RooDataHist(const char *name, const char *title, const RooArgList&
 	hmap[token] = (TH1*) hiter->Next() ;
 	token = strtok(0,",") ;
       }
-      importTH1Set(vars,*indexCat,hmap,initWgt,kTRUE) ;
+      importTH1Set(vars,*indexCat,hmap,initWgt,kFALSE) ;
     } else {
 
       // Initialize importing mapped set of RooDataHists
@@ -394,7 +394,7 @@ void RooDataHist::importTH1(const RooArgList& vars, TH1& histo, Double_t wgt, Bo
     zmin = offset[2] ;
     volume *= (zvar->getMax()-zvar->getMin()) ;
   }
-  Double_t avgBV = volume / numEntries() ;
+  //Double_t avgBV = volume / numEntries() ;
 //   cout << "average bin volume = " << avgBV << endl ;
 
   Int_t ix(0),iy(0),iz(0) ;
@@ -406,18 +406,16 @@ void RooDataHist::importTH1(const RooArgList& vars, TH1& histo, Double_t wgt, Bo
 	if (zvar) {
 	  for (iz=0 ; iz < zvar->getBins() ; iz++) {
 	    zvar->setBin(iz) ;
-	    Double_t bv = doDensityCorrection ? binVolume(vset)/avgBV : 1;
+	    Double_t bv = doDensityCorrection ? binVolume(vset) : 1;
 	    add(vset,bv*histo.GetBinContent(ix+1+xmin,iy+1+ymin,iz+1+zmin)*wgt,bv*TMath::Power(histo.GetBinError(ix+1+xmin,iy+1+ymin,iz+1+zmin)*wgt,2)) ;
 	  }
 	} else {
-	  Double_t bv = doDensityCorrection ? binVolume(vset)/avgBV : 1;
+	  Double_t bv = doDensityCorrection ? binVolume(vset) : 1;
 	  add(vset,bv*histo.GetBinContent(ix+1+xmin,iy+1+ymin)*wgt,bv*TMath::Power(histo.GetBinError(ix+1+xmin,iy+1+ymin)*wgt,2)) ;
 	}
       }
     } else {
-      Double_t bv = doDensityCorrection ? binVolume(vset)/avgBV : 1 ;
-//       cout << " RooDataHist(" << GetName() << ") ix = " << ix << " binVolume = " << bv << " binContent = " << histo.GetBinContent(ix+1+xmin) 
-// 	   << " wgt = " << wgt << " value in RDH = " << bv*histo.GetBinContent(ix+1+xmin)*wgt << endl ;
+      Double_t bv = doDensityCorrection ? binVolume(vset) : 1 ;
       add(vset,bv*histo.GetBinContent(ix+1+xmin)*wgt,bv*TMath::Power(histo.GetBinError(ix+1+xmin)*wgt,2)) ;	    
     }
   }  
@@ -852,9 +850,10 @@ void RooDataHist::initialize(const char* binningName, Bool_t fillTree)
       RooAbsLValue* arglv = dynamic_cast<RooAbsLValue*>(arg2) ;
       arglv->setBin(idx) ;
       theBinVolume *= arglv->getBinWidth(idx) ;
-//       cout << "init: binv[" << idx << "] = " << theBinVolume << endl ;
+//       cout << "init: bin width at idx=" << idx << " = " << arglv->getBinWidth(idx) << " binv[" << idx << "] = " << theBinVolume << endl ;
     }
     _binv[ibin] = theBinVolume ;
+//     cout << "binv[" << ibin << "] = " << theBinVolume << endl ;
     fill() ;
   }
 
@@ -1132,6 +1131,13 @@ RooPlot *RooDataHist::plotOn(RooPlot *frame, PlotOpt o) const
 
 
 
+//_____________________________________________________________________________
+Double_t RooDataHist::weightSquared() const {
+  return _curSumW2 ;
+}
+
+
+
 //_____________________________________________________________________________
 Double_t RooDataHist::weight(const RooArgSet& bin, Int_t intOrder, Bool_t correctForBinSize, Bool_t cdfBoundaries, Bool_t oneSafe) 
 {
@@ -1150,6 +1156,7 @@ Double_t RooDataHist::weight(const RooArgSet& bin, Int_t intOrder, Bool_t correc
     return 0 ;
   }
 
+
   // Handle no-interpolation case
   if (intOrder==0) {    
     _vars.assignValueOnly(bin,oneSafe) ;
@@ -1253,7 +1260,11 @@ void RooDataHist::weightError(Double_t& lo, Double_t& hi, ErrorType etype) const
   switch (etype) {
 
   case Auto:
-    throw string(Form("RooDataHist::weightError(%s) weight type Auto not allowed here",GetName())) ;
+    throw string(Form("RooDataHist::weightError(%s) error type Auto not allowed here",GetName())) ;
+    break ;
+
+  case Expected:
+    throw string(Form("RooDataHist::weightError(%s) error type Expected not allowed here",GetName())) ;
     break ;
 
   case Poisson:
@@ -1502,12 +1513,15 @@ Double_t RooDataHist::sum(Bool_t correctForBinSize, Bool_t inverseBinCor) const
   checkInit() ;
 
   Int_t i ;
-  Double_t total(0) ;
+  Double_t total(0), carry(0);
   for (i=0 ; i<_arrSize ; i++) {
     
     Double_t theBinVolume = correctForBinSize ? (inverseBinCor ? 1/_binv[i] : _binv[i]) : 1.0 ;
     // cout << "total += " << _wgt[i] << "*" << theBinVolume << endl ;
-    total += _wgt[i]*theBinVolume ;
+    Double_t y = _wgt[i]*theBinVolume - carry;
+    Double_t t = total + y;
+    carry = (t - total) - y;
+    total = t;
   }
 
   return total ;
@@ -1561,7 +1575,7 @@ Double_t RooDataHist::sum(const RooArgSet& sumSet, const RooArgSet& sliceSet, Bo
   }
     
   // Loop over entire data set, skipping masked entries
-  Double_t total(0) ;
+  Double_t total(0), carry(0);
   Int_t ibin ;
   for (ibin=0 ; ibin<_arrSize ; ibin++) {
 
@@ -1581,7 +1595,10 @@ Double_t RooDataHist::sum(const RooArgSet& sumSet, const RooArgSet& sliceSet, Bo
     if (!skip) {
       Double_t theBinVolume = correctForBinSize ? (inverseBinCor ? 1/(*_pbinv)[i] : (*_pbinv)[i] ) : 1.0 ;
 //       cout << "adding bin[" << ibin << "] to sum wgt = " << _wgt[ibin] << " binv = " << theBinVolume << endl ;
-      total += _wgt[ibin]*theBinVolume ;
+      Double_t y = _wgt[ibin]*theBinVolume - carry;
+      Double_t t = total + y;
+      carry = (t - total) - y;
+      total = t;
     }
   }
   delete ssIter ;
@@ -1663,10 +1680,13 @@ Int_t RooDataHist::numEntries() const
 Double_t RooDataHist::sumEntries() const 
 {
   Int_t i ;
-  Double_t n(0) ;
+  Double_t n(0), carry(0);
   for (i=0 ; i<_arrSize ; i++) {
     if (!_binValid || _binValid[i]) {
-      n+= _wgt[i] ;
+      Double_t y = _wgt[i] - carry;
+      Double_t t = n + y;
+      carry = (t - n) - y;
+      n = t;
     }
   }
   return n ;
@@ -1683,14 +1703,7 @@ Double_t RooDataHist::sumEntries(const char* cutSpec, const char* cutRange) cons
   checkInit() ;
 
   if (cutSpec==0 && cutRange==0) {
-    Int_t i ;
-    Double_t n(0) ;
-    for (i=0 ; i<_arrSize ; i++) {
-      if (!_binValid || _binValid[i]) {
-	n+= _wgt[i] ;
-      }
-    }
-    return n ;
+    return sumEntries();
   } else {
     
     // Setup RooFormulaVar for cutSpec if it is present
@@ -1700,7 +1713,7 @@ Double_t RooDataHist::sumEntries(const char* cutSpec, const char* cutRange) cons
     }
     
     // Otherwise sum the weights in the event
-    Double_t sumw(0) ;
+    Double_t sumw(0), carry(0);
     Int_t i ;
     for (i=0 ; i<numEntries() ; i++) {
       get(i) ;
@@ -1708,7 +1721,10 @@ Double_t RooDataHist::sumEntries(const char* cutSpec, const char* cutRange) cons
       if (cutRange && !_vars.allInRange(cutRange)) continue ;
 
       if (!_binValid || _binValid[i]) {
-	sumw += weight() ;
+	Double_t y = weight() - carry;
+	Double_t t = sumw + y;
+	carry = (t - sumw) - y;
+	sumw = t;
       }
     }
     
