@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooDataSet.cxx 43190 2012-03-01 12:09:43Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -56,6 +56,8 @@
 char* operator+( streampos&, char* );
 #endif
 
+using namespace std;
+
 ClassImp(RooDataSet)
 ;
 
@@ -245,9 +247,11 @@ RooDataSet::RooDataSet(const char* name, const char* title, const RooArgSet& var
   pc.defineObject("cutVar","CutVar",0) ;
   pc.defineString("cutRange","CutRange",0,"") ;
   pc.defineString("wgtVarName","WeightVarName",0,"") ;
+  pc.defineInt("newWeight1","WeightVarName",0,0) ;
   pc.defineString("fname","ImportFromFile",0,"") ;
   pc.defineString("tname","ImportFromFile",1,"") ;
   pc.defineObject("wgtVar","WeightVar",0) ;
+  pc.defineInt("newWeight2","WeightVar",0,0) ;
   pc.defineObject("dummy1","ImportDataSliceMany",0) ;
   pc.defineObject("dummy2","LinkDataSliceMany",0) ;
   pc.defineSet("errorSet","StoreError",0) ;
@@ -291,7 +295,7 @@ RooDataSet::RooDataSet(const char* name, const char* title, const RooArgSet& var
   const char* fname = pc.getString("fname") ;
   const char* tname = pc.getString("tname") ;
   Int_t ownLinked = pc.getInt("ownLinked") ;
-
+  Int_t newWeight = pc.getInt("newWeight1") + pc.getInt("newWeight2") ;
 
   // Case 1 --- Link multiple dataset as slices
   if (lnkSliceNames) {
@@ -352,6 +356,13 @@ RooDataSet::RooDataSet(const char* name, const char* title, const RooArgSet& var
     if (wgtVar) {
       wgtVarName = wgtVar->GetName() ;
     }
+    
+    // Clone weight variable of imported dataset if we are not weighted
+    if (!wgtVar && !wgtVarName && impData && impData->_wgtVar) {
+      _wgtVar = (RooRealVar*) impData->_wgtVar->createFundamental() ;
+      _vars.addOwned(*_wgtVar) ;
+      wgtVarName = _wgtVar->GetName() ;
+    }
 
     // Create empty datastore 
     RooTreeDataStore* tstore(0) ;
@@ -361,6 +372,12 @@ RooDataSet::RooDataSet(const char* name, const char* title, const RooArgSet& var
       tstore = new RooTreeDataStore(name,title,_vars,wgtVarName) ;
       _dstore = tstore ;
     } else if (defaultStorageType==Vector) {
+      if (wgtVarName && newWeight) {
+	RooAbsArg* wgttmp = _vars.find(wgtVarName) ;
+	if (wgttmp) {
+	  wgttmp->setAttribute("NewWeight") ;
+	}
+      }
       vstore = new RooVectorDataStore(name,title,_vars,wgtVarName) ;
       _dstore = vstore ;
     } else {
@@ -418,8 +435,10 @@ RooDataSet::RooDataSet(const char* name, const char* title, const RooArgSet& var
     if (wgtVarName && *wgtVarName) {
       // Use the supplied weight column
       initialize(wgtVarName) ;    
+
     } else {
       if (impData && impData->_wgtVar && vars.find(impData->_wgtVar->GetName())) {
+
 	// Use the weight column of the source data set
 	initialize(impData->_wgtVar->GetName()) ;
 
@@ -976,6 +995,17 @@ Double_t RooDataSet::weight() const
 
 
 
+//_____________________________________________________________________________
+Double_t RooDataSet::weightSquared() const 
+{
+  // Return event weight of current event
+
+  return store()->weight()*store()->weight() ; 
+}
+
+
+
+
 //_____________________________________________________________________________
 void RooDataSet::weightError(Double_t& lo, Double_t& hi, ErrorType etype) const 
 {
@@ -1014,11 +1044,14 @@ Double_t RooDataSet::sumEntries() const
   }
 
   // Otherwise sum the weights in the event
-  Double_t sumw(0) ;
+  Double_t sumw(0), carry(0);
   Int_t i ;
   for (i=0 ; i<numEntries() ; i++) {
     get(i) ;
-    sumw += weight() ;
+    Double_t y = weight() - carry;
+    Double_t t = sumw + y;
+    carry = (t - sumw) - y;
+    sumw = t;
   }  
 
   return sumw ;  
@@ -1043,13 +1076,16 @@ Double_t RooDataSet::sumEntries(const char* cutSpec, const char* cutRange) const
   }
 
   // Otherwise sum the weights in the event
-  Double_t sumw(0) ;
+  Double_t sumw(0), carry(0);
   Int_t i ;
   for (i=0 ; i<numEntries() ; i++) {
     get(i) ;
     if (select && select->eval()==0.) continue ;
     if (cutRange && !_vars.allInRange(cutRange)) continue ;
-    sumw += weight() ;
+    Double_t y = weight() - carry;
+    Double_t t = sumw + y;
+    carry = (t - sumw) - y;
+    sumw = t;
   }
 
   if (select) delete select ;
