@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooMinimizer.cxx 42279 2011-11-28 15:07:56Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -77,6 +77,8 @@
 char* operator+( streampos&, char* );
 #endif
 
+using namespace std;
+
 ClassImp(RooMinimizer)
 ;
 
@@ -119,6 +121,7 @@ RooMinimizer::RooMinimizer(RooAbsReal& function)
   _optConst = kFALSE ;
   _verbose = kFALSE ;
   _profile = kFALSE ;
+  _profileStart = kFALSE ;
   _printLevel = 1 ;
   _minimizerType = "Minuit"; // default minimizer
 
@@ -206,6 +209,14 @@ void RooMinimizer::setEps(Double_t eps)
 }
 
 
+//_____________________________________________________________________________
+void RooMinimizer::setOffsetting(Bool_t flag) 
+{
+  // Enable internal likelihood offsetting for enhanced numeric precision
+  _func->enableOffsetting(flag) ; 
+}
+
+
 
 
 //_____________________________________________________________________________
@@ -587,7 +598,7 @@ RooFitResult* RooMinimizer::save(const char* userName, const char* userTitle)
   title = userTitle ? userTitle : Form("%s", _func->GetTitle()) ;
   RooFitResult* fitRes = new RooFitResult(name,title) ;
 
-  // Move eventual fixed paramaters in floatList to constList
+  // Move eventual fixed parameters in floatList to constList
   Int_t i ;
   RooArgList saveConstList(*(_fcn->GetConstParamList())) ;
   RooArgList saveFloatInitList(*(_fcn->GetInitFloatParamList())) ;
@@ -728,7 +739,8 @@ void RooMinimizer::profileStart()
   // Start profiling timer
   if (_profile) {
     _timer.Start() ;
-    _cumulTimer.Start(kFALSE) ;
+    _cumulTimer.Start(_profileStart?kFALSE:kTRUE) ;
+    _profileStart = kTRUE ;
   }
 }
 
