@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooRealIntegral.cxx 43190 2012-03-01 12:09:43Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -50,6 +50,8 @@
 #include "RooConstVar.h"
 #include "RooDouble.h"
 
+using namespace std;
+
 ClassImp(RooRealIntegral) 
 ;
 
@@ -759,6 +761,7 @@ RooRealIntegral::~RooRealIntegral()
   delete _facListIter ;
   delete _jacListIter ;
   if (_sumCatIter)  delete _sumCatIter ;
+  if (_params) delete _params ;
 }
 
 
