@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooRealSumPdf.cxx 42279 2011-11-28 15:07:56Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -45,9 +45,12 @@
 #include <memory>
 #include <algorithm>
 
+using namespace std;
+
 ClassImp(RooRealSumPdf)
 ;
 
+Bool_t RooRealSumPdf::_doFloorGlobal = kFALSE ; 
 
 //_____________________________________________________________________________
 RooRealSumPdf::RooRealSumPdf() 
@@ -57,6 +60,7 @@ RooRealSumPdf::RooRealSumPdf()
   _funcIter  = _funcList.createIterator() ;
   _coefIter  = _coefList.createIterator() ;
   _extended = kFALSE ;
+  _doFloor = kFALSE ;
 }
 
 
@@ -68,7 +72,8 @@ RooRealSumPdf::RooRealSumPdf(const char *name, const char *title) :
   _haveLastCoef(kFALSE),
   _funcList("!funcList","List of functions",this),
   _coefList("!coefList","List of coefficients",this),
-  _extended(kFALSE)
+  _extended(kFALSE),
+  _doFloor(kFALSE)
 {
   // Constructor with name and title
   _funcIter   = _funcList.createIterator() ;
@@ -85,7 +90,8 @@ RooRealSumPdf::RooRealSumPdf(const char *name, const char *title,
   _haveLastCoef(kFALSE),
   _funcList("!funcList","List of functions",this),
   _coefList("!coefList","List of coefficients",this),
-  _extended(kFALSE)
+  _extended(kFALSE),
+  _doFloor(kFALSE)
 {
   // Construct p.d.f consisting of coef1*func1 + (1-coef1)*func2
   // The input coefficients and functions are allowed to be negative
@@ -109,7 +115,8 @@ RooRealSumPdf::RooRealSumPdf(const char *name, const char *title, const RooArgLi
   _haveLastCoef(kFALSE),
   _funcList("!funcList","List of functions",this),
   _coefList("!coefList","List of coefficients",this),
-  _extended(extended)
+  _extended(extended),
+  _doFloor(kFALSE)
 { 
   // Constructor p.d.f implementing sum_i [ coef_i * func_i ], if N_coef==N_func
   // or sum_i [ coef_i * func_i ] + (1 - sum_i [ coef_i ] )* func_N if Ncoef==N_func-1
@@ -172,7 +179,8 @@ RooRealSumPdf::RooRealSumPdf(const RooRealSumPdf& other, const char* name) :
   _haveLastCoef(other._haveLastCoef),
   _funcList("!funcList",this,other._funcList),
   _coefList("!coefList",this,other._coefList),
-  _extended(other._extended)
+  _extended(other._extended),
+  _doFloor(other._doFloor)
 {
   // Copy constructor
 
@@ -222,7 +230,7 @@ Double_t RooRealSumPdf::evaluate() const
     func = (RooAbsReal*)funcIter.next() ;
     Double_t coefVal = coef->getVal() ;
     if (coefVal) {
-      cxcoutD(Eval) << "RooRealSumPdf::eval(" << GetName() << ") coefVal = " << coefVal << " funcVal = " << func->getVal() << endl ;
+      cxcoutD(Eval) << "RooRealSumPdf::eval(" << GetName() << ") coefVal = " << coefVal << " funcVal = " << func->IsA()->GetName() << "::" << func->GetName() << " = " << func->getVal() << endl ;
       if (func->isSelectedComp()) {
 	value += func->getVal()*coefVal ;
       }
@@ -247,6 +255,11 @@ Double_t RooRealSumPdf::evaluate() const
     } 
   }
 
+  // Introduce floor if so requested
+  if (value<0 && (_doFloor || _doFloorGlobal)) {
+    value = 0 ;
+  }
+
   return value ;
 }
 
