@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooVectorDataStore.cxx 43190 2012-03-01 12:09:43Z wouter $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *   DK, David Kirkby,    UC Irvine,         dkirkby@uci.edu                 *
@@ -35,6 +35,7 @@
 #include "RooFormulaVar.h"
 #include "RooRealVar.h"
 #include "RooCategory.h"
+#include "RooNameSet.h"
 #include "RooHistError.h"
 
 #include <iomanip>
@@ -61,6 +62,7 @@ RooVectorDataStore::RooVectorDataStore() :
   _firstRealF(0),
   _firstCat(0),
   _sumWeight(0),
+  _sumWeightCarry(0),
   _extWgtArray(0),
   _extWgtErrLoArray(0),
   _extWgtErrHiArray(0),
@@ -89,6 +91,7 @@ RooVectorDataStore::RooVectorDataStore(const char* name, const char* title, cons
   _firstRealF(0),
   _firstCat(0),
   _sumWeight(0),
+  _sumWeightCarry(0),
   _extWgtArray(0),
   _extWgtErrLoArray(0),
   _extWgtErrHiArray(0),
@@ -108,7 +111,6 @@ RooVectorDataStore::RooVectorDataStore(const char* name, const char* title, cons
   delete iter ;
   
   setAllBuffersNative() ;
-  
 }
 
 
@@ -179,6 +181,7 @@ RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const ch
   _nCat(0),
   _nEntries(other._nEntries),	 
   _sumWeight(other._sumWeight),
+  _sumWeightCarry(other._sumWeightCarry),
   _extWgtArray(other._extWgtArray),
   _extWgtErrLoArray(other._extWgtErrLoArray),
   _extWgtErrHiArray(other._extWgtErrHiArray),
@@ -231,6 +234,7 @@ RooVectorDataStore::RooVectorDataStore(const RooTreeDataStore& other, const RooA
   _firstRealF(0),
   _firstCat(0),
   _sumWeight(0),
+  _sumWeightCarry(0),
   _extWgtArray(0),
   _extWgtErrLoArray(0),
   _extWgtErrHiArray(0),
@@ -252,6 +256,7 @@ RooVectorDataStore::RooVectorDataStore(const RooTreeDataStore& other, const RooA
   setAllBuffersNative() ;
   
   // now copy contents of tree storage here
+  reserve(other.numEntries());
   for (Int_t i=0 ; i<other.numEntries() ; i++) {
     other.get(i) ;
     _varsww = other._varsww ;
@@ -271,6 +276,7 @@ RooVectorDataStore::RooVectorDataStore(const RooVectorDataStore& other, const Ro
   _nCat(0),
   _nEntries(other._nEntries),	 
   _sumWeight(other._sumWeight),
+  _sumWeightCarry(other._sumWeightCarry),
   _extWgtArray(other._extWgtArray),
   _extWgtErrLoArray(other._extWgtErrLoArray),
   _extWgtErrHiArray(other._extWgtErrHiArray),
@@ -346,6 +352,7 @@ RooVectorDataStore::RooVectorDataStore(const char *name, const char *title, RooA
   _firstRealF(0),
   _firstCat(0),
   _sumWeight(0),
+  _sumWeightCarry(0),
   _extWgtArray(0),
   _extWgtErrLoArray(0),
   _extWgtErrHiArray(0),
@@ -379,7 +386,7 @@ RooVectorDataStore::RooVectorDataStore(const char *name, const char *title, RooA
   
   loadValues(&tds,cloneVar,cutRange,nStart,nStop);
 
-  if (cloneVar) delete cloneVar ;
+  delete cloneVar ;
 }
 
 
@@ -406,7 +413,7 @@ RooVectorDataStore::~RooVectorDataStore()
     delete *iter2 ;
   }
 
-  if (_cache) delete _cache ;
+  delete _cache ;
 }
 
 
@@ -439,7 +446,11 @@ Int_t RooVectorDataStore::fill()
   for ( ; iter2!=_catStoreList.end() ; ++iter2) {
     (*iter2)->fill() ;
   }
-  _sumWeight += _wgtVar ? _wgtVar->getVal() : 1. ;
+  // use Kahan's algorithm to sum up weights to avoid loss of precision
+  Double_t y = (_wgtVar ? _wgtVar->getVal() : 1.) - _sumWeightCarry;
+  Double_t t = _sumWeight + y;
+  _sumWeightCarry = (t - _sumWeight) - y;
+  _sumWeight = t;
   _nEntries++ ;  
 
   return 0 ;
@@ -647,7 +658,11 @@ void RooVectorDataStore::weightError(Double_t& lo, Double_t& hi, RooAbsData::Err
     switch (etype) {
       
     case RooAbsData::Auto:
-      throw string(Form("RooDataHist::weightError(%s) weight type Auto not allowed here",GetName())) ;
+      throw string(Form("RooDataHist::weightError(%s) error type Auto not allowed here",GetName())) ;
+      break ;
+      
+    case RooAbsData::Expected:
+      throw string(Form("RooDataHist::weightError(%s) error type Expected not allowed here",GetName())) ;
       break ;
       
     case RooAbsData::Poisson:
@@ -726,6 +741,23 @@ void RooVectorDataStore::loadValues(const RooAbsDataStore *ads, const RooFormula
 
   Bool_t isTDS = dynamic_cast<const RooTreeDataStore*>(ads) ;
   Bool_t isVDS = dynamic_cast<const RooVectorDataStore*>(ads) ;
+
+  // Check if weight is being renamed - if so set flag to enable special handling in copy loop
+  Bool_t weightRename(kFALSE) ;
+  Bool_t newWeightVar = _wgtVar ? _wgtVar->getAttribute("NewWeight") : kFALSE ;
+
+  if (_wgtVar && isVDS && ((RooVectorDataStore*)(ads))->_wgtVar) {
+    if (string(_wgtVar->GetName())!=((RooVectorDataStore*)(ads))->_wgtVar->GetName() && !newWeightVar) {
+      weightRename=kTRUE ;
+    }
+  }
+  if (_wgtVar && isTDS && ((RooTreeDataStore*)(ads))->_wgtVar) {
+    if (string(_wgtVar->GetName())!=((RooTreeDataStore*)(ads))->_wgtVar->GetName() && !newWeightVar) {
+      weightRename=kTRUE ;
+    }
+  }
+
+  reserve(numEntries() + (nevent - nStart));
   for(Int_t i=nStart; i < nevent ; ++i) {
     ads->get(i) ;
     
@@ -737,8 +769,14 @@ void RooVectorDataStore::loadValues(const RooAbsDataStore *ads, const RooFormula
 
     if (isTDS) {
       _varsww.assignValueOnly(((RooTreeDataStore*)ads)->_varsww) ;
+      if (weightRename) {
+	_wgtVar->setVal(((RooTreeDataStore*)ads)->_wgtVar->getVal()) ;
+      }
     } else if (isVDS) {
       _varsww.assignValueOnly(((RooVectorDataStore*)ads)->_varsww) ;
+      if (weightRename) {
+	_wgtVar->setVal(((RooVectorDataStore*)ads)->_wgtVar->getVal()) ;
+      }
     } else {
       _varsww.assignValueOnly(*ads->get()) ;
     }
@@ -950,6 +988,7 @@ RooAbsDataStore* RooVectorDataStore::merge(const RooArgSet& allVars, list<RooAbs
   RooVectorDataStore* mergedStore = new RooVectorDataStore("merged","merged",allVars) ;
 
   Int_t nevt = dstoreList.front()->numEntries() ;
+  mergedStore->reserve(nevt);
   for (int i=0 ; i<nevt ; i++) {
 
     // Copy data from self
@@ -968,12 +1007,27 @@ RooAbsDataStore* RooVectorDataStore::merge(const RooArgSet& allVars, list<RooAbs
 
 
 
-
+void RooVectorDataStore::reserve(Int_t nEvts)
+{
+  vector<RealVector*>::iterator iter = _realStoreList.begin() ;
+  for ( ; iter!=_realStoreList.end() ; ++iter) {
+    (*iter)->reserve(nEvts);
+  }
+  vector<RealFullVector*>::iterator iter3 = _realfStoreList.begin() ;
+  for ( ; iter3!=_realfStoreList.end() ; ++iter3) {
+    (*iter3)->reserve(nEvts);
+  }
+  vector<CatVector*>::iterator iter2 = _catStoreList.begin() ;
+  for ( ; iter2!=_catStoreList.end() ; ++iter2) {
+    (*iter2)->reserve(nEvts);
+  }
+}
 
 //_____________________________________________________________________________
 void RooVectorDataStore::append(RooAbsDataStore& other) 
 {
   Int_t nevt = other.numEntries() ;
+  reserve(nevt + numEntries());
   for (int i=0 ; i<nevt ; i++) {  
     _vars = *other.get(i) ;
     if (_wgtVar) {
@@ -998,7 +1052,7 @@ Int_t RooVectorDataStore::numEntries() const
 void RooVectorDataStore::reset() 
 {
   _nEntries=0 ;
-  _sumWeight=0 ;
+  _sumWeight=_sumWeightCarry=0 ;
   vector<RealVector*>::iterator iter = _realStoreList.begin() ;
   for ( ; iter!=_realStoreList.end() ; ++iter) {
     (*iter)->reset() ;
@@ -1032,11 +1086,8 @@ void RooVectorDataStore::cacheArgs(const RooAbsArg* owner, RooArgSet& newVarSet,
   // precalculated value and it's dirty flag will be cleared.
 
   // Delete previous cache, if any
-  if (_cache) {
-    delete _cache ;
-    _cache = 0 ;
-  }
-
+  delete _cache ;
+  _cache = 0 ;
 
   // Reorder cached elements. First constant nodes, then tracked nodes in order of dependence
 
@@ -1084,7 +1135,7 @@ void RooVectorDataStore::cacheArgs(const RooAbsArg* owner, RooArgSet& newVarSet,
   checkInit() ;
   
   list<RooArgSet*> vlist ;
-  RooArgSet cloneSet ;
+  RooArgList cloneSet ;
 
   while((var=(RooAbsArg*)vIter->Next())) {
 
@@ -1107,50 +1158,98 @@ void RooVectorDataStore::cacheArgs(const RooAbsArg* owner, RooArgSet& newVarSet,
 
   RooAbsArg::setDirtyInhibit(kTRUE) ;
 
+  vector<RooArgSet*> nsetList ;
+  list<RooArgSet*> argObsList ;
+
   // Now need to attach branch buffers of clones
   TIterator* it = cloneSet.createIterator() ;
+  RooArgSet *anset(0), *acset(0) ;
   while ((arg=(RooAbsArg*)it->Next())) {
     arg->attachToVStore(*newCache) ;
+    
+    RooArgSet* argObs = nset ? arg->getObservables(*nset) : arg->getVariables() ;
+    argObsList.push_back(argObs) ;
+    
+    RooArgSet* normSet(0) ;
+    const char* catNset = arg->getStringAttribute("CATNormSet") ;
+    if (catNset) {
+//       cout << "RooVectorDataStore::cacheArgs() cached node " << arg->GetName() << " has a normalization set specification CATNormSet = " << catNset << endl ;
+      RooNameSet rns ;
+      rns.setNameList(catNset) ;
+      anset = rns.select(nset?*nset:RooArgSet()) ;
+      normSet = (RooArgSet*) anset->selectCommon(*argObs) ;
+      
+    }
+    const char* catCset = arg->getStringAttribute("CATCondSet") ;
+    if (catCset) {
+//       cout << "RooVectorDataStore::cacheArgs() cached node " << arg->GetName() << " has a conditional observable set specification CATCondSet = " << catCset << endl ;
+      RooNameSet rns ;
+      rns.setNameList(catCset) ;
+      acset = rns.select(nset?*nset:RooArgSet()) ;
+      
+      argObs->remove(*acset,kTRUE,kTRUE) ;
+      normSet = argObs ;
+    }
+
+    // now construct normalization set for component from cset/nset spec
+//     if (normSet) {
+//       cout << "RooVectorDaraStore::cacheArgs() component " << arg->GetName() << " has custom normalization set " << *normSet << endl ;
+//     }
+    nsetList.push_back(normSet) ;    
   }
   delete it ;
 
   // Fill values of of placeholder
+  newCache->reserve(numEntries());
   for (int i=0 ; i<numEntries() ; i++) {
     getNative(i) ;
-
-    cIter->Reset() ;
-    while((cloneArg=(RooAbsArg*)cIter->Next())) {
-      cloneArg->syncCache(nset) ;
-//       cout << "RVDS::cacheArgs writing #" << i << " " ; cloneArg->Print() ;
+    if (weight()!=0) {    
+      cIter->Reset() ;
+      vector<RooArgSet*>::iterator niter = nsetList.begin() ;
+      while((cloneArg=(RooAbsArg*)cIter->Next())) {
+	// WVE need to intervene here for condobs from ProdPdf
+	RooArgSet* argNset = *niter ;
+	cloneArg->syncCache(argNset?argNset:nset) ;
+	++niter ;
+      }
     }
     newCache->fill() ;
   }
-  
   delete cIter ;
 
   RooAbsArg::setDirtyInhibit(kFALSE) ;
 
-  for (list<RooArgSet*>::iterator iter = vlist.begin() ; iter!=vlist.end() ; ++iter) {
-    delete *iter ;
-  }  
-  
+
   // Now need to attach branch buffers of original function objects 
   it = orderedArgs.createIterator() ;
   while ((arg=(RooAbsArg*)it->Next())) {
     arg->attachToVStore(*newCache) ;
-
+    
     // Activate change tracking mode, if requested
     if (!arg->getAttribute("ConstantExpression") && dynamic_cast<RooAbsReal*>(arg)) {
       RealVector* rv = newCache->addReal((RooAbsReal*)arg) ;      
       RooArgSet* deps = arg->getParameters(_vars) ;
       rv->setDependents(*deps) ;
+
+      // WV lookup normalization set and associate with RealVector
+      // find ordinal number of arg in original list 
+      Int_t idx = cloneSet.index(arg->GetName()) ;
+
       coutI(Optimization) << "RooVectorDataStore::cacheArg() element " << arg->GetName() << " has change tracking enabled on parameters " << *deps << endl ;
+      rv->setNset(nsetList[idx]) ;
       delete deps ;
     }
     
   }
   delete it ;
 
+  for (list<RooArgSet*>::iterator iter = vlist.begin() ; iter!=vlist.end() ; ++iter) {
+    delete *iter ;
+  }  
+  for (list<RooArgSet*>::iterator iter = argObsList.begin() ; iter!=argObsList.end() ; ++iter) {
+    delete *iter ;
+  }  
+
   _cache = newCache ;
   _cache->setDirtyProp(_doDirtyProp) ;
 }
@@ -1179,9 +1278,13 @@ void RooVectorDataStore::recalculateCache( const RooArgSet *projectedArgs, Int_t
     }    
   }
 
+  // If no recalculations are neede stop here
+  if (ntv==0) {
+    return ;
+  }
   // Refill caches of elements that require recalculation
 //   cout << "recalc error count before update = " << RooAbsReal::numEvalErrors() << endl ;
-  RooAbsReal::ErrorLoggingMode origMode = RooAbsReal::evalErrorLoggingMode() ;
+  //RooAbsReal::ErrorLoggingMode origMode = RooAbsReal::evalErrorLoggingMode() ;
   RooArgSet* ownedNset = 0 ;
   RooArgSet* usedNset = 0 ;
   if (projectedArgs && projectedArgs->getSize()>0) {
@@ -1197,19 +1300,25 @@ void RooVectorDataStore::recalculateCache( const RooArgSet *projectedArgs, Int_t
   for (int i=firstEvent ; i<lastEvent ; i+=stepSize) {
     get(i) ;    
     Bool_t zeroWeight = (weight()==0) ;
-    if (zeroWeight) {
-//       cout << "update - slot " << i << " has zero weight, ignoring event errors" << endl ;
-      RooAbsReal::setEvalErrorLoggingMode(RooAbsReal::Ignore) ;
-    }
-    for (int j=0 ; j<ntv ; j++) {
-      tv[j]->_nativeReal->_valueDirty=kTRUE ;
-      tv[j]->_nativeReal->getValV(usedNset) ;
-      //       cout << "updating tracked cache slot " << i << " element " ; tv[j]->_nativeReal->Print() ;
-      tv[j]->write(i) ;
-    }
-    if (zeroWeight) {
-      RooAbsReal::setEvalErrorLoggingMode(origMode) ;
+    if (!zeroWeight) {
+      for (int j=0 ; j<ntv ; j++) {
+	tv[j]->_nativeReal->_valueDirty=kTRUE ;
+	tv[j]->_nativeReal->getValV(tv[j]->_nset ? tv[j]->_nset : usedNset) ;
+	tv[j]->write(i) ;
+      }
     }
+//     if (zeroWeight) {
+// //       cout << "update - slot " << i << " has zero weight, ignoring event errors" << endl ;
+//       RooAbsReal::setEvalErrorLoggingMode(RooAbsReal::Ignore) ;
+//     }
+//     for (int j=0 ; j<ntv ; j++) {
+//       tv[j]->_nativeReal->_valueDirty=kTRUE ;
+//       tv[j]->_nativeReal->getValV(tv[j]->_nset ? tv[j]->_nset : usedNset) ;
+//       tv[j]->write(i) ;
+//     }
+//     if (zeroWeight) {
+//       RooAbsReal::setEvalErrorLoggingMode(origMode) ;
+//     }
   }  
   
 //   cout << "recalculate: end of updating" << endl ;
@@ -1219,9 +1328,7 @@ void RooVectorDataStore::recalculateCache( const RooArgSet *projectedArgs, Int_t
       tv[j]->_nativeReal->setOperMode(RooAbsArg::AClean) ;
   }  
 
-  if (ownedNset) {
-    delete ownedNset ;
-  }
+  delete ownedNset ;
 
 }
 
@@ -1274,11 +1381,9 @@ void RooVectorDataStore::attachCache(const RooAbsArg* newOwner, const RooArgSet&
 //_____________________________________________________________________________
 void RooVectorDataStore::resetCache() 
 {
-  if (_cache) {
-    delete _cache ;
-    _cache = 0 ;
-    _cacheOwner = 0 ;
-  }
+  delete _cache ;
+  _cache = 0 ;
+  _cacheOwner = 0 ;
   return ;
 }
 
@@ -1373,9 +1478,12 @@ void RooVectorDataStore::Streamer(TBuffer &R__b)
    if (R__b.IsReading()) {
       R__b.ReadClassBuffer(RooVectorDataStore::Class(),this);
 
-      _firstReal = &_realStoreList.front() ;
-      _firstRealF = &_realfStoreList.front() ;
-      _firstCat = &_catStoreList.front() ;
+	  if (_realStoreList.size() > 0)
+	      _firstReal = &_realStoreList.front() ;
+	  if (_realfStoreList.size() > 0)
+	      _firstRealF = &_realfStoreList.front() ;
+	  if (_catStoreList.size() > 0)
+	      _firstCat = &_catStoreList.front() ;
 
       for (vector<RealVector*>::iterator iter1 = _realStoreList.begin() ; iter1!=_realStoreList.end() ; ++iter1) {
 	RooAbsArg* arg = _varsww.find((*iter1)->_nativeReal->GetName()) ;
@@ -1416,9 +1524,16 @@ void RooVectorDataStore::RealFullVector::Streamer(TBuffer &R__b)
    // Stream an object of class RooVectorDataStore::RealFullVector.
 
    if (R__b.IsReading()) {
-      R__b.ReadClassBuffer(RooVectorDataStore::RealFullVector::Class(),this);
+     R__b.ReadClassBuffer(RooVectorDataStore::RealFullVector::Class(),this);
+
+     // WVE - It seems that ROOT persistence turns null pointers to vectors into pointers to null-sized vectors 
+     //       Intervene here to remove those null-sized vectors and replace with null pointers to not break
+     //       assumptions made elsewhere in this class
+     if (_vecE  && _vecE->empty()) { delete _vecE   ; _vecE = 0 ; }
+     if (_vecEL && _vecEL->empty()) { delete _vecEL ; _vecEL = 0 ; }
+     if (_vecEH && _vecEH->empty()) { delete _vecEH ; _vecEH = 0 ; }
    } else {
-      R__b.WriteClassBuffer(RooVectorDataStore::RealFullVector::Class(),this);
+     R__b.WriteClassBuffer(RooVectorDataStore::RealFullVector::Class(),this);
    }
 }
 
