@@ -1,7 +1,7 @@
 /*****************************************************************************
  * Project: RooFit                                                           *
  * Package: RooFitCore                                                       *
- * @(#)root/roofitcore:$Id: RooWorkspace.cxx 44381 2012-05-31 06:45:52Z moneta $
+ * @(#)root/roofitcore:$Id$
  * Authors:                                                                  *
  *   WV, Wouter Verkerke, UC Santa Barbara, verkerke@slac.stanford.edu       *
  *                                                                           *
@@ -44,6 +44,8 @@
 #include "RooMsgService.h"
 #include "RooConstVar.h"
 #include "RooResolutionModel.h"
+#include "RooPlot.h"
+#include "RooRandom.h"
 #include "TInterpreter.h"
 #include "TClassTable.h"
 #include "TBaseClass.h"
@@ -166,7 +168,7 @@ RooWorkspace::RooWorkspace(const char* name, Bool_t doCINTExport)  :
 
 //_____________________________________________________________________________
 RooWorkspace::RooWorkspace(const RooWorkspace& other) : 
-  TNamed(other), _uuid(other._uuid), _classes(this), _dir(0), _factory(0), _doExport(kFALSE), _openTrans(kFALSE)
+  TNamed(other), _uuid(other._uuid), _classes(other._classes,this), _dir(0), _factory(0), _doExport(kFALSE), _openTrans(kFALSE)
 {
   // Workspace copy constructor
 
@@ -235,14 +237,17 @@ RooWorkspace::~RooWorkspace()
 
 
 //_____________________________________________________________________________
-Bool_t RooWorkspace::import(const char* fileSpec, const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3) 
+Bool_t RooWorkspace::import(const char* fileSpec, 
+			    const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3, 
+			    const RooCmdArg& arg4, const RooCmdArg& arg5, const RooCmdArg& arg6, 
+			    const RooCmdArg& arg7, const RooCmdArg& arg8, const RooCmdArg& arg9) 
 {
   // Import a RooAbsArg or RooAbsData set from a workspace in a file. Filespec should be constructed as "filename:wspacename:objectname"
   // The arguments will be passed on to the relevant RooAbsArg& or RooAbsData& import call
 
   // Parse file/workspace/objectname specification
-  char buf[1024] ;
-  strlcpy(buf,fileSpec,1024) ;
+  char buf[10240] ;
+  strlcpy(buf,fileSpec,10240) ;
   char* filename = strtok(buf,":") ;
   char* wsname = strtok(0,":") ;
   char* objname = strtok(0,":") ;
@@ -271,13 +276,13 @@ Bool_t RooWorkspace::import(const char* fileSpec, const RooCmdArg& arg1, const R
   // Check that workspace contains object and forward to appropriate import method
   RooAbsArg* warg = w->arg(objname) ;
   if (warg) {
-    Bool_t ret = import(*warg,arg1,arg2,arg3) ;
+    Bool_t ret = import(*warg,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9) ;
     delete f ;
     return ret ;    
   }
   RooAbsData* wdata = w->data(objname) ;
   if (wdata) {
-    Bool_t ret = import(*wdata,arg1,arg2,arg3) ;
+    Bool_t ret = import(*wdata,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9) ;
     delete f ;
     return ret ;    
   }
@@ -289,7 +294,10 @@ Bool_t RooWorkspace::import(const char* fileSpec, const RooCmdArg& arg1, const R
 
 
 //_____________________________________________________________________________
-Bool_t RooWorkspace::import(const RooArgSet& args, const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3) 
+Bool_t RooWorkspace::import(const RooArgSet& args,
+			    const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3, 
+			    const RooCmdArg& arg4, const RooCmdArg& arg5, const RooCmdArg& arg6, 
+			    const RooCmdArg& arg7, const RooCmdArg& arg8, const RooCmdArg& arg9) 
 {
   // Import multiple RooAbsArg objects into workspace. For details on arguments see documentation
   // of import() method for single RooAbsArg
@@ -298,7 +306,7 @@ Bool_t RooWorkspace::import(const RooArgSet& args, const RooCmdArg& arg1, const
   RooAbsArg* oneArg ;
   Bool_t ret(kFALSE) ;
   while((oneArg=(RooAbsArg*)iter->Next())) {
-    ret |= import(*oneArg,arg1,arg2,arg3) ;
+    ret |= import(*oneArg,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9) ;
   }
   return ret ;
 }
@@ -306,7 +314,10 @@ Bool_t RooWorkspace::import(const RooArgSet& args, const RooCmdArg& arg1, const
 
 
 //_____________________________________________________________________________
-Bool_t RooWorkspace::import(const RooAbsArg& inArg, const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3) 
+Bool_t RooWorkspace::import(const RooAbsArg& inArg,
+			    const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3, 
+			    const RooCmdArg& arg4, const RooCmdArg& arg5, const RooCmdArg& arg6, 
+			    const RooCmdArg& arg7, const RooCmdArg& arg8, const RooCmdArg& arg9) 
 {
   //  Import a RooAbsArg object, e.g. function, p.d.f or variable into the workspace. This import function clones the input argument and will
   //  own the clone. If a composite object is offered for import, e.g. a p.d.f with parameters and observables, the
@@ -335,6 +346,12 @@ Bool_t RooWorkspace::import(const RooAbsArg& inArg, const RooCmdArg& arg1, const
   args.Add((TObject*)&arg1) ;
   args.Add((TObject*)&arg2) ;
   args.Add((TObject*)&arg3) ;
+  args.Add((TObject*)&arg4) ;
+  args.Add((TObject*)&arg5) ;
+  args.Add((TObject*)&arg6) ;
+  args.Add((TObject*)&arg7) ;
+  args.Add((TObject*)&arg8) ;
+  args.Add((TObject*)&arg9) ;
 
   // Select the pdf-specific commands 
   RooCmdConfig pc(Form("RooWorkspace::import(%s)",GetName())) ;
@@ -381,19 +398,19 @@ Bool_t RooWorkspace::import(const RooAbsArg& inArg, const RooCmdArg& arg1, const
   if (strlen(varChangeIn)>0) {
     
     // Parse comma separated lists into map<string,string>
-    char tmp[1024] ;
-    strlcpy(tmp,varChangeIn,1024) ;
+    char tmp[10240] ;
+    strlcpy(tmp,varChangeIn,10240) ;
     list<string> tmpIn,tmpOut ;
-    char* ptr = strtok(tmp,",") ;
+    char* ptr = strtok(tmp,", ") ;
     while (ptr) {
       tmpIn.push_back(ptr) ;
-      ptr = strtok(0,",") ;
+      ptr = strtok(0,", ") ;
     }
-    strlcpy(tmp,varChangeOut,1024) ;
-    ptr = strtok(tmp,",") ;
+    strlcpy(tmp,varChangeOut,10240) ;
+    ptr = strtok(tmp,", ") ;
     while (ptr) {
       tmpOut.push_back(ptr) ;
-      ptr = strtok(0,",") ;
+      ptr = strtok(0,", ") ;
     }    
     list<string>::iterator iin = tmpIn.begin() ;
     list<string>::iterator iout = tmpOut.begin() ;
@@ -405,13 +422,13 @@ Bool_t RooWorkspace::import(const RooAbsArg& inArg, const RooCmdArg& arg1, const
   // Process RenameAllVariables argument if specified  
   // First convert exception list if provided
   std::set<string> exceptVarNames ;
-  char tmp[1024] ;
+  char tmp[10240] ;
   if (exceptVars && strlen(exceptVars)) {
-    strlcpy(tmp,exceptVars,1024) ;
-    char* ptr = strtok(tmp,",") ;
+    strlcpy(tmp,exceptVars,10240) ;
+    char* ptr = strtok(tmp,", ") ;
     while(ptr) {
       exceptVarNames.insert(ptr) ;
-      ptr = strtok(0,",") ;
+      ptr = strtok(0,", ") ;
     }
   }
 
@@ -606,6 +623,8 @@ Bool_t RooWorkspace::import(const RooAbsArg& inArg, const RooCmdArg& arg1, const
 
       // Delete clone of incoming node
       nodesToBeDeleted.addOwned(*node) ;
+
+      //cout << "WV: recycling existing node " << existingNode << " = " << existingNode->GetName() << " for imported node " << node << endl ;
       
     } else {
       // Import node
@@ -648,7 +667,11 @@ Bool_t RooWorkspace::import(const RooAbsArg& inArg, const RooCmdArg& arg1, const
 
 
 //_____________________________________________________________________________
-Bool_t RooWorkspace::import(RooAbsData& inData, const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3) 
+Bool_t RooWorkspace::import(RooAbsData& inData, 
+			    const RooCmdArg& arg1, const RooCmdArg& arg2, const RooCmdArg& arg3, 
+			    const RooCmdArg& arg4, const RooCmdArg& arg5, const RooCmdArg& arg6, 
+			    const RooCmdArg& arg7, const RooCmdArg& arg8, const RooCmdArg& arg9) 
+
 {
   //  Import a dataset (RooDataSet or RooDataHist) into the work space. The workspace will contain a copy of the data
   //  The dataset and its variables can be renamed upon insertion with the options below
@@ -664,6 +687,12 @@ Bool_t RooWorkspace::import(RooAbsData& inData, const RooCmdArg& arg1, const Roo
   args.Add((TObject*)&arg1) ;
   args.Add((TObject*)&arg2) ;
   args.Add((TObject*)&arg3) ;
+  args.Add((TObject*)&arg4) ;
+  args.Add((TObject*)&arg5) ;
+  args.Add((TObject*)&arg6) ;
+  args.Add((TObject*)&arg7) ;
+  args.Add((TObject*)&arg8) ;
+  args.Add((TObject*)&arg9) ;
 
   // Select the pdf-specific commands 
   RooCmdConfig pc(Form("RooWorkspace::import(%s)",GetName())) ;
@@ -712,15 +741,15 @@ Bool_t RooWorkspace::import(RooAbsData& inData, const RooCmdArg& arg1, const Roo
   if (strlen(varChangeIn)>0) {
     
     // Parse comma separated lists of variable name changes
-    char tmp[1024] ;
-    strlcpy(tmp,varChangeIn,1024) ;
+    char tmp[10240] ;
+    strlcpy(tmp,varChangeIn,10240) ;
     list<string> tmpIn,tmpOut ;
     char* ptr = strtok(tmp,",") ;
     while (ptr) {
       tmpIn.push_back(ptr) ;
       ptr = strtok(0,",") ;
     }
-    strlcpy(tmp,varChangeOut,1024) ;
+    strlcpy(tmp,varChangeOut,10240) ;
     ptr = strtok(tmp,",") ;
     while (ptr) {
       tmpOut.push_back(ptr) ;
@@ -849,8 +878,8 @@ Bool_t RooWorkspace::extendSet(const char* name, const char* newContents)
   RooArgSet wsargs ;
 
   // Check all constituents of provided set
-  char buf[1024] ;
-  strlcpy(buf,newContents,1024) ;
+  char buf[10240] ;
+  strlcpy(buf,newContents,10240) ;
   char* token = strtok(buf,",") ;
   while(token) {
     // If missing, either import or report error
@@ -1005,7 +1034,7 @@ Bool_t RooWorkspace::commitTransaction()
   }
   delete iter ;
 
-  // Remove all commited objects from the sandbox
+  // Remove all committed objects from the sandbox
   _sandboxNodes.removeAll() ;
 
   // Mark transaction as finished
@@ -1047,6 +1076,7 @@ Bool_t RooWorkspace::importClassCode(const char* pat, Bool_t doReplace)
     }
   }  
   delete iter ;
+
   return ret ;
 }
 
@@ -1122,6 +1152,25 @@ Bool_t RooWorkspace::loadSnapshot(const char* name)
 }
 
 
+//_____________________________________________________________________________
+const RooArgSet* RooWorkspace::getSnapshot(const char* name) const
+{
+  // Return the RooArgSet containgin a snapshot of variables contained in the workspace
+  //
+  // Note that the variables of the objects in the snapshots are _copies_ of the
+  // variables in the workspace. To load the values of a snapshot in the workspace
+  // variables use loadSnapshot() instead
+
+  RooArgSet* snap = (RooArgSet*) _snapshots.find(name) ;
+  if (!snap) {
+    coutE(ObjectHandling) << "RooWorkspace::loadSnapshot(" << GetName() << ") no snapshot with name " << name << " is available" << endl ;
+    return 0 ;
+  }
+
+  return snap ;
+}
+
+
 
 // //_____________________________________________________________________________
 // RooAbsPdf* RooWorkspace::joinPdf(const char* jointPdfName, const char* indexName, const char* inputMapping) 
@@ -1213,8 +1262,8 @@ RooArgSet RooWorkspace::argSet(const char* nameList) const
   // Return set of RooAbsArgs matching to given list of names
   RooArgSet ret ;
 
-  char tmp[1024] ;
-  strlcpy(tmp,nameList,1024) ;
+  char tmp[10240] ;
+  strlcpy(tmp,nameList,10240) ;
   char* token = strtok(tmp,",") ;
   while(token) {
     RooAbsArg* oneArg = arg(token) ;
@@ -1574,7 +1623,7 @@ Bool_t RooWorkspace::CodeRepo::autoImportClass(TClass* tc, Bool_t doReplace)
     }
   }
   
-  char buf[1024] ;
+  char buf[10240] ;
 
   // *** Phase 3 *** Prepare to import code from files into STL string buffer
   //
@@ -1594,6 +1643,8 @@ Bool_t RooWorkspace::CodeRepo::autoImportClass(TClass* tc, Bool_t doReplace)
   string declfilebase = declfilename.substr(0,dotpos2) ;
   string declfileext = declfilename.substr(dotpos2+1) ;
 
+  list<string> extraHeaders ;
+
   // If file has not beed stored yet, enter stl strings with implementation and declaration in file map
   if (_fmap.find(declfilebase) == _fmap.end()) {
 
@@ -1616,8 +1667,42 @@ Bool_t RooWorkspace::CodeRepo::autoImportClass(TClass* tc, Bool_t doReplace)
     // Read entire file into an stl string
     string decl ;
     while(fdecl.getline(buf,1023)) {
-      decl += buf ;
-      decl += '\n' ;
+      
+      // Look for include state of self
+      Bool_t processedInclude = kFALSE ;
+      char* extincfile = 0 ;
+      
+      // Look for include of declaration file corresponding to this implementation file
+      if (strstr(buf,"#include")) {
+	// Process #include statements here
+	char tmp[10240] ;
+	strlcpy(tmp,buf,10240) ;
+	Bool_t stdinclude = strchr(buf,'<') ;
+	strtok(tmp," <\"") ;
+	char* incfile = strtok(0," <>\"") ;	
+	
+	if (!stdinclude) {
+	  // check if it lives in $ROOTSYS/include
+	  TString hpath = gSystem->Getenv("ROOTSYS") ;
+	  hpath += "/include/" ;
+	  hpath += incfile ;
+	  if (gSystem->AccessPathName(hpath.Data())) {
+	    oocoutI(_wspace,ObjectHandling)  << "RooWorkspace::autoImportClass(" << _wspace->GetName() << ") scheduling include file " << incfile << " for import" << endl ;
+	    extraHeaders.push_back(incfile) ;
+	    extincfile = incfile ;
+	    processedInclude = kTRUE ;
+	  }
+	  
+	}
+      }
+      
+      if (processedInclude) {
+	decl += "// external include file below retrieved from workspace code storage\n" ;
+	decl += Form("#include \"%s\"\n",extincfile) ; 
+      } else {
+	decl += buf ;
+	decl += '\n' ;
+      }
     }
     
     // Open implementation file
@@ -1638,38 +1723,100 @@ Bool_t RooWorkspace::CodeRepo::autoImportClass(TClass* tc, Bool_t doReplace)
       
       // Look for include state of self
       Bool_t foundSelfInclude=kFALSE ;
+      Bool_t processedInclude = kFALSE ;
+      char* extincfile = 0 ;
       
       // Look for include of declaration file corresponding to this implementation file
       if (strstr(buf,"#include")) {
 	// Process #include statements here
-	char tmp[1024] ;
-	strlcpy(tmp,buf,1024) ;
+	char tmp[10240] ;
+	strlcpy(tmp,buf,10240) ;
+	Bool_t stdinclude = strchr(buf,'<') ;
 	strtok(tmp," <\"") ;
-	char* incfile = strtok(0," <\"") ;
-	
+	char* incfile = strtok(0," <>\"") ;	
+
 	if (strstr(incfile,declfilename.c_str())) {
 	  foundSelfInclude=kTRUE ;
 	}
-      } 
+
+	if (!stdinclude && !foundSelfInclude) {
+	  // check if it lives in $ROOTSYS/include
+	  TString hpath = gSystem->Getenv("ROOTSYS") ;
+	  hpath += "/include/" ;
+	  hpath += incfile ;
+	  
+	  if (gSystem->AccessPathName(hpath.Data())) {
+	    oocoutI(_wspace,ObjectHandling)  << "RooWorkspace::autoImportClass(" << _wspace->GetName() << ") scheduling include file " << incfile << " for import" << endl ;
+	    extraHeaders.push_back(incfile) ;
+	    extincfile = incfile ;
+	    processedInclude = kTRUE ;
+	  }
+	  
+	}
+      }
       
       // Explicitly rewrite include of own declaration file to string
       // any directory prefixes, copy all other lines verbatim in stl string
       if (foundSelfInclude) {
 	// If include of self is found, substitute original include 
 	// which may have directory structure with a plain include
+	impl += "// class declaration include file below retrieved from workspace code storage\n" ;
 	impl += Form("#include \"%s.%s\"\n",declfilebase.c_str(),declfileext.c_str()) ;
+      } else if (processedInclude) {
+	impl += "// external include file below retrieved from workspace code storage\n" ;
+	impl += Form("#include \"%s\"\n",extincfile) ; 
       } else {
 	impl += buf ;
 	impl += '\n' ;
       }
     }
-    
-        
+            
     // Create entry in file map
     _fmap[declfilebase]._hfile = decl ;
     _fmap[declfilebase]._cxxfile = impl ;   
     _fmap[declfilebase]._hext = declfileext ;
 
+    // Process extra includes now
+    for (list<string>::iterator ehiter = extraHeaders.begin() ; ehiter != extraHeaders.end() ; ++ehiter ) {
+      if (_ehmap.find(*ehiter) == _ehmap.end()) {
+
+	ExtraHeader eh ;
+	eh._hname = ehiter->c_str() ;
+	fstream fehdr(ehiter->c_str()) ;
+	string ehimpl ;
+	char buf2[1024] ;
+	while(fehdr.getline(buf2,1023)) {	  
+	  
+	  // Look for include of declaration file corresponding to this implementation file
+	  if (strstr(buf2,"#include")) {
+	    // Process #include statements here
+	    char tmp[10240] ;
+	    strlcpy(tmp,buf2,10240) ;
+	    Bool_t stdinclude = strchr(buf,'<') ;
+	    strtok(tmp," <\"") ;
+	    char* incfile = strtok(0," <>\"") ;	
+
+	    if (!stdinclude) {
+	      // check if it lives in $ROOTSYS/include
+	      TString hpath = gSystem->Getenv("ROOTSYS") ;
+	      hpath += "/include/" ;
+	      hpath += incfile ;
+	      if (gSystem->AccessPathName(hpath.Data())) {
+		oocoutI(_wspace,ObjectHandling)  << "RooWorkspace::autoImportClass(" << _wspace->GetName() << ") scheduling recursive include file " << incfile << " for import" << endl ;
+		extraHeaders.push_back(incfile) ;
+	      }	      
+	    }
+	  }
+      
+	  ehimpl += buf2 ;
+	  ehimpl += '\n' ;
+	}
+	eh._hfile = ehimpl.c_str() ;
+
+	_ehmap[ehiter->c_str()] = eh  ;
+      }
+    }
+
   } else {
 
     // Inform that existing file entry is being recycled because it already contained class code
@@ -1720,6 +1867,7 @@ Bool_t RooWorkspace::CodeRepo::autoImportClass(TClass* tc, Bool_t doReplace)
     delete[] declpath ;
   }
 
+
   return kTRUE ;
 }
 
@@ -1774,6 +1922,7 @@ Bool_t RooWorkspace::import(TObject& object, Bool_t replaceExisting)
   // Grab the current state of the directory Auto-Add
   ROOT::DirAutoAdd_t func = object.IsA()->GetDirectoryAutoAdd();
   object.IsA()->SetDirectoryAutoAdd(0);
+  Bool_t tmp = RooPlot::setAddDirectoryStatus(kFALSE) ;
 
   if (oldObj) {
     _genObjects.Replace(oldObj,object.Clone()) ;
@@ -1784,6 +1933,7 @@ Bool_t RooWorkspace::import(TObject& object, Bool_t replaceExisting)
 
   // Reset the state of the directory Auto-Add
   object.IsA()->SetDirectoryAutoAdd(func);
+  RooPlot::setAddDirectoryStatus(tmp) ;
 
   return kFALSE ;
 }
@@ -2220,7 +2370,7 @@ void RooWorkspace::CodeRepo::Streamer(TBuffer &R__b)
    if (R__b.IsReading()) {
 
      UInt_t R__s, R__c;
-     R__b.ReadVersion(&R__s, &R__c); 
+     Version_t R__v =  R__b.ReadVersion(&R__s, &R__c); 
 
      // Stream contents of ClassFiles map
      Int_t count(0) ;
@@ -2237,11 +2387,24 @@ void RooWorkspace::CodeRepo::Streamer(TBuffer &R__b)
      count=0 ;
      R__b >> count ;
      while(count--) {
-       TString name,bname,fbase ;
+       TString name ;
        name.Streamer(R__b) ;       
        _c2fmap[name]._baseName.Streamer(R__b) ;
        _c2fmap[name]._fileBase.Streamer(R__b) ;
      }     
+
+     if (R__v==2) {
+
+       count=0 ;
+       R__b >> count ;
+       while(count--) {
+	 TString name ;
+	 name.Streamer(R__b) ;       
+	 _ehmap[name]._hname.Streamer(R__b) ;
+	 _ehmap[name]._hfile.Streamer(R__b) ;
+       }                  
+     }
+
      R__b.CheckByteCount(R__s, R__c, thisClass::IsA());
 
      // Instantiate any classes that are not defined in current session
@@ -2278,6 +2441,18 @@ void RooWorkspace::CodeRepo::Streamer(TBuffer &R__b)
        ++iter2 ;
      }
 
+     // Stream contents of ExtraHeader map
+     count = _ehmap.size() ;
+     R__b << count ;
+     map<TString,ExtraHeader>::iterator iter3 = _ehmap.begin() ;
+     while(iter3!=_ehmap.end()) {
+       TString key_copy(iter3->first) ;
+       key_copy.Streamer(R__b) ;
+       iter3->second._hname.Streamer(R__b) ;
+       iter3->second._hfile.Streamer(R__b);
+       ++iter3 ;
+     }
+
      R__b.SetByteCount(R__c, kTRUE);
      
    }
@@ -2297,11 +2472,18 @@ void RooWorkspace::Streamer(TBuffer &R__b)
    if (R__b.IsReading()) {
 
       R__b.ReadClassBuffer(RooWorkspace::Class(),this);
-
+            
+      // Perform any pass-2 schema evolution here
+      RooFIter fiter = _allOwnedNodes.fwdIterator() ;
+      RooAbsArg* node ;
+      while((node=fiter.next())) {
+	node->ioStreamerPass2() ;
+      }
+      RooAbsArg::ioStreamerPass2Finalize() ;
+      
       // Make expensive object cache of all objects point to intermal copy.
       // Somehow this doesn't work OK automatically
       TIterator* iter = _allOwnedNodes.createIterator() ;
-      RooAbsArg* node ;
       while((node=(RooAbsArg*)iter->Next())) {
 	node->setExpensiveObjectCache(_eocache) ;
 	if (node->IsA()->InheritsFrom(RooAbsOptTestStatistic::Class())) {
@@ -2316,7 +2498,6 @@ void RooWorkspace::Streamer(TBuffer &R__b)
 
    } else {
 
-
      // Make lists of external clients of WS objects, and remove those links temporarily
 
      map<RooAbsArg*,list<RooAbsArg*> > extClients, extValueClients, extShapeClients ;
@@ -2436,6 +2617,8 @@ Bool_t RooWorkspace::CodeRepo::compileClasses()
   // Retrieve name of directory in which to export code files
   string dirName = Form(_classFileExportDir.c_str(),_wspace->uuid().AsString(),_wspace->GetName()) ;
 
+  Bool_t writeExtraHeaders(kFALSE) ;
+
   // Process all class entries in repository
   map<TString,ClassRelInfo>::iterator iter = _c2fmap.begin() ;
   while(iter!=_c2fmap.end()) {
@@ -2468,8 +2651,53 @@ Bool_t RooWorkspace::CodeRepo::compileClasses()
 	}
       }
       haveDir=kTRUE ;
+
     }
 
+    // First write any extra header files 
+    if (!writeExtraHeaders) {      
+      writeExtraHeaders = kTRUE ;
+      
+      map<TString,ExtraHeader>::iterator eiter = _ehmap.begin() ;
+      while(eiter!=_ehmap.end()) {
+
+	// Check if identical declaration file (header) is already written
+	Bool_t needEHWrite=kTRUE ;
+	string fdname = Form("%s/%s",dirName.c_str(),eiter->second._hname.Data()) ;
+	ifstream ifdecl(fdname.c_str()) ;
+	if (ifdecl) {
+	  TString contents ;
+	  char buf[10240] ;
+	  while(ifdecl.getline(buf,10240)) {
+	    contents += buf ;
+	    contents += "\n" ;
+	  }      
+	  UInt_t crcFile = RooAbsArg::crc32(contents.Data()) ;
+	  UInt_t crcWS   = RooAbsArg::crc32(eiter->second._hfile.Data()) ;
+	  needEHWrite = (crcFile!=crcWS) ;
+	}
+	
+	// Write declaration file if required 
+	if (needEHWrite) {
+	  oocoutI(_wspace,ObjectHandling) << "RooWorkspace::CodeRepo::compileClasses() Extracting extra header file " << fdname << endl ;
+	  
+	  // Extra headers may contain non-existing path - create first to be sure
+	  gSystem->MakeDirectory(gSystem->DirName(fdname.c_str())) ;	  
+
+	  ofstream fdecl(fdname.c_str()) ;
+	  if (!fdecl) {
+	    oocoutE(_wspace,ObjectHandling) << "RooWorkspace::CodeRepo::compileClasses() ERROR opening file " 
+					    << fdname << " for writing" << endl ;
+	    return kFALSE ;
+	  }
+	  fdecl << eiter->second._hfile.Data() ;
+	  fdecl.close() ;
+	}	       
+	eiter++ ;
+      }
+    }
+    
+
     // Navigate from class to file
     ClassFiles& cfinfo = _fmap[iter->second._fileBase] ;
 
@@ -2488,8 +2716,8 @@ Bool_t RooWorkspace::CodeRepo::compileClasses()
     ifstream ifdecl(fdname.c_str()) ;
     if (ifdecl) {
       TString contents ;
-      char buf[1024] ;
-      while(ifdecl.getline(buf,1024)) {
+      char buf[10240] ;
+      while(ifdecl.getline(buf,10240)) {
 	contents += buf ;
 	contents += "\n" ;
       }      
@@ -2503,7 +2731,7 @@ Bool_t RooWorkspace::CodeRepo::compileClasses()
       oocoutI(_wspace,ObjectHandling) << "RooWorkspace::CodeRepo::compileClasses() Extracting declaration code of class " << iter->first << ", file " << fdname << endl ;
       ofstream fdecl(fdname.c_str()) ;
       if (!fdecl) {
-	oocoutE(_wspace,ObjectHandling) << "RooWorkspace::CodeRepo::compileClasses() ERROR opening file" 
+	oocoutE(_wspace,ObjectHandling) << "RooWorkspace::CodeRepo::compileClasses() ERROR opening file " 
 					<< fdname << " for writing" << endl ;
 	return kFALSE ;
       }
@@ -2517,8 +2745,8 @@ Bool_t RooWorkspace::CodeRepo::compileClasses()
     ifstream ifimpl(finame.c_str()) ;
     if (ifimpl) {
       TString contents ;
-      char buf[1024] ;
-      while(ifimpl.getline(buf,1024)) {
+      char buf[10240] ;
+      while(ifimpl.getline(buf,10240)) {
 	contents += buf ;
 	contents += "\n" ;
       }      
@@ -2626,7 +2854,6 @@ void RooWorkspace::exportToCint(const char* nsname)
     return ;
   }
 
-
   // Set flag so that future import to workspace are automatically exported to CINT
   _doExport = kTRUE ;
 
@@ -2704,12 +2931,12 @@ Bool_t RooWorkspace::isValidCPPID(const char* name)
 void RooWorkspace::unExport()
 {
   // Delete exported reference in CINT namespace 
-  char buf[1024] ;
+  char buf[10240] ;
   TIterator* iter = _allOwnedNodes.createIterator() ;
   TObject* wobj ;
   while((wobj=iter->Next())) {
     if (isValidCPPID(wobj->GetName())) {
-      strlcpy(buf,Form("%s::%s",_exportNSName.c_str(),wobj->GetName()),1024) ;
+      strlcpy(buf,Form("%s::%s",_exportNSName.c_str(),wobj->GetName()),10240) ;
       G__deletevariable(buf) ;
     }
   }
