@@ -1,4 +1,4 @@
-// @(#)root/pgsql:$Id: TPgSQLStatement.cxx 40996 2011-09-24 05:58:21Z pcanal $
+// @(#)root/pgsql:$Id$
 // Author: Dennis Box (dbox@fnal.gov)  3/12/2007
 
 /*************************************************************************
@@ -27,6 +27,8 @@ ClassImp(TPgSQLStatement)
 
 #ifdef PG_VERSION_NUM
 
+#include "libpq/libpq-fs.h"
+
 static const Int_t kBindStringSize = 25;
 
 //______________________________________________________________________________
@@ -46,7 +48,7 @@ TPgSQLStatement::TPgSQLStatement(PgSQL_Stmt_t* stmt, Bool_t errout):
    // Normal constructor.
    // Checks if statement contains parameters tags.
 
-   fStmt->fRes = PQdescribePrepared(fStmt->fConn,"");
+   fStmt->fRes = PQdescribePrepared(fStmt->fConn,"preparedstmt");
    unsigned long paramcount = PQnparams(fStmt->fRes);
    fNumResultCols = PQnfields(fStmt->fRes);
    fIterationCount = -1;
@@ -78,6 +80,9 @@ void TPgSQLStatement::Close(Option_t *)
 
    fStmt->fRes = 0;
 
+   PGresult *res=PQexec(fStmt->fConn,"DEALLOCATE preparedstmt;");
+   PQclear(res);
+
    FreeBuffers();
    //TPgSQLServers responsibility to free connection
    fStmt->fConn=0;
@@ -106,6 +111,24 @@ void TPgSQLStatement::Close(Option_t *)
       }                                                 \
    }
 
+#define CheckErrResult(method, pqresult, retVal)         \
+   {                                                     \
+      ExecStatusType stmterrno=PQresultStatus(pqresult); \
+      if (!pgsql_success(stmterrno)) {                   \
+       const char* stmterrmsg = PQresultErrorMessage(fStmt->fRes);  \
+       SetError(stmterrno, stmterrmsg, method);                     \
+       PQclear(res);                                     \
+       return retVal;                                    \
+     }                                                   \
+   }
+
+#define RollBackTransaction(method)                          \
+   {                                                         \
+      PGresult *resnum=PQexec(fStmt->fConn,"COMMIT");        \
+      CheckErrResult("RollBackTransaction", resnum, kFALSE); \
+      PQclear(res);                                          \
+   }
+
 // check last pgsql statement error code
 #define CheckGetField(method, res)                      \
    {                                                    \
@@ -128,13 +151,13 @@ Bool_t TPgSQLStatement::Process()
    CheckStmt("Process",kFALSE);
 
    if (IsSetParsMode()) {
-      fStmt->fRes= PQexecPrepared(fStmt->fConn,"",fNumBuffers,
+      fStmt->fRes= PQexecPrepared(fStmt->fConn,"preparedstmt",fNumBuffers,
                                  (const char* const*)fBind,
                                  0,0,0);
 
    } else { //result set mode
 
-      fStmt->fRes= PQexecPrepared(fStmt->fConn,"",0,(const char* const*) 0,0,0,0);
+      fStmt->fRes= PQexecPrepared(fStmt->fConn,"preparedstmt",0,(const char* const*) 0,0,0,0);
    }
    ExecStatusType stat = PQresultStatus(fStmt->fRes);
    if (!pgsql_success(stat))
@@ -159,11 +182,11 @@ Int_t TPgSQLStatement::GetNumParameters()
 
    CheckStmt("GetNumParameters", -1);
 
-   Int_t res = PQnparams(fStmt->fRes);
-
-   CheckErrNo("GetNumParameters", kFALSE, -1);
-
-   return res;
+   if (IsSetParsMode()) {
+      return fNumBuffers;
+   } else {
+      return 0;
+   }
 }
 
 //______________________________________________________________________________
@@ -242,7 +265,7 @@ Bool_t TPgSQLStatement::NextIteration()
 
    if (fIterationCount==0) return kTRUE;
 
-   fStmt->fRes= PQexecPrepared(fStmt->fConn,"",fNumBuffers,
+   fStmt->fRes= PQexecPrepared(fStmt->fConn,"preparedstmt",fNumBuffers,
                                (const char* const*)fBind,
                                0,//fParamLengths,
                                0,//fParamFormats,
@@ -436,6 +459,73 @@ Bool_t TPgSQLStatement::GetBinary(Int_t npar, void* &mem, Long_t& size)
    return kTRUE;
 }
 
+//______________________________________________________________________________
+Bool_t TPgSQLStatement::GetLargeObject(Int_t npar, void* &mem, Long_t& size)
+{
+   // Return large object whose oid is in the given field.
+
+   Int_t objID = atoi(PQgetvalue(fStmt->fRes,fIterationCount,npar));
+
+   // All this needs to happen inside a transaction, or it will NOT work.
+   PGresult *res=PQexec(fStmt->fConn,"BEGIN");
+
+   CheckErrResult("GetLargeObject", res, kFALSE);
+   PQclear(res);
+
+   Int_t lObjFD = lo_open(fStmt->fConn, objID, INV_READ);
+
+   if (lObjFD<0) {
+      Error("GetLargeObject", "SQL Error on lo_open: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+   // Object size is not known beforehand.
+   // Possible fast ways to get it are:
+   // (1) Create a function that does fopen, fseek, ftell on server
+   // (2) Query large object table with size()
+   // Both can not be expected to work in general,
+   // as  (1) needs permissions and changes DB,
+   // and (2) needs permission.
+   // So we use
+   // (3) fopen, fseek and ftell locally.
+
+   lo_lseek(fStmt->fConn, lObjFD, 0, SEEK_END);
+   Long_t sz = lo_tell(fStmt->fConn, lObjFD);
+   lo_lseek(fStmt->fConn, lObjFD, 0, SEEK_SET);
+
+   if ((Long_t)sz>size) {
+      delete [] (unsigned char*) mem;
+      mem = (void*) new unsigned char[sz];
+      size=sz;
+   }
+
+   Int_t readBytes = lo_read(fStmt->fConn, lObjFD, (char*)mem, size);
+
+   if (readBytes != sz) {
+      Error("GetLargeObject", "SQL Error on lo_read: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+
+   if (lo_close(fStmt->fConn, lObjFD) != 0) {
+      Error("GetLargeObject", "SQL Error on lo_close: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+
+   res=PQexec(fStmt->fConn,"COMMIT");
+
+   ExecStatusType stat = PQresultStatus(res);
+   if (!pgsql_success(stat)) {
+      Error("GetLargeObject", "SQL Error on COMMIT: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+   PQclear(res);
+
+   return kTRUE;
+}
+
 //______________________________________________________________________________
 Bool_t TPgSQLStatement::GetDate(Int_t npar, Int_t& year, Int_t& month, Int_t& day)
 {
@@ -603,6 +693,59 @@ Bool_t TPgSQLStatement::SetBinary(Int_t npar, void* mem, Long_t size, Long_t max
    return kTRUE;
 }
 
+//______________________________________________________________________________
+Bool_t TPgSQLStatement::SetLargeObject(Int_t npar, void* mem, Long_t size, Long_t /*maxsize*/)
+{
+   // Set parameter value to large object and immediately insert the large object into DB.
+
+   // All this needs to happen inside a transaction, or it will NOT work.
+   PGresult *res=PQexec(fStmt->fConn,"BEGIN");
+
+   CheckErrResult("GetLargeObject", res, kFALSE);
+   PQclear(res);
+
+   Int_t lObjID = lo_creat(fStmt->fConn, INV_READ | INV_WRITE);
+   if (lObjID<0) {
+      Error("SetLargeObject", "Error in SetLargeObject: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+
+   Int_t lObjFD = lo_open(fStmt->fConn, lObjID, INV_READ | INV_WRITE);
+   if (lObjFD<0) {
+      Error("SetLargeObject", "Error in SetLargeObject: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+
+   Int_t writtenBytes = lo_write(fStmt->fConn, lObjFD, (char*)mem, size);
+
+   if (writtenBytes != size) {
+      Error("SetLargeObject", "SQL Error on lo_write: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+
+   if (lo_close(fStmt->fConn, lObjFD) != 0) {
+      Error("SetLargeObject", "SQL Error on lo_close: %s", PQerrorMessage(fStmt->fConn));
+      RollBackTransaction("GetLargeObject");
+      return kFALSE;
+   }
+
+   res=PQexec(fStmt->fConn,"COMMIT");
+   ExecStatusType stat = PQresultStatus(res);
+   if (!pgsql_success(stat)) {
+      Error("SetLargeObject", "SQL Error on COMMIT: %s", PQerrorMessage(fStmt->fConn));
+      PQclear(res);
+      return kFALSE;
+   }
+   PQclear(res);
+
+   snprintf(fBind[npar],kBindStringSize,"%d",lObjID);
+
+   return kTRUE;
+}
+
 //______________________________________________________________________________
 Bool_t TPgSQLStatement::SetDate(Int_t npar, Int_t year, Int_t month, Int_t day)
 {
@@ -611,7 +754,7 @@ Bool_t TPgSQLStatement::SetDate(Int_t npar, Int_t year, Int_t month, Int_t day)
    TDatime d =TDatime(year,month,day,0,0,0);
    snprintf(fBind[npar],kBindStringSize,"%s",(char*)d.AsSQLString());
 
-   return kFALSE;
+   return kTRUE;
 }
 
 //______________________________________________________________________________
@@ -833,6 +976,13 @@ Bool_t TPgSQLStatement::GetBinary(Int_t, void* &, Long_t&)
    return kFALSE;
 }
 
+//______________________________________________________________________________
+Bool_t TPgSQLStatement::GetLargeObject(Int_t, void* &, Long_t&)
+{
+   // Return large object whose oid is in the given field.
+
+   return kFALSE;
+}
 
 //______________________________________________________________________________
 Bool_t TPgSQLStatement::GetDate(Int_t, Int_t&, Int_t&, Int_t&)
@@ -954,6 +1104,14 @@ Bool_t TPgSQLStatement::SetBinary(Int_t, void*, Long_t, Long_t)
    return kFALSE;
 }
 
+//______________________________________________________________________________
+Bool_t TPgSQLStatement::SetLargeObject(Int_t, void*, Long_t, Long_t)
+{
+   // Set parameter value to large object and immediately insert the large object into DB.
+
+   return kFALSE;
+}
+
 //______________________________________________________________________________
 Bool_t TPgSQLStatement::SetDate(Int_t, Int_t, Int_t, Int_t)
 {
