@@ -1,4 +1,4 @@
-// @(#)root/tmva $Id: MethodMLP.cxx 44112 2012-05-04 10:00:41Z evt $
+// @(#)root/tmva $Id$
 // Author: Krzysztof Danielowski, Andreas Hoecker, Matt Jachowski, Kamil Kraszewski, Maciej Kruk, Peter Speckmayer, Joerg Stelzer, Eckhard v. Toerne, Jan Therhaag, Jiahang Zhong
 
 /**********************************************************************************
@@ -1657,7 +1657,7 @@ void TMVA::MethodMLP::GetHelpMessage() const
          << "only the TMlpANN performs an explicit separation of the" << Endl;
    Log() << "full training sample into independent training and validation samples." << Endl;
    Log() << "We have found that in most high-energy physics applications the " << Endl;
-   Log() << "avaliable degrees of freedom (training events) are sufficient to " << Endl;
+   Log() << "available degrees of freedom (training events) are sufficient to " << Endl;
    Log() << "constrain the weights of the relatively simple architectures required" << Endl;
    Log() << "to achieve good performance. Hence no overtraining should occur, and " << Endl;
    Log() << "the use of validation samples would only reduce the available training" << Endl;
