@@ -1,4 +1,4 @@
-// @(#)root/tmva $Id: PDEFoam.cxx 44110 2012-05-04 08:34:05Z evt $
+// @(#)root/tmva $Id$
 // Author: S.Jadach, Tancredi Carli, Dominik Dannheim, Alexander Voigt
 
 /**********************************************************************************
@@ -539,7 +539,7 @@ void TMVA::PDEFoam::Explore(PDEFoamCell *cell)
    Double_t intTrue = ceSum[0]/(nevMC+0.000001);
    Double_t intDriv=0.;
 
-   if (kBest == -1) Varedu(ceSum,kBest,xBest,yBest); // determine the best edge,
+   Varedu(ceSum,kBest,xBest,yBest);        // determine the best division edge,
    intDriv =sqrt(ceSum[1]/nevMC) -intTrue; // Foam build-up, sqrt(<w**2>) -<w>
 
    //=================================================================================
@@ -1471,7 +1471,7 @@ void TMVA::PDEFoam::RootPlot2dim( const TString& filename, TString opt,
    // as rectangles in C++ format readable for ROOT.
    //
    // Parameters:
-   // - filename - filename of ouput root macro
+   // - filename - filename of output root macro
    //
    // - opt - cell_value, rms, rms_ov_mean
    //   If cell_value is set, the following values will be filled into
