@@ -1,4 +1,4 @@
-// @(#)root/tree:$Id: TIndArray.h 30815 2009-10-20 13:49:22Z rdm $
+// @(#)root/tree:$Id$
 // Author:  Lukasz Janyst <ljanyst@cern.ch> 23/01/2008
 
 //------------------------------------------------------------------------------
@@ -24,9 +24,9 @@ class TIndArray
          delete [] fArr;
       }
 
-      void Reserve( UInt_t size )
+      void ClearAndResize( UInt_t size )
       {
-         delete fArr;
+         delete [] fArr;
          fElems = 0;
          fArr = new UChar_t[size];
          fCapacity = size;
