@@ -1,4 +1,4 @@
-// @(#)root/tree:$Id: TBranchSTL.cxx 41244 2011-10-06 19:42:35Z pcanal $
+// @(#)root/tree:$Id$
 // Author Lukasz Janyst <ljanyst@cern.ch>  23/01/2008
 
 //////////////////////////////////////////////////////////////////////////
@@ -216,7 +216,7 @@ Int_t TBranchSTL::Fill()
    // Set up the container of indices
    //---------------------------------------------------------------------------
    if( fInd.GetCapacity() < size )
-      fInd.Reserve( size );
+      fInd.ClearAndResize( size );
 
    fInd.SetNumItems( size );
 
