@@ -1,4 +1,4 @@
-// @(#)root/tree:$Id: TTreeCache.cxx 43934 2012-04-24 21:55:59Z pcanal $
+// @(#)root/tree:$Id$
 // Author: Rene Brun   04/06/2006
 
 /*************************************************************************
@@ -265,7 +265,9 @@ TTreeCache::TTreeCache() : TFileCacheRead(),
    fFillTimes(0),
    fFirstTime(kTRUE),
    fFirstEntry(-1),
-   fReadDirectionSet(kFALSE)
+   fReadDirectionSet(kFALSE),
+   fEnabled(kTRUE),
+   fPrefillType(TTreeCache::kNoPrefill)
 {
    // Default Constructor.
 }
@@ -291,7 +293,9 @@ TTreeCache::TTreeCache(TTree *tree, Int_t buffersize) : TFileCacheRead(tree->Get
    fFillTimes(0),
    fFirstTime(kTRUE),                                                        
    fFirstEntry(-1),
-   fReadDirectionSet(kFALSE)
+   fReadDirectionSet(kFALSE),
+   fEnabled(kTRUE),
+   fPrefillType(TTreeCache::kNoPrefill)
 {
    // Constructor.
 
@@ -324,6 +328,12 @@ void TTreeCache::AddBranch(TBranch *b, Bool_t subbranches /*= kFALSE*/)
    // Reject branch that are not from the cached tree.
    if (!b || fTree->GetTree() != b->GetTree()) return;
 
+   // Is this the first addition of a branch (and we are learning and we are in
+   // the expected TTree), then prefill the cache.  (We expect that in future
+   // release the Prefill-ing will be the default so we test for that inside the
+   // LearnPrefill call).
+   if (fNbranches == 0 && b->GetReadEntry() == fEntryMin) LearnPrefill();
+
    //Is branch already in the cache?
    Bool_t isNew = kTRUE;
    for (int i=0;i<fNbranches;i++) {
@@ -357,7 +367,7 @@ void TTreeCache::AddBranch(const char *bname, Bool_t subbranches /*= kFALSE*/)
    // this is to be used by user (thats why we pass the name of the branch).
    // It works in exactly the same way as TTree::SetBranchStatus so you
    // probably want to look over ther for details about the use of bname
-   // with regular expresions.
+   // with regular expressions.
    // The branches are taken with respect to the Owner of this TTreeCache
    // (i.e. the original Tree)
    // NB: if bname="*" all branches are put in the cache and the learning phase stopped
@@ -439,8 +449,8 @@ void TTreeCache::AddBranch(const char *bname, Bool_t subbranches /*= kFALSE*/)
 //_____________________________________________________________________________
 void TTreeCache::DropBranch(TBranch *b, Bool_t subbranches /*= kFALSE*/)
 {
-   //Remove a branch to the list of branches to be stored in the cache
-   //this function is called by TBranch::GetBasket
+   // Remove a branch to the list of branches to be stored in the cache
+   // this function is called by TBranch::GetBasket.
 
    if (!fIsLearning) return;
 
@@ -584,12 +594,13 @@ Bool_t TTreeCache::FillBuffer()
             }
          }
 
-         if (fReverseRead){ 
-            //reverse reading with prefetching
-            if (fEntryCurrent >0 && entry < fEntryNext){
-               //we can prefetch the next buffer
-               if (entry > fEntryCurrent)
+         if (fReverseRead) { 
+            // Reverse reading with prefetching
+            if (fEntryCurrent >0 && entry < fEntryNext) {
+               // We can prefetch the next buffer
+               if (entry >= fEntryCurrent) {
                   entry = fEntryCurrent - tree->GetAutoFlush() * fFillTimes;
+               }
                if (entry < 0) entry = 0;
             }
             else if (fEntryCurrent >= 0)
@@ -602,11 +613,12 @@ Bool_t TTreeCache::FillBuffer()
          else {
             //normal reading with prefetching
             if (fEnablePrefetching){
-               if (entry < 0 && fEntryNext > 0)
+               if (entry < 0 && fEntryNext > 0) {
                   entry = fEntryCurrent;
-               else if (entry > fEntryCurrent){
-                  if (entry < fEntryNext)
+               } else if (entry >= fEntryCurrent) {
+                  if (entry < fEntryNext) {
                      entry = fEntryNext;
+                  }
                }
                else {
                   //we are still reading from the oldest buffer, no need to prefetch a new one
@@ -636,6 +648,13 @@ Bool_t TTreeCache::FillBuffer()
    if (fEntryMax <= 0) fEntryMax = tree->GetEntries();
    if (fEntryNext > fEntryMax) fEntryNext = fEntryMax;
 
+   if ( fEnablePrefetching ) {
+      if ( entry == fEntryMax ) {
+         // We are at the end, no need to do anything else
+         return kFALSE;
+      }
+   }
+
    // Check if owner has a TEventList set. If yes we optimize for this
    // Special case reading only the baskets containing entries in the
    // list.
@@ -822,18 +841,15 @@ Bool_t TTreeCache::FillBuffer()
       if (fEntryNext > fEntryMax) fEntryNext = fEntryMax;
    } while (kTRUE);
 
-   //in learning mode clear cache
-   if (fIsLearning) {
-      fEntryNext = -1;
-      fEntryCurrent = -1;
-      TFileCacheRead::Prefetch(0, 0);
-      TFileCacheRead::SecondPrefetch(0, 0);
-      fFirstBuffer = !fFirstBuffer;
-   }
-   if (!fIsLearning && fFirstTime){
-      // first time we add autoFlush entries , after fFillTimes * autoFlush
-      // only in reverse prefetching mode
-      fFirstTime = kFALSE;
+   if (fEnablePrefetching) {
+      if (fIsLearning) {
+         fFirstBuffer = !fFirstBuffer;
+      }
+      if (!fIsLearning && fFirstTime){
+         // First time we add autoFlush entries , after fFillTimes * autoFlush
+         // only in reverse prefetching mode
+         fFirstTime = kFALSE;
+      }
    }
    fIsLearning = kFALSE;
    return kTRUE;
@@ -905,7 +921,7 @@ void TTreeCache::Print(Option_t *option) const
 
    TString opt = option;
    opt.ToLower();
-   printf("******TreeCache statistics for file: %s ******\n",fFile ? fFile->GetName() : "no file set");
+   printf("******TreeCache statistics for tree: %s in file: %s ******\n",fTree ? fTree->GetName() : "no tree set",fFile ? fFile->GetName() : "no file set");
    if (fNbranches <= 0) return;
    printf("Number of branches in the cache ...: %d\n",fNbranches);
    printf("Cache Efficiency ..................: %f\n",GetEfficiency());
@@ -969,13 +985,19 @@ Int_t TTreeCache::ReadBufferPrefetch(char *buf, Long64_t pos, Int_t len){
       return 1;
    }
 
-   //keep on prefetching until request is satisfied
-   while (1){
+   // try to prefetch a couple of times and if request is still not satisfied then
+   // fall back to normal reading without prefetching for the current request
+   Int_t counter = 0;
+   while (1) {
       if(TFileCacheRead::ReadBuffer(buf, pos, len)) {
          break;
       }
       FillBuffer();
       fNReadMiss++;
+      counter++;
+      if (counter>1) {
+        return 0;
+      }
    }
 
    fNReadOk++;
@@ -995,6 +1017,8 @@ Int_t TTreeCache::ReadBuffer(char *buf, Long64_t pos, Int_t len)
    //     1 in case read from cache.
    // This function overloads TFileCacheRead::ReadBuffer.
 
+   if (!fEnabled) return 0;
+
    if (fEnablePrefetching)
       return TTreeCache::ReadBufferPrefetch(buf, pos, len);
    else
@@ -1037,6 +1061,22 @@ void TTreeCache::SetEntryRange(Long64_t emin, Long64_t emax)
    }
 }
 
+//_____________________________________________________________________________
+void TTreeCache::SetFile(TFile *file, TFile::ECacheAction action)
+{
+   // Overload to make sure that the object specific
+
+   // The infinite recursion is 'broken' by the fact that
+   // TFile::SetCacheRead remove the entry from fCacheReadMap _before_
+   // calling SetFile (and also by setting fFile to zero before the calling).
+   if (fFile) {
+      TFile *prevFile = fFile;
+      fFile = 0;
+      prevFile->SetCacheRead(0, fTree, action);
+   }
+   TFileCacheRead::SetFile(file, action);
+}
+
 //_____________________________________________________________________________
 void TTreeCache::SetLearnEntries(Int_t n)
 {
@@ -1047,6 +1087,18 @@ void TTreeCache::SetLearnEntries(Int_t n)
    fgLearnEntries = n;
 }
 
+//_____________________________________________________________________________
+void TTreeCache::SetLearnPrefill(TTreeCache::EPrefillType type /* = kNoPrefill */)
+{
+   // Set whether the learning period is started with a prefilling of the
+   // cache and which type of prefilling is used.
+   // The two value currently supported are:
+   //   TTreeCache::kNoPrefill    disable the prefilling
+   //   TTreeCache::kAllBranches  fill the cache with baskets from all branches.
+
+   fPrefillType = type;
+}
+
 //_____________________________________________________________________________
 void TTreeCache::StartLearningPhase()
 {
@@ -1120,3 +1172,51 @@ void TTreeCache::UpdateBranches(TTree *tree)
       fNbranches++;
    }
 }
+
+//_____________________________________________________________________________
+void TTreeCache::LearnPrefill()
+{
+   // Perform an initial prefetch, attempting to read as much of the learning
+   // phase baskets for all branches at once
+
+   // This is meant for the learning phase
+   if (!fIsLearning) return;
+
+   // This should be called before reading entries, otherwise we'll
+   // always exit here, since TBranch adds itself before reading
+   if (fNbranches > 0) return;
+
+   // Is the LearnPrefill enabled (using an Int_t here to allow for future
+   // extension to alternative Prefilling).
+   if (fPrefillType == kNoPrefill) return;
+
+   // Force only the learn entries to be cached by temporarily setting min/max
+   // to the learning phase entry range
+   // But save all the old values, so we can restore everything to how it was
+   Long64_t eminOld = fEntryMin;
+   Long64_t emaxOld = fEntryMax;
+   Long64_t ecurrentOld = fEntryCurrent;
+   Long64_t enextOld = fEntryNext;
+
+   fEntryMin = fEntryCurrent;
+   fEntryMax = fEntryNext;
+
+   // Add all branches to be cached. This also sets fIsManual, stops learning,
+   // and makes fEntryNext = -1 (which forces a cache fill, which is good)
+   AddBranch("*");
+   fIsManual = kFALSE; // AddBranch sets fIsManual, so we reset it
+
+   // Now, fill the buffer with the learning phase entry range
+   FillBuffer();
+
+   // Leave everything the way we found it
+   fIsLearning = kTRUE;
+   DropBranch("*"); // This doesn't work unless we're already learning
+
+   // Restore entry values
+   fEntryMin = eminOld;
+   fEntryMax = emaxOld;
+   fEntryCurrent = ecurrentOld;
+   fEntryNext = enextOld;
+}
+
