@@ -1,4 +1,4 @@
-// @(#)root/treeplayer:$Id: TTreeProxyGenerator.cxx 43993 2012-04-27 16:23:16Z pcanal $
+// @(#)root/treeplayer:$Id$
 // Author: Philippe Canal 06/06/2004
 
 /*************************************************************************
@@ -1354,7 +1354,7 @@ static TVirtualStreamerInfo *GetBaseClass(TStreamerElement *element)
          if ( branch->GetListOfBranches()->GetEntries() == 0 ) {
 
             if (cl) {
-               // We have a non-splitted object!
+               // We have a non-split object!
 
                if (desc) {
                   TVirtualStreamerInfo *cinfo = cl->GetStreamerInfo();
@@ -1382,7 +1382,7 @@ static TVirtualStreamerInfo *GetBaseClass(TStreamerElement *element)
 
          } else {
 
-            // We have a splitted object
+            // We have a split object
 
             TIter subnext( branch->GetListOfBranches() );
             if (desc) {
@@ -1462,7 +1462,45 @@ static TVirtualStreamerInfo *GetBaseClass(TStreamerElement *element)
       if (!element) return;
 
       if (strcmp(element->GetName(),"This")==0) {
-         // Skip the artifical streamer element.
+         TClass *cl = element->GetClassPointer();
+         containerName = cl->GetName();
+         cl = cl->GetCollectionProxy()->GetValueClass();
+         if (!cl) {
+            // Skip the artifical streamer element.
+            return;
+         }
+         // else return;
+
+         // In case the content is a class, move forward
+         AddForward(cl);
+         AddHeader(cl);
+        
+         if (level<=fMaxUnrolling) {
+
+            // See AnalyzeTree for similar code!
+            // TBranchProxyClassDescriptor *cldesc;
+            if (cl && cl->CanSplit()) {
+               // cldesc = new TBranchProxyClassDescriptor(cl->GetName(), cl->GetStreamerInfo(),
+               //                                          branch->GetName(),
+               //                                          isclones, 0 /* non-split object */,
+               //                                          containerName);
+               
+               TVirtualStreamerInfo *info = cl->GetStreamerInfo();
+               TStreamerElement *elem = 0;
+               
+               TString subpath = path;
+               if (subpath.Length()>0) subpath += ".";
+               subpath += dataMemberName;
+               
+               TIter next(info->GetElements());
+               while( (elem = (TStreamerElement*)next()) ) {
+                  AnalyzeElement(branch, elem, level+1, topdesc, subpath.Data());
+               }
+
+               // TBranchProxyClassDescriptor *added = AddClass(cldesc);
+               // if (added) type = added->GetName();
+            }
+         }
          return;
       }
 
